#------------------------------------------------- {COPYRIGHT-TOP-RM}
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#------------------------------------------------- {COPYRIGHT-END-RM}

# File: archive/mgmt/hmsqueries.ksh, tbsm_base, tbsm_base31e
# SID: 1.42.1.35
# Date Modified: 07/03/08 14:47:54
# Default variables

# Base registry path for profile subkeys
PROFILE_BASE_KEY="HKEY_LOCAL_MACHINE\\SOFTWARE\\Accessible Software, Inc.\\Access1\\1.0\\Health Monitor\\Profiles\\"

# Base path for shell scripts
INSTALL_DIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\\\\\\\\/g' | sed 's/\\\\\\\\$//g')
# Making sure there is a '\' on the end
INSTALL_DIR=$(echo $INSTALL_DIR | awk '/[^\]$/ { $0 = $0 "\\" ; print $0 ; next} // { print $0 }')
BASE_DIR="$INSTALL_DIR"Mgmt\\HMS\\ShellScripts\\

# The local host name, converted to all caps if not
#      932 - Japanese
#      936 - Simplified Chinese (PRC)
#      950 - Traditional Chinese (Taiwan)
#      949 - Korean
CODE_PAGE=$(SetConsoleCodePage.exe | awk '/Current Console Code Page/ {print $NF;}')
if [  $CODE_PAGE ==  932  -o  $CODE_PAGE ==  936  -o  $CODE_PAGE ==  950  -o  $CODE_PAGE ==  949  ]; then
  localhost=$(hostname)
else
localhost=$(hostname | tr '[a-z]' '[A-Z]')
fi

typeset -i nCycleInterval=60
typeset -i nNumCycles=0

export nNumCycles

# Root directory
ROOTDIR=//$localhost/HMS

# Required component host names
DBHost=       # Central database host
MonitorHost=  # Host running the component monitoring services
MVSHost=      # Host running MVS listeners and event handlers

# Database login parameters
DBUser=sa
DBPassword=

LastDBBlocked=1 # Keeps track of the last status of the GetDBBlocks function

test=0    # If 1 then test by outputting the configuration parameters and exiting
verbose=0 # If 1 then verbose messages are generated to standard output during run

# Prints output if verbose option is set, otherwise swallows it.
vprint()
{
  now=$(date +"%Y/%m/%d %H:%M:%S")
  print -R >${Input}LASTACTION.TXT $nNumCycles "| " $now "| " $*
  if [ $verbose = 0 ]; then return; fi
  print -u2 -R >&2 $nNumCycles "| " $now "| " $*
}

Usage()
{
  ASIShellMsg ASIHMonScripts 1 "HMSQueries"
}

# Defines a variable that must be set
typeset -i nRequiredVariables=0
DefineRequiredVariable()
{
  var=$1
  nRequiredVariables=$[$nRequiredVariables+1]
  REQUIRED_VARIABLES[$nRequiredVariables]=$var
}

# Array of variable names that must be specified in the configuration profile
DefineRequiredVariable ALHost
DefineRequiredVariable DBHost
DefineRequiredVariable DBLogDir
DefineRequiredVariable DBUser
DefineRequiredVariable DBPassword
DefineRequiredVariable DBQueryTimeout
DefineRequiredVariable DBLoginTimeout
DefineRequiredVariable Input
DefineRequiredVariable Output
DefineRequiredVariable MVSHost
DefineRequiredVariable MonitorHost
DefineRequiredVariable MonitoredServicesProfile
DefineRequiredVariable EIFHost

CheckRequiredVariable()
{
  if [ -z "$(eval echo \$$1)" ]; then
  # Some variables can be empty
    if [ $1 != "ALHost" ] && [ $1 != "EIFHost" ] && [ $1 != "DBUser" ] \
         && [ $1 != "DBPassword" ]; then
      ASIShellMsg ASIHMonScripts 2 "$1" >&2
      exit 1
    fi  
  fi
}

CheckRequiredVariables()
{
  typeset -i nVars=${#REQUIRED_VARIABLES[*]}
  typeset -i i=1
  nVars=$nVars+1
  while [ $i -lt $nVars ]; do
    varName=${REQUIRED_VARIABLES[$i]}
    CheckRequiredVariable $varName
    i=$i+1
  done
}

DumpVariables()
{
  for varName in ${REQUIRED_VARIABLES[*]}; do
  # Dump required variables except for the database userid and password
    if [ $varName != "DBUser" ] && [ $varName != "DBPassword" ]; then 
      echo "$varName=$(eval echo \$$varName)"
    fi  
  done

  ASIShellMsg ASIHMonScripts 3
  for host in ${RequiredHosts[*]}; do
    echo "    $host"
  done
}

LastSQLServerRestarts()
{
  ASIShellMsg ASIHMonScripts 4
  print "#-40s|-30s"
  for f in "//$DBHost/$DBLogDir"/errorlog*; 
  do 
  {
    head -1 $f | awk -v f=$f '{printf("%s|%s %s\n", f, $1, $2)}'; 
  }
  done;
}

DiskUsage()
{
	host=$1
	ASIGetDiskSpace -S$host | awk '{gsub("%", " %");print;}'
}

DiskUsageAll()
{
  for host in ${RequiredHosts[*]}; do
  	DiskUsage $host
  done
}

CalcDiskUsageAll()
{
  print "#Caption|"$(ASIShellMsg ASIHMonScripts 56)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 55)
  ASIShellMsg ASIHMonScripts 5
  print "#-15s |-6s |8s |8s |7s |-8s"
  DiskUsageAll | awk '{
    free_mb = $3 / 1024;
    size_mb = $7 / 1024;
    percent_used = 100 - ((free_mb * 100) / size_mb);

    if (percent_used >= 80 && percent_used < 90)
      color="YELLOW"
    else if (percent_used >= 90)
      color="RED"
    else
      color="GREEN"

    printf("%s|%s|%d|%d|%d|%s\n", $1, $2, free_mb, size_mb, percent_used, color);
  }' | $mkspath/sort -t'|' -r -k5 -n
}

PostProcessSQLResults()
{
  fmt=$1;
  awk -v fmt="$fmt" '
  NR==2 {
    print fmt;
    nFields=NF;
    for (i=1; i<=nFields; i++) {
      flen[i] = length($i);
    }
  }
  NF>0&&NR>2  {
    offset=2;
    for (i=1;i<nFields;i++) {
      field=substr($0, offset, flen[i]);
      sub("^[ ]*", "", field);
      sub("[ ]*$", "", field);
      offset += flen[i] + 1;
      printf("%s|", field);
    }
    field=substr($0, offset, flen[nFields]);
    sub("^[ ]*", "", field);
    sub("[ ]*$", "", field);
    printf("%s\n", field);
  }'
}

# CheckSqlErrors - checks for SQL error messages.  If found, the messages are output and colored
# YELLOW
#
CheckSqlErrors()
{
  awk -v sqlerr="$sqlerr" '
    function output_error() {
    # Print at least the same number of non-error column headings so the GUI does not print garbage 
      print "#Message|COLOR||||||||||||||||||||||"
      print "#-25s|-8s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s|1s"
      print $0"|YELLOW"
      while (getline > 0) {
        if ($0 ~ /^#/)
          continue
        printf("%s|YELLOW\n", $0);
      }  
      system("rm -f "sqlerr)  
    }
  
    $0 ~ /Error = \[Microsoft\]\[ODBC SQL Server Driver\]/ {output_error()}
    $0 ~ /^Msg[[:space:]]*[[:digit:]]*/ {output_error()}
    $0 ~ /Login failed for user/ {output_error()}
    $0 ~ /\[DBNETLIB\]SQL Server does not exist or access denied/ {output_error()}
    # Check if cannot access remote registry
    $0 ~ /Unable to access registry specifed/ {output_error()}
' $sqlerr    

}

GetDBLockSummary()
{
     print "#Caption|"$(ASIShellMsg ASIHMonScripts 44)
     print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 43)
     # We are trying to fix the null problem...
     ${OSQL} -Q"sp_locksummaryalldbs" | TR '\000' ' ' > "$sqlerr"
     CheckSqlErrors
     if [ -e "$sqlerr" ]; then
       print $(ASIShellMsg ASIHMonScripts 69)
       PostProcessSQLResults "#5s |-13s|4s |-23s|-11s|12s |-17s|-19s" < "$sqlerr" 
       rm -f "$sqlerr"
     fi
}

GetDBBlocks()
{
   print "#Caption|"$(ASIShellMsg ASIHMonScripts 42)
   print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 41)
   ${OSQL} -Q"set nocount on exec sp_showblocks" > "$sqlerr"
   CheckSqlErrors
   if [ -e "$sqlerr" ]; then
     print $(ASIShellMsg ASIHMonScripts 68)
     cat "$sqlerr" | PostProcessSQLResults "#-5s|-8s|-12s|-13s|-16s" | awk -v OutDir="$Input" -v LastDBBlocked=$LastDBBlocked '
      NR==1 { printf("%s|-8s\n", $0); }
      NR>1  { 
        if (NR==2)
          if (LastDBBlocked==1)
            LastDBBlocked = 2;
          else
            LastDBBlocked = 3;
        if (LastDBBlocked == 2) color = "YELLOW";
        else color = "RED";
        printf("%s|%s\n", $0, color);
      }
      END { 
        if (NR<2)
          LastDBBlocked = 1;
        statfile=sprintf("%sLastDBBlocked.txt", OutDir);
        print LastDBBlocked > statfile;
      }'
     LastDBBlocked=$(cat ${Input}LastDBBlocked.txt)
    
     rm -f "$sqlerr"
   fi
}

DumpDBQueues()
{
   print "#Caption|"$(ASIShellMsg ASIHMonScripts 48)
   print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 47)
  
   ${OSQL} -Q"set nocount on set CONCAT_NULL_YIELDS_NULL OFF exec asisp_dumpAllQueues '|'" > "$sqlerr"
   CheckSqlErrors
   if [ -e "$sqlerr" ]; then
     print $(ASIShellMsg ASIHMonScripts 70)
     awk 'NR>3&&NF>0 {sub("^[ ]",""); sub("[ ][ ]*$",""); print $0;}' "$sqlerr" 
     rm -f "$sqlerr"
   fi
}

GetLogShipStatus()
{

  # Initialize variables
  PrimaryServer=""
  SecondaryServer=""
  failover_enabled=0

  # Find the tbsm_server_role file on the DB Host
  if [ -f "//${DBHost}/Access1$/tbsm_server_role" ]; then

    failover_enabled=1

    PDBUser=""
    PDBPassword=""
    SDBUser=""
    SDBPassword=""

    # Get the server roles
    PrimaryServer=$(cat "//${DBHost}/Access1$/tbsm_server_role" | crypt -d FAILOVER | grep "HOST_DatabaseServer_Primary" | awk -F= '{print $2}' )
    SecondaryServer=$(cat "//${DBHost}/Access1$/tbsm_server_role" | crypt -d FAILOVER | grep "HOST_DatabaseServer_Secondary" | awk -F= '{print $2}' )

    # Get the DB access info
    eval $(cat "//${DBHost}/Access1$/tbsm_server_role" | crypt -d FAILOVER | grep "DB_ACCESS_${PrimaryServer}" | awk -F= '{print $2}' | awk '{printf("PDBUser=\"%s\";PDBPassword=\"%s\"", $1, $2); }')
    eval $(cat "//${DBHost}/Access1$/tbsm_server_role" | crypt -d FAILOVER | grep "DB_ACCESS_${SecondaryServer}" | awk -F= '{print $2}' | awk '{printf("SDBUser=\"%s\";SDBPassword=\"%s\"", $1, $2); }')

    # Find out if the databases are read-only
    # attempt to find the db access info from remote registry
    #result=$(regdmp -m \\\\$PrimaryServer \
    #  "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" |
    #  awk ' /User/ {printf("PDBUser=%s\n",$3)}
    #        /Password/ {printf("PDBPassword=%s\n",$3)}')
    #eval $result

    #result=$(regdmp -m \\\\$SecondaryServer \
    #  "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" |
    #  awk ' /User/ {printf("SDBUser=%s\n",$3)}
    #        /Password/ {printf("SDBPassword=%s\n",$3)}')
    #eval $result

    if [ -z "$PDBUser" -o -z "$PDBPassword" ]; then
      Psup=" -S$PrimaryServer -E "
    else
      Psup=" -S$PrimaryServer -U$PDBUser -P$PDBPassword "
    fi

    if [ -z "$SDBUser" -o -z "$SDBPassword" ]; then
      Ssup=" -S$SecondaryServer -E "
    else
      Ssup=" -S$SecondaryServer -U$SDBUser -P$SDBPassword "
    fi

    PrimaryIsRO=$(osql $Psup -dmsdb -Q"EXEC master..sp_dboption 'Object'" | grep -i "^[ ]read only" | sed "s/[ ]//g")
    SecondaryIsRO=$(osql $Ssup -dmsdb -Q"EXEC master..sp_dboption 'Object'" | grep -i "^[ ]read only" | sed "s/[ ]//g")

    # if the primary is read-only, then switch the roles
    if [ -z "$SecondaryIsRO" -a ! -z "$PrimaryIsRO" ]; then

      #-? TempServer="$PrimaryServer"
      #-? PrimaryServer="$SecondaryServer"
      #-? SecondaryServer="$TempServer"
      #-? TempUser="$PDBUser"
      #-? PDBUser="$SDBUser"
      #-? SDBUser="$TempUser"
      #-? TempPassword="$PDBPassword"
      #-? PDBPassword="$SDBPassword"
      #-? SDBPassword="$TempPassword"

      logship_sup=$Psup
    else
      logship_sup=$Ssup
    fi

  fi

  # Output data
  print "#Caption|"$(ASIShellMsg ASIHMonScripts 75)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 76)

  # If failover is not enabled, output the data to indicate this.
  if [ $failover_enabled -ne 1 ]; then
    ASIShellMsg ASIHMonScripts 77
    ASIShellMsg ASIHMonScripts 78
    ASIShellMsg ASIHMonScripts 79
  else
    ASIShellMsg ASIHMonScripts 80
    ASIShellMsg ASIHMonScripts 81

    osql $logship_sup -dmsdb -w10000 -Q"SET NOCOUNT ON SET CONCAT_NULL_YIELDS_NULL OFF EXEC log_ship_status_hms"
  fi

}

StagedEventStatus()
{
  print "#Caption|"$(ASIShellMsg ASIHMonScripts 60)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 59)
  
  ${ISQL} -Q"set nocount on exec asisp_stagedeventstatus @exec=1, @delim=N'|'"  > "$sqlerr"
  CheckSqlErrors
  if [ -e "$sqlerr" ]; then
    ASIShellMsg ASIHMonScripts 73
    awk '
      NR>3&&NF>0 {sub("^[ ]",""); sub("[ ][ ]*$",""); print $0;}' "$sqlerr"
    rm -f "$sqlerr"
  fi
}

StagedEvProcStatus()
{
  print "#Caption|"$(ASIShellMsg ASIHMonScripts 66)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 65)
  
  ${ISQL} -Q"set nocount on exec asisp_stagedeventprocstatus @delim=N'|'"  > "$sqlerr"
  CheckSqlErrors
  if [ -e "$sqlerr" ]; then
    ASIShellMsg ASIHMonScripts 74
    awk '
      NR>3&&NF>0 {sub("^[ ]",""); sub("[ ][ ]*$",""); print $0;}' "$sqlerr"
    rm -f "$sqlerr"
  fi
}

MonitorRequiredServices()
{
   print "#Caption|"$(ASIShellMsg ASIHMonScripts 54)
   print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 53)
  ASIShellMsg ASIHMonScripts 6
  print "#-34s|-11s |-21s |-11s|-8s"
  
  regdmp "$PROFILE_BASE_KEY$MonitoredServicesProfile" | awk '{
    while (1) {
      if ($0 ~ /^    [A-Za-z]/) { # start of service
        display_name=$0;
        setting["Host"]="";
        setting["Service"]="";
        setting["DesiredState"]="";
        getline;
        while ($0 ~ /^        [A-Za-z]/) {
          key=$1
          sub(".* = ", "")
          val=$0 
          setting[key] = val;
          getline;
        }
        printf("display_name=\"%s\";host=%s;service=\"%s\";desiredstate=%s\n", display_name, setting["Host"], setting["Service"], setting["DesiredState"]);
      }
      else
        if (getline != 1)
          break;
    }
  }' > MonitorRequiredServices.lst
  exec 4< MonitorRequiredServices.lst
  while read line <&4; do
    # reset variables
    display_name=
    host=
    service=
    regcurrentstate=
    currentstate=
    desiredstate=
    color=
    regcolor=

    # evaluate variables for next service
    eval $line;

    # If host is the special value "UNUSED" then skip
    if [ "$host" = "UNUSED" ]; then
      continue
    fi

    # if any of the required settings are missing, assign a current state of ???
    if [ -z "$host" ]; then
      regcurrentstate="UNKNOWN_HOST"
      regcolor="YELLOW"
    else 
       host=$(echo $host | tr '[a-z]' '[A-Z]')  
    fi
    if [ -z "$service" ]; then
      regcurrentstate="UNKNOWN_SERVICE"
      regcolor="YELLOW"
    fi
    if [ -z "$desiredstate" ]; then
      regcurrentstate="UNKNOWN_DESIREDSTATE"
      regcolor="YELLOW"
    fi

    # Determine service status for each host.  MVSHost and SenderServiceHost can have multiple hosts.
    for hostone in ${host[*]}; do
    
    
      # Determine the IBMHTTPServer service name
      echo $service | awk -v host=$hostone -v tempsc="tempsc" '
          /IBMHTTPServer/ {system("sclist  \\\\\\\\"host" > tempsc")
                           while ( getline <tempsc > 0 ) {
                             if ($0 ~ "IBMHTTPServer") {
                               printf("service=\"%s\"\n",$2)
                               sub($1,"")
                               sub($1,"")
                               printf("display_name=\"%s\"\n",$0)
                             }
                           }  
          }
          
          ' > newsc
               
      if [ -s "newsc" ]; then
        exec 5< newsc
        while read line <&5; do
          # evaluate variables for HTTP service
          eval $line;
        done
      fi  

      # Query state of service
      if [ -z "$regcurrentstate" ]; then
        currentstate=$(sc '\\'$hostone query "$service" | grep 'STATE.*:' | awk '{print $4}')
        if [ -z "$currentstate" ]; then
          currentstate="QUERY_FAILED"
        fi
      else   
        currentstate=$regcurrentstate
      fi

      # Compute color by comparing current and desired states
      if [ -z $regcolor ]; then
        if [ $currentstate = $desiredstate ]; then
          color="GREEN";
        elif [ $currentstate = START_PENDING -a $desiredstate = RUNNING ]; then
          color="YELLOW";
        elif [ $currentstate = STOP_PENDING -a $desiredstate = STOPPED ]; then
          color="YELLOW";
        else
          color="RED";
        fi
      else
        color=$regcolor  
      fi

      # Output service status
      print "$display_name|$hostone|$service|$currentstate|$color"
    done
  done
}

GetMVSStatus()
{
  print "#Caption|"$(ASIShellMsg ASIHMonScripts 50)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 49)
  if [ $verbose = 1 ];then
    vopt=-v
  else
    vopt=
  fi

  "$BASE_DIR"MVSStatus -H "$MVSHost" -K $MonitorHost $vopt
}

EventEnablementStatus()
{
  print "#Caption|"$(ASIShellMsg ASIHMonScripts 85)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 84)
  print $(ASIShellMsg ASIHMonScripts 83)"SColor"
  print "#-30s|7s |-20s|-8s"
  
  # Make sure we have Event Enablement Config directory
  EE_CONFIG_DIR="//${ALHost}/Access1$/AgentListener/Config/"
  if [ ! -d "$EE_CONFIG_DIR" ]; then
    return
  else
    EEConfigFile="$EE_CONFIG_DIR"GemEEConfig.properties
  fi
  
  # Retrieve Event Enablement Status from <hostname>.properties files
  # The hostnames are contained in GemEEConfig.properties file
  
  awk -F'=' -v EE_CONFIG_DIR=$EE_CONFIG_DIR '
  
    $0 !~ /\=/ { continue }
    
    { 
      # Retrieve hostnames
      hostIndex = $2
      hostName[++nHosts] = $2
      hostFile = EE_CONFIG_DIR$2".properties"
      hostStat[hostIndex] = 99
      hostEnable[hostIndex] = 0
      
      # Retrieve connection status for each host
      while ( getline <hostFile > 0 ) {
        if ( $0 ~ "connectionStatus" ) { hostStat[hostIndex] = $2 }
        if ( $0 ~ "enabled" ) { hostEnable[hostIndex] = $2 }
      }
      
    }
  
  END {
    for (i=1; i<=nHosts; i++) {
      host=hostName[i]
      hostText = "unknown"
      hostColor = "RED"
      if (hostStat[host] == 0 ) {
        hostText = "not connected"
        hostColor = "GREEN"
      }  
      if (hostStat[host] == 1 ) {
        hostText = "connection pending"
        hostColor = "YELLOW"
      }  
      if (hostStat[host] == 2 ) {
        hostText = "connected"
        hostColor = "GREEN"
      }  
      if (hostStat[host] == 5 ) {
        hostText = "inactive"
        hostColor = "RED"
      }  
      if (hostStat[host] == 6 ) {
        hostText = "no connections available"
        hostColor = "YELLOW"
      }
        
      printf("%s|%s|%s|%s\n", host, hostEnable[host], hostText, hostColor)
    }

  }

  ' < $EEConfigFile
}

EIFSenderStatus()
{
  print "#Caption|"$(ASIShellMsg ASIHMonScripts 86)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 87)
  
  # Get the various queues associated with the service
  ${OSQL} -Q"set nocount on SET CONCAT_NULL_YIELDS_NULL OFF exec asisp_eifHealthMonitor '|'"  > "$sqlerr"
  CheckSqlErrors
  if [ -e "$sqlerr" ]; then
    print $(ASIShellMsg ASIHMonScripts 88)
    print "#-35s|35s|20s|-8s"
    awk '
      NR>2&&NF>0 {
        sub("^[ ]",""); 
        sub("[ ][ ]*$",""); 
        sub("^[-]","");
        sub("[-][-]*$",""); 
        print $0;
    }' "$sqlerr"
    rm -f "$sqlerr"
  else
    return  
  fi
  
  # Make sure we have EIFSender Config directory
  EIF_CONFIG_DIR="//${EIFHost}/Access1$/EIFSender"
  if [ ! -d "$EIF_CONFIG_DIR" ]; then
    return
  fi
  
  eiftitle="0"
  
  valw=$(${OSQL} -Q"USE Object;DECLARE @var NVARCHAR(100);SELECT @var=propertyvalue FROM SystemConfiguration WHERE groupname='EIF_SENDER_SVC' AND propertyname = 'EIF_QUEUE_WARNING';PRINT @var")
  vals=$(${OSQL} -Q"USE Object;DECLARE @var NVARCHAR(100);SELECT @var=propertyvalue FROM SystemConfiguration WHERE groupname='EIF_SENDER_SVC' AND propertyname = 'EIF_QUEUE_SEVERE';PRINT @var")
  
  #set +f
  # Get the queues associated with the EIF adapters
  for THEFILE in `ls "$EIF_CONFIG_DIR"/*.conf` ; do
    # Write the title first time through
    if [ "$eiftitle" = "0" ] ; then
      echo " | | | "
      echo "$(ASIShellMsg ASIHMonScripts 90)| | |"
      eiftitle="1"
    fi
    
    confFile=`cat $THEFILE | grep BufEvtPath | awk -F"=" '{ print $2}'`
    if [ -n "$confFile" ] ; then
      server=`basename $THEFILE | sed -e 's/.conf$//g'`
      # Check for the EIF queue, if not existent then no data queued
      if [ -f "$confFile" ] ; then
        ct=`wc -l "$confFile" | awk -F" " '{print $1}'`
        xx=$((ct-3))
      
        if [ "$xx" -lt 0 ] ; then
          xx="0"
        fi
        
        if   [ "$xx" -lt "$valw" ] ; then
          echo "$server|n/a|$xx|GREEN"
        elif [ "$xx" -lt "$vals" ] ; then
          echo "$server|n/a|$xx|YELLOW"
        else 
          echo "$server|n/a|$xx|RED"
        fi  
      else
        echo "$server|n/a|0|GREEN"   
      fi
    fi  
  done
  #set -f
}

CommonListenerStatus()
{

  print "#Caption|"$(ASIShellMsg ASIHMonScripts 100)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 101)
  
  ${OSQL} -Q"set nocount on SET CONCAT_NULL_YIELDS_NULL OFF exec asisp_clHealthMonitor '|'"  > "$sqlerr"
  CheckSqlErrors
  if [ -e "$sqlerr" ]; then
    print $(ASIShellMsg ASIHMonScripts 102)
    print "#-35s|-15s|-10s|-30s|-15s|-15s|-10s|-30s"
    awk '
      NR>2&&NF>0 {
        sub("^[ ]",""); 
        sub("[ ][ ]*$",""); 
        sub("^[-]","");
        sub("[-][-]*$",""); 
        print $0;
    }' "$sqlerr"
    rm -f "$sqlerr"
  fi
}

AgentListenerStatus()
{

  print "#Caption|"$(ASIShellMsg ASIHMonScripts 109)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 110)

  ${OSQL} -Q"set nocount on SET CONCAT_NULL_YIELDS_NULL OFF exec asisp_alHealthMonitor '|'" > "$sqlerr"
  CheckSqlErrors
  if [ -e "$sqlerr" ]; then
    print $(ASIShellMsg ASIHMonScripts 111)
    print "#-15s|-40s|-22s|-22s|-10s|-30s"
    awk '
      NR>2&&NF>0 {
        sub("^[ ]",""); 
        sub("[ ][ ]*$",""); 
        sub("^[-]","");
        sub("[-][-]*$",""); 
        print $0;
    }' "$sqlerr"
    rm -f "$sqlerr"
  fi  
}

# added as part of defect 150015 for Discovery Library Phase 2 to monitor the 
# health of the DL book readers and writers.
DiscoveryLibraryStatus()
{

  print "#Caption|"$(ASIShellMsg ASIHMonScripts 113)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 114)

  ${OSQL} -Q"set nocount on SET CONCAT_NULL_YIELDS_NULL OFF exec asisp_clHealthMonitorDL '|'" > "$sqlerr"
  CheckSqlErrors
  if [ -e "$sqlerr" ]; then
    print $(ASIShellMsg ASIHMonScripts 115)
    print "#-35s|-15s|-10s|-30s|-22s|-22s|-10s|-30s"
    awk '
      NR>2&&NF>0 {
        sub("^[ ]",""); 
        sub("[ ][ ]*$",""); 
        sub("^[-]","");
        sub("[-][-]*$",""); 
        print $0;
    }' "$sqlerr"
    rm -f "$sqlerr"
  fi  
}

SqlJobs()
{

  # Use comma as a delimiter instead of | to avoid DBCS problems
  print "#Caption|"$(ASIShellMsg ASIHMonScripts 105)
  print "#Abbreviation|"$(ASIShellMsg ASIHMonScripts 106)
  
  errout="//${DBHost}/Access1$/errtemp.txt"
  
  rm -f "$errout"
 
  # Retrieve enabled job information 
  ${ISQL} -Q"exec asisp_collectJobHistory @delim=N','" > "$sqlerr"
  CheckSqlErrors
  if [ -e "$sqlerr" ]; then
    print $(ASIShellMsg ASIHMonScripts 107)"|RCOLOR|"$(ASIShellMsg ASIHMonScripts 108)
    print "#-40s|-10s|-6s|-11s|-8s|7s|-20s|-5s|-9s"
    cp "$sqlerr" "$errout"
    rm -f "$sqlerr"
  else
    return  
  fi
  
  # Remove extraneous spaces
  # Format the date and time to be consistent with other health monior views
  awk -F',' '
    NR>2&&NF>1 {
      sub("^[ ]","");
      sub("[ ][ ]*$","");
      split($0, formarray, ",");

Imaginary Buffer Line
      if (length(formarray[4]) == 8) {
        formatteddate=substr($4, 1, 4)"-"substr($4, 5, 2)"-"substr($4, 7, 2)
        #sub($4,formatteddate)
        $4=formatteddate
      }
    
      if (formarray[5] > 0) {
        lentime=length(formarray[5])
        numz=6 - lentime
        fulltime=formarray[5]
        for (i=0; i<numz; i++)
          fulltime="0"fulltime
        formattedtime=substr(fulltime, 1, 2)":"substr(fulltime, 3, 2)":"substr(fulltime, 5, 2)  
        #sub($5, formattedtime)
        $5=formattedtime
      }
      
      print $1"|"$2"|"$3"|"$4"|"$5"|"$6"|"$7"|"$8"|"$9;
    }
' "$errout"

  rm -f "$errout"
}

SectionBreak()
{
	title=$1
	echo
	echo
	echo "**************************************************************************"
	echo "*" $title
	echo "**************************************************************************"
}

# Use comma as a delimiter instead of '|' to avoid DBCS problems
typeset -i nSections=0
DefineSection()
{
  function=$1
  message=$2

  nSections=$[$nSections+1]
  Sections[$nSections]="$function,$message"
}

ExecSection()
{
  typeset -i sectionNo=$1
  SectionDef=${Sections[$sectionNo]}
  function=${SectionDef%,*}
  message=${SectionDef#*,}

  vprint "$message"
  
  $function > "$Input"$function.txt
}

SummarizeSection()
{
  typeset -i sectionNo=$1
  SectionDef=${Sections[$sectionNo]}
  function=${SectionDef%,*}
  message=${SectionDef#*,}

  SectionBreak "$message"
  "$BASE_DIR"pprint "$Input"$function.txt
}

SummarizeAll()
{
  typeset TimeStamp="$1"
	SectionBreak "Begin Time Stamp"
  echo "$TimeStamp"

  # Summarize all of the findings
  typeset -i i=1
  while [ $i -le $nSections ]; do
    SummarizeSection $i
    i=$[$i+1]
  done

	SectionBreak "End Time Stamp"
  date +"%Y/%m/%d %H:%M:%S"
}

DefineSection DumpVariables           "$(ASIShellMsg ASIHMonScripts 10)"
DefineSection GetDBBlocks             "$(ASIShellMsg ASIHMonScripts 12)"
DefineSection GetDBLockSummary        "$(ASIShellMsg ASIHMonScripts 13)"
DefineSection CalcDiskUsageAll        "$(ASIShellMsg ASIHMonScripts 19)"
DefineSection GetLogShipStatus        "$(ASIShellMsg ASIHMonScripts 75)"
DefineSection StagedEventStatus       "$(ASIShellMsg ASIHMonScripts 20)"
DefineSection StagedEvProcStatus      "$(ASIShellMsg ASIHMonScripts 67)"
DefineSection DumpDBQueues            "$(ASIShellMsg ASIHMonScripts 21)"
DefineSection MonitorRequiredServices "$(ASIShellMsg ASIHMonScripts 22)"
DefineSection GetMVSStatus            "$(ASIShellMsg ASIHMonScripts 23)"
DefineSection LastSQLServerRestarts   "$(ASIShellMsg ASIHMonScripts 26)"
DefineSection EventEnablementStatus   "$(ASIShellMsg ASIHMonScripts 82)"
DefineSection EIFSenderStatus         "$(ASIShellMsg ASIHMonScripts 89)"
DefineSection CommonListenerStatus    "$(ASIShellMsg ASIHMonScripts 103)"
DefineSection SqlJobs                 "$(ASIShellMsg ASIHMonScripts 104)"
DefineSection AgentListenerStatus     "$(ASIShellMsg ASIHMonScripts 112)"
DefineSection DiscoveryLibraryStatus  "$(ASIShellMsg ASIHMonScripts 116)"

cycle()
{
  typeset StartTime;
  StartTime=$(date +"%Y/%d/%m %H:%M:%S")

  echo $StartTime > "$Input"HMSQueries.inprogress

  # Execute each defined section
  typeset -i i=1
  while [ $i -le $nSections ]; do
    ExecSection $i
    i=$[$i+1]
  done

  vprint "$(ASIShellMsg ASIHMonScripts 27 "${Input}")"
  SummarizeAll "$StartTime" > "$Input"Total_Summary.txt

  # Copying output file to contain timestamp
  vprint $(cp -v "$Input"Total_Summary.txt "$Output"Total_Summary`date +""%Y""%m""%d"_"%H""%M""%S""`".TXT")

  rm -f "$Input"HMSQueries.inprogress
}

# Usage: ConfigFromRegKey <key>
# Reads configuration assignments from the specified registry key. The full registry path used
# is $PROFILE_BASE_KEY\<key>.
ConfigFromRegKey()
{
  settingsFile="${Input}settings.$$.txt"
  profile_key=$PROFILE_BASE_KEY${*}
  vprint ConfigFromRegKey \"$profile_key\"
  rm -f ConfigFromRegKey.err
   
  # The registry keys and values extracted are dependent on the output format of 
  # regdmp CLI.  A temporary file is created with the registry value entries 
  # which will be evaluated as assignment statements.


  regdmp "$profile_key" | awk 'function parse_setting() {
    if ($0 ~ "REG_DWORD") { # Convert hex value to decimal using dc
      sub("REG_DWORD 0x", "");
      val=toupper($3);
      cmd="echo 16i " val " Aop|dc";
      cmd | getline decimal_val;
      $3=decimal_val;
      close(cmd);
    }
    if (prefix != "") {
      repl=prefix "&";
      sub("[A-z]", repl, $1);
    }
    if (NF == 2)
      sub(" =", "=\"\"");
    if ($0 ~ "Length not multiple") { # Convert bianry 00 to blank
      sub("\(\*\*\*.*$", "") 
    }
    env[$1] = $3;
    print $0;

    if (inhosts) {
      HostList=$0;
      sub("^.*=", "", HostList);
      maxind=split(toupper(HostList), Host);
      for ( i = 1; i <= maxind; i++ ) {
        # Add to unique list of hosts
        if ((Host[i] != "") && (HaveHost[Host[i]]==0)) {
          RequiredHosts[++nHosts]=toupper(Host[i]);
          HaveHost[Host[i]]=1;
        }
      }  
    }
  } 
  BEGIN {
    nHosts=0
  }
  {
    #line below is necessary to remove trailing garbage appended to key names with security settings specified
    gsub(" \[[0-9 ]*\]", "", $0)

    if ($0 == "    DB") {
      prefix=$1;
      inhosts=0;
    }
    else if ($0 == "    Hosts") {
      prefix="";
      inhosts=1;
    }
    else if ($0 ~ "^    [A-z]") {
      prefix="";
      inhosts=0;
    }
    if ($0 !~ /=/)
      continue;

    parse_setting();
  }
  END {
    for (i=1; i<=nHosts; i++) {
      RHost=RequiredHosts[i];
      printf("RequiredHosts[%d] = %s\n", i, RHost);
    }
  }' | sed 's/ = \(.*\)/='\''\1'\''/g' > $settingsFile
  
  if [ -f ConfigFromRegKey.err ]; then
    vprint $(cat ConfigFromRegKey.err)
    exit 1
  fi
  exec 4<$settingsFile
  while read setting<&4; do
    eval "$setting"
  done
  rm -f "$settingsFile"
  vprint "ConfigFromRegKey finished"
}

#
# main
#

# Process command line
while getopts hi:o:k:tv c; do
  case $c in
  i)  Input=$OPTARG;;
  o)  Output=$OPTARG;;
  k)  ConfigFromRegKey "$OPTARG";;
  t)  test=1;;
  v)  verbose=1;;
  h|?)  Usage; exit 0;;
  *)  Usage >&2; exit 1;;
  esac
done

# Make sure we have a good input directory
if [ -z "$Input" ]; then
  if [ ! -d "$ROOTDIR" ]; then
    ASIShellMSG ASIHMonScripts 7 "$ROOTDIR" >&2
    exit 1
  fi
  Input="$ROOTDIR/Input/"
fi
if [ ! -d "$Input" ]; then
  mkdir "$Input"
  if [ $? != 0 ]; then
    exit 1
  fi
fi
# Make sure $Input ends with a '/'
echo "$Input" | grep -q '/$'
if [ $? != 0 ]; then
  Input="$Input/"
fi

# Make sure we have a good output directory
if [ -z "$Output" ]; then
  if [ ! -d "$ROOTDIR" ]; then
    ASIShellMSG ASIHMonScripts 7 "$ROOTDIR" >&2
    exit 1
  fi
  Output="$ROOTDIR/Output/"
fi
if [ ! -d "$Output" ]; then
  mkdir "$Output"
  if [ $? != 0 ]; then
    exit 1
  fi
fi
# Make sure $Output ends with a '/'
echo "$Output" | grep -q '/$'
if [ $? != 0 ]; then
  Output="$Output/"
fi

# Set trap to cleanup process-specific files on exit
trap "rm -f HMSQueries.pid tempsc newsc MonitorRequiredServices.lst; exit" EXIT

echo $$ > HMSQueries.pid

# If in test mode then dump variables and exit
if [ $test = 1 ]; then
  DumpVariables
  CheckRequiredVariables
  exit
fi

# Make sure required variables are set
CheckRequiredVariables

# Set up standard command line for isql
#export ISQL="isql -S$DBHost -U$DBUser -P$DBPassword -t$DBQueryTimeout -l$DBLoginTimeout -n -w3000"
# If DB user is empty, use trusted connection
if [ -z "$DBUser" ]; then
  DBaccess="-E"
else
  DBaccess="-U$DBUser -P$DBPassword"
fi    
export ISQL="tbsmsql -X -S$DBHost $DBaccess -t$DBQueryTimeout -l$DBLoginTimeout -n -w3000"
export OSQL="osql -S$DBHost $DBaccess -t$DBQueryTimeout -l$DBLoginTimeout -n -w3000"
# Temp file to check SQL errors
sqlerr=sqlerr.txt

# prepend MKStoolkit path to sort.exe
mkspath=$(which which.exe | awk 'sub("mksnt/which.exe","") ')
mkspath="$mkspath"mksnt

# Loop forever
while true; do 
  cycle;
  vprint "$(ASIShellMsg ASIHMonScripts 28 "$nCycleInterval")"
  nNumCycles=$nNumCycles+1
  sleep $nCycleInterval;
done
