#!/bin/sh
#**************************************************************** {COPYRIGHT-TOP-RM} ***
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#**************************************************************** {COPYRIGHT-END-RM} ***
#
# Generic syntax help.
#
GEM_ROOT="/TDS/EventService"
PATH=$PATH:/usr/ucb
SCRIPTNAME=`basename $0 2> /dev/null`
if [ "$SCRIPTNAME" = "" ]
then
 SCRIPTNAME=unix.setup
fi

date()
 {
   cd .
 }

syntax()
 {
   echo
   echo $SCRIPTNAME
   echo
   echo "  syntax:   $SCRIPTNAME [ -? ]"
   echo
   echo "  usage:    you must be root to run this script"
   echo
   echo
 }


for i in $@
do
  if [ "$i" = "-?" ]
   then
     syntax
     exit 0
   else
     echo
     date
     echo .... ERROR - invalid flag \"$i\" entered.
     syntax
     exit 1
  fi
done

date
echo "Event Enablement setup starting"

#
# UNIX unique processing
#
if [ "`uname`" = "AIX" ] || [ "`uname`" = "SunOS" ] || [ "`uname`" = "HP-UX" ] || [ "`uname`" = "Linux" ] ; then
  if [ "`whoami`" != "root" ] ; then
   echo
   date
   echo ".... You must be the super-user (root) to run this script"
   syntax
   exit 1
  fi

  date
  echo ".... Setting Tivoli environment variables."

  if [ -n "$TSERVER_ETC" ] ; then
    if [ -f "$TSERVER_ETC/setup_env.sh" ] ; then
      . "$TSERVER_ETC/setup_env.sh"
    else
      date
      echo ".... Unable to find file $TSERVER_ETC/setup_env.sh"
      date
      echo ".... Tivoli enviroment variables can not be set."
      exit 1
    fi
  elif [ -f "/etc/Tivoli/setup_env.sh" ] ; then
   . "/etc/Tivoli/setup_env.sh"
  else
   date
   echo ".... Unable to find file /etc/Tivoli/setup_env.sh"
   date
   echo ".... Tivoli enviroment variables can not be set."
   exit 1
  fi

  #
  # Detemine the OS and set OS-specific paths, falling back to defaults
  # for AIX.
  #

  arch=`uname -s 2> /dev/null`
  if [ "$arch" = "" ] ; then
   echo ".... Unable to determine your OS."
   exit 1
  fi

  archrev=`uname -r | cut -c1`
  if [ "$arch" = "AIX" ] ; then
   NLSBASE1=/usr/lib/nls/msg
   NLSBASE2=/usr/dt/lib/nls/msg
   RESBASE1=/usr/lib/X11/app-defaults
   RESBASE2=/usr/lib/X11/
   RESBASE3=/usr/dt/app-defaults/
   MK_GROUP=/usr/bin/mkgroup
  elif [ "$arch" = "SunOS" -a "$archrev" = "5" ]; then
   NLSBASE1=/usr/lib/nls/msg
   NLSBASE2=/usr/dt/lib/nls/msg
   RESBASE1="0"
   RESBASE2="0"
   RESBASE3=/usr/dt/app-defaults/
   MK_GROUP=/usr/sbin/groupadd
  elif [ "$arch" = "HP-UX" ]; then
   NLSBASE1=/usr/lib/nls/msg
   NLSBASE2=/usr/dt/lib/nls/msg
   RESBASE1="0"
   RESBASE2="0"
   RESBASE3=/usr/dt/app-defaults/
   MK_GROUP=/usr/sbin/groupadd
  elif [ "$arch" = "Linux" ]; then
   NLSBASE1=/usr/share/locale
   NLSBASE2=/etc/locale
   RESBASE1=/usr/lib/X11/app-defaults
   RESBASE2=/usr/lib/X11/
   RESBASE3=/usr/dt/app-defaults/
   MK_GROUP=/usr/sbin/groupadd
  else
   NLSBASE1=/usr/lib/nls/msg
   NLSBASE2=/usr/dt/lib/nls/msg
   RESBASE1="0"
   RESBASE2="0"
   RESBASE3=/usr/dt/app-defaults/
   MK_GROUP=/usr/sbin/mkgroup
  fi
  export CATFILE NLSBASE1 NLSBASE2 RESBASE1 RESBASE2 RESBASE3 MK_GROUP

  date
  echo .... Creating $arch message catalog links
  if [ "$arch" = "AIX" ] ; then

   for THELOOP in "4" "5" ; do
     if [ "$THELOOP" = "4" ] ; then
       THELANG=En_US
       NEWLANG=En_US.IBM-850
     elif [ "$THELOOP" = "5" ] ; then
       THELANG=en_US
       NEWLANG=en_US.ISO8859-1
     else
       THELANG=ja_JP
       NEWLANG=ja_JP.IBM-eucJP
     fi

     for LINKDIR in "$BINDIR$GEM_ROOT/msg_cat" ; do
       if [ -d "$LINKDIR/$THELANG" ] ; then
         if [ ! -d "$LINKDIR/$NEWLANG" ] ; then
           mkdir "$LINKDIR/$NEWLANG"
         fi

         cd "$LINKDIR/$THELANG"

         for CATFILE in `ls` ; do
           if [ -f "$LINKDIR/$NEWLANG/$CATFILE" ] || [ -h "$LINKDIR/$NEWLANG/$CATFILE" ] ; then
             rm "$LINKDIR/$NEWLANG/$CATFILE"
           fi

           echo .... Creating symbolic links for message catalogs $LINKDIR/$NEWLANG/$CATFILE
           ln -s "$LINKDIR/$THELANG/$CATFILE" "$LINKDIR/$NEWLANG/$CATFILE"
         done
       fi
     done
   done

  elif [ "$arch" = "HP-UX"  ] ; then
    for THELOOP in "1" "2" "3" "4" ; do
      if    [ "$THELOOP" = "1" ] ; then
        THELANG=C
        NEWLANG=en_US.850
      elif  [ "$THELOOP" = "2" ] ; then
        THELANG=C
        NEWLANG=en_US.88591
      elif  [ "$THELOOP" = "3" ] ; then
        THELANG=C
        NEWLANG=en_US.ISO8859-1
      elif  [ "$THELOOP" = "4" ] ; then
        THELANG=C
        NEWLANG=en_US.iso88591
      else
        THELANG=C
        NEWLANG=en_US.88591
      fi

      for LINKDIR in "$BINDIR$GEM_ROOT/msg_cat" ; do
        if [ -d "$LINKDIR/$THELANG" ] ; then
          if [ ! -d "$LINKDIR/$NEWLANG" ] ; then
            mkdir "$LINKDIR/$NEWLANG"
          fi

          cd "$LINKDIR/$THELANG"

          for CATFILE in `ls` ; do
            if [ -f "$LINKDIR/$NEWLANG/$CATFILE" ] || [ -h "$LINKDIR/$NEWLANG/$CATFILE" ] ; then
              rm "$LINKDIR/$NEWLANG/$CATFILE"
            fi

            echo .... Creating symbolic links for message catalogs $LINKDIR/$NEWLANG/$CATFILE
            ln -s "$LINKDIR/$THELANG/$CATFILE" "$LINKDIR/$NEWLANG/$CATFILE"
          done
        fi
      done
    done

  elif [ "$arch" = "Linux"  ] ; then
    for THELOOP in "1" "2" "3" "4" ; do
      if    [ "$THELOOP" = "1" ] ; then
        THELANG=C
        NEWLANG=en_US.850
      elif  [ "$THELOOP" = "2" ] ; then
        THELANG=C
        NEWLANG=en_US.88591
      elif  [ "$THELOOP" = "3" ] ; then
        THELANG=C
        NEWLANG=en_US.ISO8859-1
      elif  [ "$THELOOP" = "4" ] ; then
        THELANG=C
        NEWLANG=en_US.iso88591
      else
        THELANG=C
        NEWLANG=en_US.88591
      fi

      for LINKDIR in "$BINDIR$GEM_ROOT/msg_cat" ; do
        if [ -d "$LINKDIR/$THELANG" ] ; then
          if [ ! -d "$LINKDIR/$NEWLANG" ] ; then
            mkdir "$LINKDIR/$NEWLANG"
          fi

          cd "$LINKDIR/$THELANG"

          for CATFILE in `ls` ; do
            if [ -f "$LINKDIR/$NEWLANG/$CATFILE" ] || [ -h "$LINKDIR/$NEWLANG/$CATFILE" ] ; then
              rm "$LINKDIR/$NEWLANG/$CATFILE"
            fi

            echo .... Creating symbolic links for message catalogs $LINKDIR/$NEWLANG/$CATFILE
            ln -s "$LINKDIR/$THELANG/$CATFILE" "$LINKDIR/$NEWLANG/$CATFILE"
          done
        fi
      done
    done
  else
    echo "WARNING:  Architecture $arch is not supported.  Message catalogs will not be linked"
  fi

  #
  # Symlink all of the .cat files to the specified locales.
  #
  date
  echo ".... Creating system message catalog links."

  cd "$BINDIR$GEM_ROOT/msg_cat"

  for TLANG in `ls` ; do
    for NLSBASE in $NLSBASE1 $NLSBASE2 ; do
      if [ -d "$NLSBASE" ] ; then
        if [ -d "$NLSBASE/$TLANG" ] ; then
          cd "$BINDIR$GEM_ROOT/msg_cat/$TLANG"
          for CATFILE in `ls` ; do
            if [ -f "$NLSBASE/$TLANG/$CATFILE" ] || [ -h "$NLSBASE/$TLANG/$CATFILE" ] ; then
              rm "$NLSBASE/$TLANG/$CATFILE"
            fi

            echo .... Creating symbolic links for message catalogs $NLSBASE/$TLANG/$CATFILE
            ln -s "$BINDIR$GEM_ROOT/msg_cat/$TLANG/$CATFILE" "$NLSBASE/$TLANG/$CATFILE"
          done
        else
          echo .... The system $NLSBASE/$TLANG message catalogs are not installed on this system.
          echo .... Message catalog links for $TLANG will not be set.
          echo .... If in the future the system $TLANG catalogs are installed, then
          echo .... $SCRIPTNAME should be re-run to setup this link.
        fi
      fi
    done
  done

  #
  # Make symlinks from Tivoli msg_cat to GEM's messages.
  #
  date
  echo ".... Link message catalogs from Tivoli msg_cat directory."

  cd "$BINDIR"
  cd ../..

  if [ -d "msg_cat" ] ; then
    cd "msg_cat"

    TIV_MSGCAT=`pwd`

    for TLANG in `ls` ; do
      if [ -d "$BINDIR$GEM_ROOT/msg_cat/$TLANG" ] ; then

        cd "$BINDIR$GEM_ROOT/msg_cat/$TLANG"

        for CATFILE in `ls` ; do
          #
          # Remove the symlink if it exists.
          #
          if [ -h "$TIV_MSGCAT/$TLANG/$CATFILE" ] ; then
            echo ".... Removing $TIV_MSGCAT/$TLANG/$CATFILE from Tivoli message catalog."
            rm "$TIV_MSGCAT/$TLANG/$CATFILE"
          fi

          echo .... Creating symbolic links for message catalogs $TIV_MSGCAT/$TLANG/$CATFILE
          ln -f -s "$BINDIR$GEM_ROOT/msg_cat/$TLANG/$CATFILE" "$TIV_MSGCAT/$TLANG/$CATFILE"
        done
      else
        if [ "$TLANG" != ".installed" ] ; then
          echo ".... Links from Tivoli $TIV_MSGCAT/$TLANG to msg_cat were not built."
          echo ".... The $BINDIR$GEM_ROOT/msg_cat/$TLANG messages are not installed."
          echo ".... This is not an error."
        fi
      fi
    done
  fi

  #
  # Check for and symbolically link libraries.
  #
  date
  echo ".... Checking for libraries linked to /usr/lib"

  if [ "`uname`" = "SunOS" ] ; then
    for ALIBFILE in "$BINDIR$GEM_ROOT/lib/libihscl.so.1" "$BINDIR$GEM_ROOT/lib/libihsiexit.so.1" "$BINDIR$GEM_ROOT/lib/libwapm.a" "$BINDIR$GEM_ROOT/lib/libihsutds.a" ; do
      LIBFILE=`basename "$ALIBFILE"`
    
      if [ -f "$ALIBFILE" ] && [ "$LIBFILE" != "librasgui.so.1" ] && [ "$LIBFILE" != "librasgui.so" ] ; then
        if [ -h "/usr/lib/$LIBFILE" ] ; then
          echo ".... Checking link for $LIBFILE"
          echo `ls -al "/usr/lib/$LIBFILE"` | grep GEM > /dev/null
          if [ $? = 0 ] ; then
            echo ".... Removing link for /usr/lib/$LIBFILE"
            rm "/usr/lib/$LIBFILE"
          else
            echo `ls -al "/usr/lib/$LIBFILE"` | grep TDS > /dev/null
            if [ $? = 0 ] ; then
              echo ".... Removing link for /usr/lib/$LIBFILE"
              rm "/usr/lib/$LIBFILE"
            fi
          fi
        fi
    
        if [ ! -f "/usr/lib/$LIBFILE" ] ; then
          if [ ! -h "/usr/lib/$LIBFILE" ] ; then
            echo ".... Creating link for $LIBFILE"
            ln -s "$BINDIR$GEM_ROOT/lib/$LIBFILE" "/usr/lib/$LIBFILE"
          fi
        fi
    
      fi
    done
  else
    for LIBFILE in `ls "$BINDIR$GEM_ROOT/lib"` ; do
      ALIBFILE="$BINDIR$GEM_ROOT/lib/$LIBFILE"
    
      if [ -f "$ALIBFILE" ] && [ "$LIBFILE" != "librasgui.so.1" ] && [ "$LIBFILE" != "librasgui.so" ] && [ "$LIBFILE" != "libC.a" ] ; then
        if [ -h "/usr/lib/$LIBFILE" ] ; then
          echo ".... Checking link for $LIBFILE"
          echo `ls -al "/usr/lib/$LIBFILE"` | grep GEM > /dev/null
          if [ $? = 0 ] ; then
            echo ".... Removing link for /usr/lib/$LIBFILE"
            rm "/usr/lib/$LIBFILE"
          else
            echo `ls -al "/usr/lib/$LIBFILE"` | grep TDS > /dev/null
            if [ $? = 0 ] ; then
              echo ".... Removing link for /usr/lib/$LIBFILE"
              rm "/usr/lib/$LIBFILE"
            fi
          fi
        fi
    
        if [ ! -f "/usr/lib/$LIBFILE" ] ; then
          if [ ! -h "/usr/lib/$LIBFILE" ] ; then
            echo ".... Creating link for $LIBFILE"
            ln -s "$BINDIR$GEM_ROOT/lib/$LIBFILE" "/usr/lib/$LIBFILE"
          fi
        fi
    
      fi
    done
  fi

  #
  # Generate symbolic links to versioned shared libraries for Solaris and
  # other ELF systems.
  #
  if [ "`uname`" = "SunOS" ] ; then
    for LIBFILE in "$BINDIR$GEM_ROOT/lib/lib*.so.1"
    do
      LINK=`echo "$LIBFILE" | sed 's/\.so\.1$/.so/'`
      BLINK=`basename "$LINK"`
      echo ".... Checking link for `basename $LIBFILE`"
      if [ ! -z "$LINK" -a ! -h "$LINK" ] ; then
        echo ".... Setting link for `basename $LIBFILE`"
        ln -s -f `basename "$LIBFILE"` "$LINK"
      fi
    
      echo ".... Checking link for /usr/lib/$BLINK"
      if [ ! -z "/usr/lib/$BLINK" -a ! -h "/usr/lib/$BLINK" -a "$BLINK" != "librasgui.so" ] ; then
        echo ".... Setting link for /usr/lib/$BLINK"
        ln -s -f "$LIBFILE" "/usr/lib/$BLINK"
      fi
    done
  fi

  #
  # Link the shared libraries to Tivoli lib for the TEC exits
  #
  if [ -n "$LIBDIR" ] ; then
    echo ".... Checking for libraries linked to $LIBDIR"

    if [ "`uname`" = "SunOS" ] ; then
      for ALIBFILE in "$BINDIR$GEM_ROOT/lib/libihscl.so.1" "$BINDIR$GEM_ROOT/lib/libihsiexit.so.1" "$BINDIR$GEM_ROOT/lib/libwapm.a" "$BINDIR$GEM_ROOT/lib/libihsutds.a" ; do
        LIBFILE=`basename "$ALIBFILE"`
      
        if [ -f "$ALIBFILE" ] && [ "$LIBFILE" != "librasgui.so.1" ] && [ "$LIBFILE" != "librasgui.so" ] ; then
          if [ -h "$LIBDIR/$LIBFILE" ] ; then
            echo ".... Checking link for $LIBDIR/$LIBFILE"
            echo `ls -al "$LIBDIR/$LIBFILE"` | grep GEM > /dev/null
            if [ $? = 0 ] ; then
              echo ".... Removing link for $LIBDIR/$LIBFILE"
              rm "$LIBDIR/$LIBFILE"
            else
              echo `ls -al "$LIBDIR/$LIBFILE"` | grep TDS > /dev/null
              if [ $? = 0 ] ; then
                echo ".... Removing link for $LIBDIR/$LIBFILE"
                rm "$LIBDIR/$LIBFILE"
              fi
            fi
          fi
      
          if [ ! -f "$LIBDIR/$LIBFILE" ] ; then
            if [ ! -h "$LIBDIR/$LIBFILE" ] ; then
              if [ -f "/usr/lib/$LIBFILE" ] || [ -h "/usr/lib/$LIBFILE" ] ; then
                echo ".... Creating link for $LIBDIR/$LIBFILE from /usr/lib/$LIBFILE"
                ln -s "/usr/lib/$LIBFILE" "$LIBDIR/$LIBFILE"
              else
                echo ".... Creating link for $LIBDIR/$LIBFILE from $BINDIR$GEM_ROOT/lib"
                ln -s "$BINDIR$GEM_ROOT/lib/$LIBFILE" "$LIBDIR/$LIBFILE"
              fi
            fi
          fi
        fi
      done
    else
      for LIBFILE in `ls "$BINDIR$GEM_ROOT/lib"` ; do
        ALIBFILE="$BINDIR$GEM_ROOT/lib/$LIBFLIE"
      
        if [ -f "$ALIBFILE" ] && [ "$LIBFILE" != "librasgui.so.1" ] && [ "$LIBFILE" != "librasgui.so" ] && [ "$LIBFILE" != "libC.a" ] ; then
          if [ -h "$LIBDIR/$LIBFILE" ] ; then
            echo ".... Checking link for $LIBDIR/$LIBFILE"
            echo `ls -al "$LIBDIR/$LIBFILE"` | grep GEM > /dev/null
            if [ $? = 0 ] ; then
              echo ".... Removing link for $LIBDIR/$LIBFILE"
              rm "$LIBDIR/$LIBFILE"
            else
              echo `ls -al "$LIBDIR/$LIBFILE"` | grep TDS > /dev/null
              if [ $? = 0 ] ; then
                echo ".... Removing link for $LIBDIR/$LIBFILE"
                rm "$LIBDIR/$LIBFILE"
              fi
            fi
          fi
      
          if [ ! -f "$LIBDIR/$LIBFILE" ] ; then
            if [ ! -h "$LIBDIR/$LIBFILE" ] ; then
              if [ -f "/usr/lib/$LIBFILE" ] || [ -h "/usr/lib/$LIBFILE" ] ; then
                echo ".... Creating link for $LIBDIR/$LIBFILE from /usr/lib/$LIBFILE"
                ln -s "/usr/lib/$LIBFILE" "$LIBDIR/$LIBFILE"
              else
                echo ".... Creating link for $LIBDIR/$LIBFILE from $BINDIR$GEM_ROOT/lib"
                ln -s "$BINDIR$GEM_ROOT/lib/$LIBFILE" "$LIBDIR/$LIBFILE"
              fi
            fi
          fi
        fi
      done
    fi

  else
    echo ".... Unable to link shared libraries, LIBDIR undefined"
  fi

  #
  # Copy ee_startd.sh to /etc/Tivoli/ee_startd.sh
  #

  date
  echo ".... Checking for the startup script."

  if [ -d "/etc/Tivoli" ] ; then
    if [ -f "$BINDIR$GEM_ROOT/bin/ee_startd.sh" ] ; then
      if [ -f "/etc/Tivoli/ee_startd.sh" ] || [ -h "/etc/Tivoli/ee_startd.sh" ] ; then
        echo ".... Removing existing copy of /etc/Tivoli/ee_startd.sh"
        rm "/etc/Tivoli/ee_startd.sh"
      fi

      echo ".... Moving ee_startd.sh to /etc/Tivoli/ee_startd.sh"
      mv -f "$BINDIR$GEM_ROOT/bin/ee_startd.sh" "/etc/Tivoli/ee_startd.sh"
      chmod 755 "/etc/Tivoli/ee_startd.sh"
    else
      if [ -f "/etc/Tivoli/ee_startd.sh" ] ; then
        echo ".... Unable to find $BINDIR$GEM_ROOT/bin/ee_startd.sh."
        echo ".... /etc/Tivoli/ee_startd.sh will not be updated."
        echo ".... This is not an error."
      else
        echo ".... Unable to find $BINDIR$GEM_ROOT/bin/ee_startd.sh."
        echo ".... Unable to find /etc/Tivoli/ee_startd.sh."
        echo ".... This will prevent the daemon from being started"
        echo ".... by init at startup."
      fi
    fi
  else
    echo ".... Unable to copy ee_startd.sh to /etc/Tivoli to ee_startd.sh."
    echo ".... This will prevent the daemon from being started"
    echo ".... by init at startup."
  fi

  #
  # Copy ee_stop to /etc/Tivoli/ee_stop.sh
  #

  date
  echo ".... Checking the link for the ee_stop script."

  if [ -d "/etc/Tivoli" ] ; then
    if [ -f "/etc/Tivoli/ee_stop.sh" ] || [ -h "/etc/Tivoli/ee_stop.sh" ] ; then
      echo ".... Removing existing copy of /etc/Tivoli/ee_stop.sh"
      rm "/etc/Tivoli/ee_stop.sh"
    fi

    echo ".... Copying ee_stop to /etc/Tivoli/ee_stop.sh"
    ln -fs "$BINDIR$GEM_ROOT/bin/ee_stopd.sh" "/etc/Tivoli/ee_stop.sh"
  else
    echo ".... Unable to copy ee_stop to /etc/Tivoli."
    echo ".... This will prevent the daemon from being started"
    echo ".... by init at startup."
  fi

  #
  # On AIX, update /etc/inittab to start the daemon
  #
  if [ "$INTERP" = "aix4-r1" ] ; then
   if [ -f "/etc/rc.eegem" ] || [ -h "/etc/rc.eegem" ] ; then
    echo "The daemon is already enabled to start.  No action required."
   else
    if [ -f "/etc/Tivoli/ee_startd.sh" ] || [ -h "/etc/Tivoli/ee_startd.sh" ] ; then
      ln -fs "/etc/Tivoli/ee_startd.sh" "/etc/rc.eegem"
      if   [ -f "/usr/sbin/mkitab" ] ; then
        /usr/sbin/mkitab "rceegem:2:once:/etc/rc.eegem >/dev/console 2>&1 # Tivoli BSM Event Enablement"
      elif [ -f /etc/mkitab ] ; then
        /etc/mkitab "rceegem:2:once:/etc/rc.eegem >/dev/console 2>&1 # Tivoli BSM Event Enablement"
      else
        mkitab "rceegem:2:once:/etc/rc.eegem >/dev/console 2>&1 # Tivoli BSM Event Enablement"
      fi
      echo "The daemon is enabled to start."
    fi
   fi
  fi

  #
  # On Solaris, add a link in /etc/rc3.d so that ee_startd.sh can be
  # started by init
  #
  if [ "$INTERP" = "solaris2" ] ; then
   if [ -f "/etc/Tivoli/ee_startd.sh" ] || [ -h "/etc/Tivoli/ee_startd.sh" ] ; then
    if [ ! -h "/etc/rc3.d/S95ee_gem" ] ; then
     ln -fs "/etc/Tivoli/ee_startd.sh" "/etc/rc3.d/S95ee_gem"
     echo "/etc/rc3.d has been updated"
    else
     echo "The daemon is already enabled to start.  No action required."
    fi
   fi

   if [ -f "/etc/Tivoli/ee_stop.sh" ] || [ -h "/etc/Tivoli/ee_stop.sh" ] ; then
     if [ ! -h "/etc/rc0.d/K05ee_gem" ] ; then
       ln -fs "/etc/Tivoli/ee_stop.sh" "/etc/rc0.d/K05ee_gem"
       echo "/etc/rc0.d has been updated"
     else
       echo "The daemon is already enabled to stop.  No action required."
     fi
   fi
  fi

  #
  # On HP, update /sbin and /etc
  #
  if [ "$INTERP" = "hpux10" ] ; then
    if [ -f "$BINDIR/TDS/EventService/bin/rc.config.d.eegem" ] ; then
      if [ -f "/etc/rc.config.d/eegem" ] ; then
        rm "/etc/rc.config.d/eegem"
      fi

      cp "$BINDIR/TDS/EventService/bin/rc.config.d.eegem" "/etc/rc.config.d/eegem"
      chmod 444 "/etc/rc.config.d/eegem"
      echo "/etc/rc.config.d/eegem has been updated"
    fi

    if [ -f "$BINDIR/TDS/EventService/bin/sbin.init.d.eegem" ] ; then
      if [ -f "/sbin/init.d/eegem" ] ; then
        rm "/sbin/init.d/eegem"
      fi

      cp "$BINDIR/TDS/EventService/bin/sbin.init.d.eegem" "/sbin/init.d/eegem"
      chmod 755 "/sbin/init.d/eegem"
      echo "/sbin/init.d/eegem has been updated"
    fi

    if [ -f "/sbin/init.d/eegem" ] ; then
      ln -sf "/sbin/init.d/eegem" "/sbin/rc2.d/K200eegem"
      ln -sf "/sbin/init.d/eegem" "/sbin/rc3.d/S990eegem"

      if [ -h "/sbin/rc0.d/K110eegem" ] || [ -f "/sbin/rc0.d/K110eegem" ] ; then
        rm "/sbin/rc0.d/K110eegem"
      fi
      if [ -h "/sbin/rc1.d/K110eegem" ] || [ -f "/sbin/rc1.d/K110eegem" ] ; then
        rm "/sbin/rc1.d/K110eegem"
      fi
      if [ -h "/sbin/rc2.d/K110eegem" ] || [ -f "/sbin/rc2.d/K110eegem" ] ; then
        rm "/sbin/rc2.d/K110eegem"
      fi
    else
      echo "Error:  /sbin/init.d/eegem missing"
    fi
  fi

  #
  # On Linux, add a link in /etc/rc3.d so that ee_startd.sh can be
  # started by init
  #
  if [ "$INTERP" = "linux-ix86" ] || [ "$INTERP" = "linux-s390" ] ; then
   if [ -f "/etc/Tivoli/ee_startd.sh" ] || [ -h "/etc/Tivoli/ee_startd.sh" ] ; then
    if [ ! -h "/etc/rc3.d/S95ee_gem" ] ; then
     ln -fs "/etc/Tivoli/ee_startd.sh" "/etc/rc3.d/S95ee_gem"
     echo "/etc/rc3.d has been updated"
    else
     echo "The daemon is already enabled to start.  No action required."
    fi
   fi

   if [ -f "/etc/Tivoli/ee_stop.sh" ] || [ -h "/etc/Tivoli/ee_stop.sh" ] ; then
     if [ ! -h "/etc/rc0.d/K05ee_gem" ] ; then
       ln -fs "/etc/Tivoli/ee_stop.sh" "/etc/rc0.d/K05ee_gem"
       echo "/etc/rc0.d has been updated"
     else
       echo "The daemon is already enabled to stop.  No action required."
     fi
   fi
  fi

  #
  # Change file permissions
  #
  echo "INFO: Setting owner and permissions for each file and directory"

  if [ -f "$BINDIR/TDS/remove" ] ; then
    chown root "$BINDIR/TDS/remove"
    chmod 755 "$BINDIR/TDS/remove"
  fi

  if [ -d "$BINDIR/TDS/EventService" ] ; then
    chown root "$BINDIR/TDS/EventService"
    chmod 755 "$BINDIR/TDS/EventService"
    
    cd "$BINDIR/TDS/EventService"
    
    for THEFILE in `ls` ; do
      if [ -f "$THEFILE" ] ; then
        if [ "$THEFILE" != "." ] && [ "$THEFILE" != ".." ] ; then
          echo "chmod 755 $THEFILE"
          chown root $THEFILE
          chmod 755 $THEFILE
        else
          echo "INFO: Skipping $THEFILE"
        fi
      fi
    done
    
    echo "change directory contents"
    #for THEFILE in "bin" "codeset" "config" "db" "exitcache" "ipc" "lib" "log" "man" "msg_cat" "samples" "tools" "tmp" ; do
    for THEFILE in "bin" "codeset" "config" "db" "exitcache" "lib" "man" "msg_cat" "samples" "tools" "tmp" ; do
      if [ -d "$BINDIR/TDS/EventService/$THEFILE" ] ; then
        cd "$BINDIR/TDS/EventService/$THEFILE"
        echo "cd $BINDIR/TDS/EventService/$THEFILE"
        chown root "$BINDIR/TDS/EventService/$THEFILE"
        chmod 755 "$BINDIR/TDS/EventService/$THEFILE"
        for AFILE in `find . -print` ; do
          if [ "$AFILE" != "." ] && [ "$AFILE" != ".." ] ; then
            echo "chmod 755 $AFILE"
            chown root $AFILE
            chmod 755 $AFILE
          else
            echo "INFO: Skipping $AFILE"
          fi
        done
      else
        echo "INFO: Directory $BINDIR/TDS/EventService/$THEFILE was not found, permissions not set"
      fi
    done
  else
    echo "ERROR: Unable to find directory $BINDIR/TDS/EventService"
  fi

  #
  # Ensure mode for log and ipc directories
  #
  for DIR_NAME in ipc ipc/SEM ipc/QUEUES log ; do
    chmod 775 "$BINDIR$GEM_ROOT/$DIR_NAME"
  done

  if [ -f "$BINDIR/TDS/EventService/bin/hostcmd" ] ; then
    chown root "$BINDIR/TDS/EventService/bin/hostcmd"
    chmod 755 "$BINDIR/TDS/EventService/bin/hostcmd"
  fi

  if [ -f "$BINDIR/TDS/EventService/bin/ihsshstc" ] ; then
    chown root "$BINDIR/TDS/EventService/bin/ihsshstc"
    chmod 4755 "$BINDIR/TDS/EventService/bin/ihsshstc"
  fi

  #
  # Link command files to the TME environment
  #
  if [ -d "$BINDIR/bin" ] ; then
    echo ".... Checking for command files linked to $BINDIR/bin"
    for ABINFILE in config ee_config ee_start ee_stop ee_utility getpd hostcmd hostcmdoper start stop tcpipkey tserver ihszset ihszsetapi ihszfmtrl ; do
      if [ -f "$BINDIR/TDS/EventService/bin/$ABINFILE" ] ; then
        if [ -h "$BINDIR/bin/$ABINFILE" ] ; then
          echo ".... Checking link for $BINDIR/bin/$ABINFILE"
          echo `ls -al "$BINDIR/bin/$ABINFILE"` | grep TDS > /dev/null
          if [ $? = 0 ] ; then
            echo ".... Removing link for $BINDIR/bin/$ABINFILE"
            rm "$BINDIR/bin/$ABINFILE"
          fi
        fi

        if [ ! -f "$BINDIR/bin/$ABINFILE" ] ; then
          if [ ! -h "$BINDIR/bin/$ABINFILE" ] ; then
            echo ".... Creating link for $BINDIR/bin/$ABINFILE from $BINDIR$GEM_ROOT/bin"
            ln -s "$BINDIR$GEM_ROOT/bin/$ABINFILE" "$BINDIR/bin/$ABINFILE"
          fi
        fi
      fi
    done
  else
    echo ".... Unable to link command files, $BINDIR/bin undefined"
  fi

  #
  # Link manpage files to the TME environment
  #
  if [ -d "$BINDIR/../../man/$INTERP/man1" ] ; then
    echo ".... Checking for manpage files linked to $BINDIR/../../man/$INTERP/man1"
    for AMANFILE in `ls "$BINDIR$GEM_ROOT/man/man1/"` ; do
      # */ #
      MANFILE=`basename "$AMANFILE"`
      if [ -f "$BINDIR/TDS/EventService/man/man1/$MANFILE" ] ; then
        if [ -h "$BINDIR/../../man/$INTERP/man1/$MANFILE" ] ; then
          echo ".... Checking link for $BINDIR/../../man/$INTERP/man1/$MANFILE"
          echo `ls -al "$BINDIR/../../man/$INTERP/man1/$MANFILE"` | grep TDS > /dev/null
          if [ $? = 0 ] ; then
            echo ".... Removing link for $BINDIR/../../man/$INTERP/man1/$MANFILE"
            rm "$BINDIR/../../man/$INTERP/man1/$MANFILE"
          fi
        fi

        if [ ! -f "$BINDIR/../../man/$INTERP/man1/$MANFILE" ] ; then
          if [ ! -h "$BINDIR/../../man/$INTERP/man1/$MANFILE" ] ; then
            echo ".... Creating link for $BINDIR/../../man/$INTERP/man1/$MANFILE from $BINDIR$GEM_ROOT/man/man1"
            ln -s "$BINDIR$GEM_ROOT/man/man1/$MANFILE" "$BINDIR/../../man/$INTERP/man1/$MANFILE"
          fi
        fi
      fi
    done
  else
    echo ".... Unable to link manpage files, $BINDIR/../../man/$INTERP/man1 undefined"
  fi

  #
  # Create tmp directory for ipc errors
  #
  echo  ".... Creating /tmp/Tivoli/tbsm"

  if [ ! -d "/tmp" ] ; then
     mkdir "/tmp"
  fi

  if [ -d "/tmp" ] ; then
    if [ ! -d "/tmp/Tivoli" ] ; then
       mkdir "/tmp/Tivoli"
    fi

    if [ -d "/tmp/Tivoli" ] ; then

      if [ ! -d "/tmp/Tivoli/tbsm" ] ; then
        mkdir "/tmp/Tivoli/tbsm"
      fi

      if [ ! -d "/tmp/Tivoli/tbsm" ] ; then
        echo ".... Unable to create /tmp/Tivoli/tbsm for ipc error log"
      fi
    else
        echo ".... Unable to create /tmp/Tivoli for ipc error log"
    fi
  else
    echo ".... Unable to create /tmp for ipc error log"
  fi

  #
  # end of UNIX unique processing
  #
else
  echo "Install Tivoli Event Enablement service"

  _GEM_EE_="yes"
  export _GEM_EE_

  #
  # Setup  the environment
  #
  if [ -n "$TSERVER_ETC" ] ; then
    if [ -f "$TSERVER_ETC/setup_env.sh" ] ; then
      if [ -n "$windir" ] ; then
        . "$TSERVER_ETC/setup_env.sh"
      elif [ -n "$OS2_SHELL" ] ; then
        PATH="$BINDIR/bin:$BINDIR/tools:$PATH"
        export PATH
      fi
    fi
  else
    if [ -n "$windir" ] ; then
      WINOS=`echo "$windir" | awk /:/'{ print substr($1,4) }'`
      WINDRIVE=`echo "$windir" | awk /:/'{ print substr($1,1,2) }'`
      if [ -n "$WINOS" ] ; then
        if [ -f "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/setup_env.sh" ] ; then
          . "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/setup_env.sh"
        fi
      fi
    elif [ -n "$OS2_SHELL" ] ; then
      PATH="$BINDIR/bin:$BINDIR/tools:$PATH"
      export PATH
    fi
  fi

  if [ -n "$NLSPATH" ] ; then
    NLSPATH="$NLSPATH:$BINDIR$GEM_ROOT/msg_cat/%L/%N.cat"
  else
    NLSPATH="$BINDIR$GEM_ROOT/msg_cat/%L/%N.cat"
  fi
  export NLSPATH

  PATH="$PATH:$BINDIR$GEM_ROOT/bin"
  export PATH

  #
  # Delete old dlls
  #
  for OLDFILE in cppwot3.dll ihsucrtl.dll ihsuob3i.dll ihsuor3u.dll ihsuou3i.dll ihsuov3.dll ; do
    if [ -f "$BINDIR/TDS/EventService/$OLDFILE" ] ; then
      echo "Removing: $BINDIR/TDS/EventService/$OLDFILE"
      rm "$BINDIR/TDS/EventService/$OLDFILE"
    fi
  done

  for OLDFILE in cppwot3.dll ihsucrtl.dll ihsuob3i.dll ihsuor3u.dll ihsuou3i.dll ihsuov3.dll ; do
    if [ -f "$BINDIR/TDS/EventService/bin/$OLDFILE" ] ; then
      echo "Removing: $BINDIR/TDS/EventService/bin/$OLDFILE"
      rm "$BINDIR/TDS/EventService/bin/$OLDFILE"
    fi
  done
     
  "$BINDIR$GEM_ROOT/bin/ihstsrv" install AUTO "$BINDIR$GEM_ROOT/bin"

  #
  # Copy command files to the TME environment
  #
  if [ -d "$BINDIR/bin" ] ; then
    echo ".... Checking for command files in $BINDIR/bin"
    for ABINFILE in config ee_config ee_start ee_stop ee_utility getpd hostcmd hostcmdoper start stop tcpipkey tserver.cmd ihszset.exe ihszsetapi ihszfmtrl ; do
      if [ -f "$BINDIR/TDS/EventService/bin/$ABINFILE" ] ; then
        if [ -f "$BINDIR/bin/$ABINFILE" ] ; then
          echo ".... Removing file $BINDIR/bin/$ABINFILE"
          if [ ! -f "$BINDIR$GEM_ROOT/tmp/bin/$ABINFILE" ] ; then
             cp "$BINDIR/bin/$ABINFILE" "$BINDIR$GEM_ROOT/tmp/bin/$ABINFILE"
          fi
          rm "$BINDIR/bin/$ABINFILE"
        fi

        if [ ! -f "$BINDIR/bin/$ABINFILE" ] ; then
          echo ".... Copying $BINDIR/bin/$ABINFILE from $BINDIR$GEM_ROOT/bin"
          cp "$BINDIR$GEM_ROOT/bin/$ABINFILE" "$BINDIR/bin/$ABINFILE"
        fi
      fi
    done
  else
    echo ".... Unable to copy command files, $BINDIR/bin undefined"
  fi

  #
  # Copy manpage files to the TME environment
  #
  if [ -d "$BINDIR/../../man/$INTERP/man1" ] ; then
    echo ".... Checking for manpage files in $BINDIR/../../man/$INTERP/man1"
    for AMANFILE in `ls "$BINDIR$GEM_ROOT/man/man1/*"` ; do
      # */ #
      MANFILE=`basename "$AMANFILE"`

      if [ -f "$BINDIR/TDS/EventService/man/man1/$MANFILE" ] ; then
        if [ -f "$BINDIR/../../man/$INTERP/man1/$MANFILE" ] ; then
          echo ".... Removing $BINDIR/../../man/$INTERP/man1/$MANFILE"
          rm "$BINDIR/../../man/$INTERP/man1/$MANFILE"
        fi

        if [ ! -f "$BINDIR/../../man/$INTERP/man1/$MANFILE" ] ; then
          echo ".... Copying $BINDIR/../../man/$INTERP/man1/$MANFILE from $BINDIR$GEM_ROOT/man/man1"
          cp "$BINDIR$GEM_ROOT/man/man1/$MANFILE" "$BINDIR/../../man/$INTERP/man1/$MANFILE"
        fi
      fi
    done
  else
    echo ".... Unable to copy manpage files, $BINDIR/../../man/$INTERP/man1 undefined"
  fi


fi

for MSGCAT in `find $BINDIR/../../msg_cat -name "ihs*.cat"`
do
    CATNAME=`basename $MSGCAT` # get message catalog file name
    DIRNAME=`dirname $MSGCAT`  # get message catalog directory name
    LANGNAME=`basename $DIRNAME`  # get message catalog parent directory

# Avoid operating on C directory; on Windows, C directory contains actual copies of EE 
#   message catalogs (whereas -- on UNIX -- it only contains symlinks)
    if [ ! "$LANGNAME" = "C" ]
    then
# If file already exists, back it up ... just in case
        if [ -d $BINDIR/TDS/EventService/msg_cat/$LANGNAME ]
        then 
          if [ -f $BINDIR/TDS/EventService/msg_cat/$LANGNAME/$CATNAME ]
          then
              mv -f $BINDIR/TDS/EventService/msg_cat/$LANGNAME/$CATNAME $BINDIR/TDS/EventService/msg_cat/$LANGNAME/backup.$CATNAME
           fi
# ...otherwise, create target directory
        else
            mkdir -p $BINDIR/TDS/EventService/msg_cat/$LANGNAME
        fi

# If running on Windows, COPY file from standard message catalog location to TDS message catalog location
        if [ "$INTERP" = "w32-ix86" ]
        then
            cp $MSGCAT $BINDIR/TDS/EventService/msg_cat/$LANGNAME
# ...in all other cases, MOVE file and then create a symbolic link back to the original location
        else
            mv -f $MSGCAT $BINDIR/TDS/EventService/msg_cat/$LANGNAME
            ln -s $BINDIR/TDS/EventService/msg_cat/$LANGNAME/$CATNAME $MSGCAT
        fi
    fi
done

if [ -f "$BINDIR/TDS/EventService/ihsttec.cfg" ] ; then
  rm "$BINDIR/TDS/EventService/ihsttec.cfg"
fi

if [ -f "$BINDIR/TDS/EventService/interapp.baroc" ] ; then
  rm "$BINDIR/TDS/EventService/interapp.baroc"
fi

if [ -f "$BINDIR/TDS/EventService/interapp.rls" ] ; then
  rm "$BINDIR/TDS/EventService/interapp.rls"
fi

if [ -f "$BINDIR/TDS/EventService/ihstec.baroc" ] ; then
  rm "$BINDIR/TDS/EventService/ihstec.baroc"
fi

#
# Check for TME
#
if [ -f "$BINDIR$GEM_ROOT/config/ihstctme.cfg" ] ; then
  rm "$BINDIR$GEM_ROOT/config/ihstctme.cfg"
fi

if [ ! -f "$BINDIR/bin/winterp" ] && [ ! -f "$BINDIR/bin/winterp.exe" ] ; then
  echo "TME Unavailable" > "$BINDIR$GEM_ROOT/config/ihstctme.cfg"
fi

#
# remove script
#
if [ ! -d "$BINDIR/../generic_unix" ] ; then
  mkdir "$BINDIR/../generic_unix"
fi

if [ ! -d "$BINDIR/../generic_unix/TDS" ] ; then
  mkdir "$BINDIR/../generic_unix/TDS"
fi

echo "Event Enablement setup complete."


