#!/bin/sh
#**************************************************************** {COPYRIGHT-TOP-RM} ***
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#**************************************************************** {COPYRIGHT-END-RM} ***
#******************************************************************
# TBSMSTATUS: procedure for extending the Tivoli Event Console (TEC)
# with tivoli Business Systems Manager (TBSM) definitions for
# TBSM_problem_ticket_event events.
#
# Command line input:
#   rulebasename
#   CalledFromIHSTTEC.sh
#
################################################################

################################################################
#
# Collect the input arguments.
#
################################################################

if [ $# = 2 ]; then
  OutputRulesBaseName=$1
  CalledFromIHSTTEC="TRUE"
  VariablePrompt="FALSE"
elif [ $# = 1 ] ; then
  OutputRulesBaseName=$1
  CalledFromIHSTTEC="FALSE"
  VariablePrompt="FALSE"
else
  OutputRulesBaseName="GemEvent"
  CalledFromIHSTTEC="FALSE"
  VariablePrompt="TRUE"
fi

if [ "$CalledFromIHSTTEC" = "FALSE" ] ; then
  ################################################################
  #
  # The following sets the environment so that we are sure that
  # we can address all the needed Tivoli commands.
  #
  ################################################################
  # get the operating system identity and set up for Tivoli commands
  if [ "`uname`" != "Windows_NT" ] ; then
    . "/etc/Tivoli/setup_env.sh"  # setup for Tivoli environment
    THE_OS="UNIX"
    export THE_OS
  else
    THE_OS="INTEL"
    export THE_OS
    if [ -n "$windir" ] ; then
      WINOS=`echo "$windir" | awk /:/'{ print substr($1,4) }'`
      WINDRIVE=`echo "$windir" | awk /:/'{ print substr($1,1,2) }'`
      if [ -n "$WINOS" ] ; then
        if [ -f "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/setup_env.sh" ] ; then
          if [ -z "$T_SERVT" ] ; then
            . "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/setup_env.sh"
          fi
          TDS_OS="PC"
        else
          echo "rc(5902) setup_env.sh not found"
        fi
      else
        echo "rc(5902) setup_env.sh not found"
      fi
    else
      echo "rc(5902) setup_env.sh not found"
    fi
  fi

  if [ ! -d "/tmp" ]
    then mkdir "/tmp"                  # Temporary directory required
  fi
fi

EE_HOME="$BINDIR/TDS/EventService"
export EE_HOME
EE_HOME_BIN="$EE_HOME"
export EE_HOME_BIN
EE_HOME_CONFIG="$EE_HOME/config"
export EE_HOME_CONFIG

if [ -f "$BINDIR/bin/wrb" ] ; then
  USE_WRB="Y"
  CMD_LSRBRULES="wrb -lsrbrule"
  CMD_DELRBRULES="wrb -delrbrule"
  CMD_LSRBCLASS="wrb -lsrbclass"
  CMD_DELRBCLASS="wrb -delrbclass"
  CMD_IMPRBCLASS="wrb -imprbclass"
  CMD_IMPRBRULES="wrb -imprbrule"
  CMD_IMPTGTRULE="wrb -imptgtrule"
  CMD_COMPRULES="wrb -comprules"
  CMD_LOADRB="wrb -loadrb -use"
else
  USE_WRB="N"
  CMD_LSRBRULES="wlsrbrules"
  CMD_DELRBRULES="wdelrbrules"
  CMD_LSRBCLASS="wlsrbclass"
  CMD_DELRBCLASS="wdelrbclass"
  CMD_IMPRBCLASS="wimprbclass"
  CMD_IMPRBRULES="wimprbrules"
  CMD_COMPRULES="wcomprules"
  CMD_LOADRB="wloadrb -u"
fi

if [ "$VariablePrompt" = "TRUE" ] ; then
  ################################################################
  #
  # Prompt for the name of the rulebase that we want to add the
  # new class to.
  #
  ################################################################
  echo ' '
  echo 'Please enter the name of the TEC Rules Base to which'
  echo 'the TBSM_problem_ticket_event class will be added.'
  echo 'If no name is entered we will use the name:' $OutputRulesBaseName
  read UserReply
  if [ q$UserReply != "q" ] ; then
    OutputRulesBaseName=$UserReply
  fi
fi


################################################################
# Import .baroc and .rls files
################################################################

echo "Checking rules in $OutputRulesBaseName"
for RULESET in `$CMD_LSRBRULES $OutputRulesBaseName` ; do
  if [ "$RULESET" = "tbsmtroubleticket.rls" ] ; then
    if [ "$USE_WRB" = "Y" ] ; then
      set -x
      `$CMD_DELRBRULES "tbsmtroubleticket" $OutputRulesBaseName > tbsmtroubleticket.out`
      set +x
    else
      set -x
      `$CMD_DELRBRULES "$RULESET" $OutputRulesBaseName`
      set +x
    fi
  fi
done

echo "Checking classes in $OutputRulesBaseName"
for RULECLASS in `$CMD_LSRBCLASS $OutputRulesBaseName` ; do
  if [ "$RULECLASS" = "TBSM_problem_ticket_event" ] ; then
    set -x
    `$CMD_DELRBCLASS tbsmtroubleticket.baroc $OutputRulesBaseName > tbsmtroubleticket.out`
    set +x
  fi
done

if [ -f tbsmtroubleticket.out ] ; then
  rm tbsmtroubleticket.out
fi

echo "Updating classes and rules in $OutputRulesBaseName"
set -x
$CMD_IMPRBCLASS "$EE_HOME_CONFIG/tbsmtt/itimzos/tbsmtroubleticket.baroc" $OutputRulesBaseName

$CMD_IMPRBRULES "$EE_HOME_CONFIG/tbsmtt/itimzos/tbsmtroubleticket.rls" $OutputRulesBaseName
set +x

if [ "$USE_WRB" = "Y" ] ; then
  set -x
  $CMD_IMPTGTRULE tbsmtroubleticket EventServer $OutputRulesBaseName
  set +x
fi

if [ "$CalledFromIHSTTEC" = "FALSE" ] ; then
  ################################################################
  #
  # Rules bases must be compiled and loaded for the TEC Server and
  # Event Services to function.  Compilation takes the baroc and rls
  # files and creates PROLOG expansions.  Loading copies the rules base
  # to the operational location $DBDIR/tec/rb_dir
  #
  ################################################################
  echo ' '
  echo 'Terminate execution if any errors occured during the previous step.'
  echo 'The next step is to compile' $OutputRulesBaseName 'rules base.'
  echo '  Press Enter to continue or "q" to quit.'
  read UserReply
  if [ "$UserReply" = 'q' -o "$UserReply" = 'Q' ]; then
  	exit 1
  fi

  echo 'Compiling' $OutputRulesBaseName 'rules base'
  set -x
  $CMD_COMPRULES $OutputRulesBaseName
  set +x

  echo 'Terminate execution if any errors occured during the rules compile'
  echo 'The next step is to Load the' $OutputRulesBaseName 'rules base.'
  echo '  Press Enter to continue or "q" to quit.'
  read UserReply
  if [ "$UserReply" = 'q' -o "$UserReply" = 'Q' ]; then
  	exit 1
  fi

  set -x
  $CMD_LOADRB $OutputRulesBaseName
  set +x

  ################################################################
  #
  # We need to stop and start the TEC Event Server to activate the
  # rules base we created, compiled and loaded earlier in this procedure
  #
  ################################################################
  echo ' '
  echo 'Stopping and restarting the Event Server'
  echo '  Press Enter to continue or "q" to quit.'
  read UserReply
  if [ "$UserReply" = 'q' -o "$UserReply" = 'Q' ]; then
  	exit 1
  fi

  set -x
  wstopesvr
  # Give the process time to complete before restarting.
  sleep 10
  wstartesvr
  set +x
fi
