#!/etc/Tivoli/bin/perl
#**************************************************************** {COPYRIGHT-TOP-RM} ***
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#**************************************************************** {COPYRIGHT-END-RM} ***

#-------------------------------------------------------------------------------
# Subroutine "usage" displays the help screen if we are not being invoked
# from the TEC rules.
#-------------------------------------------------------------------------------
sub usage {
   if ($ENV{TEC_BIN_DIR}) {
      return; # Do not display help screen if being called from TEC rules
   }


   print <<"HelpScreen";

File:          send_to_TBSM.pl
Version:       1.1
Date:          2004-11-29

Product:       IBM Tivoli Business Systems Manager 3.1 GA

Purpose:       Forward events to TBSM via TBSM Event Enablement

Description:   Based on parameters and environment variables this script 
               formats an event for forwarding to TBSM. It does not alter 
               the event in TEC itself.

               Events are forwarded to TBSM only via the TBSM Event Service
               exit ihstttec, thus they will all be handled by TBSM as 
               generic events. This includes DM events as well as ITM events.

Input:         a) TEC event attributes via environment variables

               b) Debug and trace control via environment variables:
                  $debug: 0 (or undefined) means this script will
                            forward the event to TBSM normally
                          1 (or any non-null value) means this script
                            will not actually forward the event, but
                            will otherwise act normally.
                  $trace: 0 No tracing
                          1 When error occurs trace:
                               a. Local timestamp
                               b. TEC server_handle, event_handle
                               c. TEC date_reception
                               d. Error information
                          2 Each event trace on 1 line everything above and:
                               e. TBSM Event Enablement command line
                          3 Trace on multiple lines everything above and:
                               f. Adds limited environment tracing
                               g. Most messages
                          4 Trace everything above and:
                               h. Complete environment trace
                          5 Trace everything above and:
                               i. $BINDIR determination messages

               c) Command line parameters

                  Generally one or more command line parameters are expected. 
                  All the parameter keywords are shown in uppercase, but they
                  can be lower or mixed case as well. 
                  
                  The following keywords tell the script how to format the 
                  event. If a keyword is not coded, then this script will 
                  attempt to figure out the event type via the environment.
                  Since you typically know this already in the TEC rule, it 
                  simplifies things if you just code the keyword.
                  
                  The only conceivable exception would be if you forwarded all
                  events to TBSM and wanted the script to decide which events
                  to process.

                  DM:          Classic DM event

                  SENTRY:      Same as keyword DM

                  ITM:         Non-PAC ITM event

                  NETVIEW:     IBM NetView event (TEC_ITS_NODE_STATUS,
                                                  TEC_ITS_INTERFACE_STATUS,
                                                  TEC_ITS_ROUTER_STATUS)

                  NVOLD:       IBM NetView event (old OV_Node_Down)

                  HPOV:        HP OpenView event

                  The following keywords can be used to modify the script's 
                  behaviour for a particular event source.

                  CHG:         Status change to an existing event. This is
                               typically coded on change rules that watch for 
                               a change to an event's status such as when an 
                               event is closed on TEC.

                  PROF value:  This keyword, along with a subsequent value,
                               tells the script to pass the supplied value
                               to TBSM in lieu of an explicit object class.
                               TBSM will then lookup the object class in a
                               TBSM table that is customized by the user.
                               A primary use of this is to supply DM or ITM
                               profile names that are used to map to a TBSM
                               object class as was historically done in TBSM
                               with DM events passed via the TBSM EE
                               ihstztec exit.

                  INTERP:      Instead of relying upon naming conventions when
                               determining TBSM object classes, use the TBSM 
                               Event Service's ability to figure out the 
                               interp type for Tivoli endpoints, and use that 
                               interp type to determine TBSM object classes. 
                               Note that if TEC is running in its own TMR that
                               this requires that endpoint names be exchanged 
                               with the TEC TMR.
                               
                  FQHOSTNAME:  Normally this script assumes the TCP hostname is
                               the value found in the event's HOSTNAME slot. If
                               you use this keyword the fully qualified (domain 
                               qualified) TCP hostname will be substituted as
                               the event flows through the TBSM EventService. 
                               
                               TBSM EE can do this because it maintains a cache
                               of all TCP hostnames for every host that sends
                               an event. Whenever a new hostname is seen TBSM
                               performs either a gethostbyaddr of the dotted 
                               decimal value passed on the -q flag or a 
                               gethostbyname of the hostname value passed on
                               the -h flag depending upon which of the two 
                               flags were used with ihstttec.
                               
                               Using this keyword puts the requirement on TEC 
                               that it can correctly resolve the passed 
                               hostnames and dotted decimal addresses.

                  HOSTNAME:    As does the FQHOSTNAME option above, this causes 
                               the TBSM Event Service to substitute a portion 
                               of the fully qualified TCP hostname for the 
                               hostname in the mapped event.
                               
                               The difference between FQHOSTNAME and FQHOSTNAME
                               keywords is that instead of using the full TCP 
                               hostname with domain qualification, a shortened
                               TCP hostname is used. The number of domain tokens
                               tokens removed is configured with the -t option 
                               on the ee_config command.
                               
                  NOTE: In the ihstttec command this script builds to forward 
                        the event, the strings %INTER%, %FQHOSTNAME% and 
                        %HOSTNAME% are used as appropriate depending upon 
                        which, if any, of the input keywords INTERP, FQHOSTNAME
                        and HOSTNAME are used. As the event flows through the 
                        Event Service the %INTER%, %FQHOSTNAME% and %HOSTNAME%
                        strings are substituted with the appropriate value.
                        
                        The TBSM Event Service will substitiute the %INTERP%,
                        %FQHOSTNAME% and %HOSTNAME% keywords from selected 
                        ihstttec command line options as follows:
                        
                                      -b (ProdName)   -i (instID)   -d (label)
                                      -------------   -----------   ----------
                        %INTERP%            y                            y
                        %HOSTNAME%                         y             y
                        %FQHOSTNAME%                       y             y

                        
Output:        1) Event forwarded to TBSM Event Service via ihstttec
               2) Trace file send_to_TBSM.trace if enabled via $trace variable.
                  Trace file is saved in first directory found: /tmp, /temp, /
                  Default is to enable tracing at the rate of 1 line per event.

Dependencies:  Root authority is required to execute ihstttec when forwarding

Restrictions:

Disclaimer:    Several techniques have been implemented to determine which 
               TBSM object class a given event is posted to. You may or may 
               not be able to use one of these techniques depending upon the
               specifics of your environment. The expectation is that you 
               will have to make adjustments to this script.

Usage:         Run this script from TEC rules. For debug purposes you can run
               it from the command line if the appropriate environment 
               variables have been set to simulate the event.

               Turn trace and debug on or off as desired via the $trace and 
               $debug variables. For extended use $trace should be set to 
               $trace_none, $trace_error or at most $trace_limited.

Invocation:    send_to_TBSM.pl DM|ITM|NETVIEW|NVOLD|HPOV [CHG] [INTERP] [FQHOSTNAME|HOSTNAME]

Return Codes:  Bit mapped as follows

               0000 0000 0000 0000 - No problems
               0000 0000 0000 0001 - Unable to make trace file directory

               0000 0000 0001 0000 - Abend: ihstttec format error
               0000 0000 0010 0000 - Abend: Unable to determine $BINDIR
               0000 0000 0100 0000 - Abend: Bad parameter passed
               0000 0000 1000 0000 - Abend: ITM PAC event

               xxxx xxxx 0000 0000 - Bad ihstttec return code in upper byte

Future Enhancements:

------------------------------------------------------------------------------

Change Activity:

2003-08-28 Development version
2004-02-08 TBSM 3.1 GA version

HelpScreen
}

# Main Program:
#    Determine trace level and initialize trace 
#    Copy event attributes to perl variables
#    Process command line parameters
#    Determine default behaviour if expected command line parameters are missing
#    Normalize and tokenize hostname
#    Call format_generic_event to process the event we are forwarding
#       Call default_object_class with "Server" specified as default
#          Customer-specific code may be added to interpret TCP host naming conventions and set new default TBSM object class
#       Set default values common to all events
#       Depending on config enable clear of closed events rather than deletion
#       Case statement selects stanza for this event source and extract source-specific information
#          Call set_obj_class_and_icon_label from each stanza to possibly override object class and to set icon label
#       Call sendGenericEventToTBSM to build actual ihstttec command that forwards the event
#          Build ihstttec command
#          Check for malformed ihstttec command and abend if malformed
#          Execute ihstttec command to forward event to TBSM

# Subroutines:
#   usage                           Display help screen and exit
#   format_generic_event            One stanza per event source to build strings used by sendGenericEventToTBSM
#   append_itm_slots_to_desc        Builds list of non-standard ITM slots for use in TBSM description
#   sendGenericEventToTBSM          Builds actual TBSM EE ihstttec command line and executes it
#   handleError                     Handles fatal errors and exits
#   setUpTrace                      Prepares trace
#   traceEnv                        Adds copy of environment variable settings to trace
#   setCmdPaths                     Determines absolute path to ihstttec regardles of environment (TEC rule, command line, TBSM DB server, Unix, Windows)
#   default_object_class            Decodes TCP hostname naming conventions to determine TBSM object class
#   set_obj_class_and_icon_label    Determines ihstttec -b and -d values. Default object class -b value may be overridden here due to optional keywords passed to script.
#   

                                           # Trace Level Constants
                                           # ---------------------
$trace_none       = 0;                     # 0 = No tracing
$trace_error      = 1;                     # 1 = Limited amount of tracing, can leave enabled longer
                                           #     Includes timestamp, event_handle, server_handle, date_reception, error information
$trace_limited    = 2;                     # 2 = Limited amount of tracing, can leave enabled longer
                                           #     Traces TBSM event enablement command line
$trace_full       = 3;                     # 3 = All trace messages enabled except EE path determination.
                                           #     Traces environment variables: $event_type
$trace_full_env   = 4;                     # 4 = All trace messages enabled except EE path determination.
                                           #     Before & after copy of environment.
$trace_setup      = 5;                     # 5 = All trace messages enabled including EE path determination.
                                           #     Before & after copy of environment.


$trace_append     = 1;                     # 0 = overwrite trace file for each new event
                                           # 1 = append    trace file for each new event

$trace_max        = 2000000;               # max size of trace file in decimal bytes, e.g. 2000000

#-------------------------------------------------------------------------------
# The trace and debug control variables ($trace and $debug) are typically set 
# during command line testing via the environment, e.g. "export trace=3 or export debug=1
#
# To force tracing at a certain level change the setting of:              $trace
# To force a specific directory for the trace file change the setting of: $trace_dir
#             default directory is first available of:                    /tmp,  /temp,  /
# To force a specific trace file name change the setting of:              $trace_fn
#-------------------------------------------------------------------------------
if ( defined($ENV{trace}) ) {
   $trace     = $ENV{trace};               # Inherit decision on whether or not to trace, and at what verbosity
} else {
   $trace     = $trace_error;              # otherwise default to this trace level
}

$trace_dir = "";                           # Default trace directory. If null then use first directory found: /tmp, /temp, /. Creates directory otherwise.
$trace_fn  = "send_to_TBSM.trace";         # Default trace file name

if ( defined($ENV{debug}) ) {
   $debug            = $ENV{debug};        # Inherit decision whether to send events: 0 to send events, 1 to inhibit sending
} else {
   $debug            = 0;                  # otherwise default to sending them:       0 to send events, 1 to inhibit sending
}

$date             = `date`; chop $date;    # useful to have consistent timestamp for messages

# Hardcode trace file destination or set to null for automatic placement
# by subroutine setUpTrace. If automatic the filename will be
# send_to_TBSM.trace, and if running on a Tivoli managed node it will be
# placed in the directory pointed to by wtemp. If not running on a
# managed node. and if the location is not hardcoded, it will default to
# /tmp.
if ( $trace > $trace_none) {
   &setUpTrace();
}

if ( $trace >= $trace_full) {
   print TRACE_OUT "   Cmd Line Tokens:     ";
   foreach $arg (@ARGV) {
      print TRACE_OUT $arg . " ";
   }
   print TRACE_OUT "\n";
}


#-------------------------------------------------------------------------------
# TBSM EE release expressed as a 3 digit number
#
# Example Values:
#    211 means TBSM 2.1.1
#    310 means TBSM 3.1    (trailing zero required)
#
# This enables/disables script features and behaviour. In particular you don't
# want to to enable 3.1 function on a 2.1.1 system where the new function is
# not yet available.
#
# Here are its affects:
#
# eever <  310    a. Use ProdName for icon labels. The expectation is that the TBSM 
#                    console will be configured to use medium or short labels.
#
# eever >= 310    a. Use instance id for icon labels. The expectation is that TBSM EE will be
#                    configured to not append the string "@tcp_hostname" to all labels,
#                    and that the TBSM console will be configured to use either medium or short
#                    icon labels.
#-------------------------------------------------------------------------------
$eever = 310;

if ($eever < 310) {                        # ProductVersion value to be appended to the $obj_class
   $prodver       = "1.0";                 # value when $obj_class contains a ProductName. Note that
} else {                                   # as of TBSM 3.1 this becomes optional and is probably best
   $prodver       = "";                    # left to null.
}


#-------------------------------------------------------------------------------
# The TEC <--> TBSM status synchronization feature shipped with TBSM, in the
#     TEC --> TBSM direction, is intended to delete an event on TEC when it is
# closed at TBSM. This is done with a change rule on TEC that catches the event
# status changing to CLOSED (or ACK or RESPONSE). The TEC rule then forwards the
# event to TBSM.
#
# If this script is used to forward the event then the TEC rule will pass the CHG
# keyword to the script. The following $closed_events_green_cfg variable controls
# how this script handles the closed event.
#
# If the variable is set to 0 this script will forward the event to TBSM with the
# ihstttec -n option which causes the original event to be deleted from TBSM
#
# If the variable is set to 1 then the original event will be sent again without
# the ihstttec -n option, and the original event will be updated on TBSM to green
# with "Closed at TEC: " prepended to the alert description on TBSM.
#-------------------------------------------------------------------------------
$closed_events_green_cfg = 0; # 0=delete closed events on TBSM   1=change closed events to green on TBSM

#-------------------------------------------------------------------------------
# Get event slots of interest from the environment. Not all of these are
# preset for all event types. The table shows which event source(s) have code
# for which event attributes.
#
#                                                          E  v  e n  t      A  t  t  r  i  b  u  t  e  s
#                                                         -----------------------------------------------
#  Script Variable              Event Attribute            DM   ITM  ITMHB ITMTP  NETVIEW NVOLD  HPOV 
#----------------------       --------------------------- ---- ----- ----- -----  ------- -----  ---- 
$EVENT_CLASS            = $ENV{EVENT_CLASS};            #        y     y     y       y      y     y   
$source                 = $ENV{source};                 #   y    y     y     y       y      y     y   
$sub_source             = $ENV{sub_source};             #   y
$hostname_slot          = $ENV{hostname};               #   y    y     y     y       y      y     y   
$origin                 = $ENV{origin};                 #                                   y     y
$severity               = $ENV{severity};               #   y    y     y     y       y      y     y   
$msg                    = $ENV{msg};                    #   y    y     y     y       y      y     y   
$message                = $ENV{message};                #        y           y
$status                 = $ENV{status};                 #   y    y     y     y       y
$monitor                = $ENV{monitor};                #   y
$probe                  = $ENV{probe};                  #   y
$probe_arg              = $ENV{probe_arg};              #   y
$value                  = $ENV{value};                  #   y
$units                  = $ENV{units};                  #   y
$application_class      = $ENV{application_class};      #        y           y
$profilename            = $ENV{profilename};            #        y           y
$modelname              = $ENV{modelname};              #        y           y
$event_key              = $ENV{event_key};              #        y           y
$eventname              = $ENV{eventname};              #        y           y
$SLOTS                  = $ENV{SLOTS};                  #        y           y
$applicationname        = $ENV{applicationname};        #                    y
$transactionname        = $ENV{transactionname};        #                    y
$nodestatus             = $ENV{nodestatus};             #                            y
$ifstatus               = $ENV{ifstatus};               #                            y
$ifname                 = $ENV{ifname};                 #                            y
$routerstatus           = $ENV{routerstatus};           #                            y
$networkid              = $ENV{networkid};              #                            y
$OV_status              = $ENV{OV_status};              #                                   y     y
$ic_class_name          = $ENV{ic_class_name};          #                                             
#$server_handle         = $ENV{server_handle};          #   y    y     y     y       y      y     y         # Used in routine setUpTrace for debug correlation with TEC
#$date_reception        = $ENV{date_reception};         #   y    y     y     y       y      y     y         # Used in routine setUpTrace for debug correlation with TEC
#$event_handle          = $ENV{event_handle};           #   y    y     y     y       y      y     y         # Used in routine setUpTrace for debug correlation with TEC

#------------------------------------------------------------------------
# Process command line parameters
#------------------------------------------------------------------------

# Default behaviour if keywords are not explicitly coded on command line
$status_chg         = 0;
$ee_interp_lookup   = 0;    # 0 means do not imbed %INTERP%     anywhere in the ihsttec command
$ee_hostname_kwd    = 0;    # 0 means do not imbed %HOSTNAME%   anywhere in the ihsttec command
$ee_fqhostname_kwd  = 0;    # 0 means do not imbed %FQHOSTNAME% anywhere in the ihsttec command

for ($parm_cnt = 0; $parm_cnt <= $#ARGV; $parm_cnt++) {
   $keyword = $ARGV[$parm_cnt];
   $_       = $keyword;
   tr/a-z/A-Z/;

   # Perl does not have a nice switch statement for us to use, but the perl
   # reference manual says the structure used below (as well as several other
   # versions) will be optimized internally to a nice switch structure.
   parms: {
      /^CHG$/         && do { $status_chg = 1                   ; last parms; };

      /^DM$/          && do { $event_type = $_                  ; last parms; };
      /^SENTRY$/      && do { $event_type = "DM"                ; last parms; };
      /^ITM$/         && do { $event_type = $_                  ; last parms; };
      /^ITMHB$/       && do { $event_type = $_                  ; last parms; };
      /^NETVIEW$/     && do { $event_type = $_                  ; last parms; };
      /^NVOLD$/       && do { $event_type = $_                  ; last parms; };
      /^HPOV$/        && do { $event_type = $_                  ; last parms; };

      /^INTERP$/      && do { $ee_interp_lookup  = 1            ; last parms; };
      /^HOSTNAME$/    && do { $ee_hostname_kwd   = 1            ; last parms; };
      /^FQHOSTNAME$/  && do { $ee_fqhostname_kwd = 1            ; last parms; };
      /^PROF$/        && do { $parm_cnt++;
                              $profile = $ARGV[$parm_cnt];
                              if ( !$profile) {
                                 &handleError ( 0x0040,
                                                "ERROR: Keyword \"$keyword\" not followed by profile name.\n",
                                                "abend" );
                              }
                                                                ; last parms; };

      # Only allow help requests if we are invoked from the command line.
      # That's because it makes no sense to code the help parameter
      # in the TEC rules, and if we have an isolated "H" parameter from
      # a TEC rule we want to issue the "Unknown command line parameter"
      # error message.
      if (!$ENV{TEC_BIN_DIR}) {
         # Accept parameter of h, -h, /h, ?, -?, or /? as a help request.
         /^H$/         && do { &usage; exit $RetCode; };
         /^-H$/        && do { &usage; exit $RetCode; };
         /^\/H$/       && do { &usage; exit $RetCode; };
         /^?$/         && do { &usage; exit $RetCode; };
         /^-?$/        && do { &usage; exit $RetCode; };
         /^\/?$/       && do { &usage; exit $RetCode; };
      }

      # If we get this far there is a command line parameter we do
      # not recognize. Send an appropriate message to STDOUT so that
      # if invoked from the TEC rules is will be visible from the
      # TEC console and if invoked from the command line it will be
      # visible in the command line window.
      &handleError ( 0x0040,
                     "ERROR: Unknown parameter \"$keyword\" passed to script.\n",
                     "abend" );

   } # end pseudo-case statement "parms:" that extracts parameters
}


#-------------------------------------------------------------------------
# If the script was called without a keyword telling us how to format
# the event, then we will try to make a best guess based on the environment.
#-------------------------------------------------------------------------
if ( $event_type eq "" ) {
   if ($source eq "SENTRY") {
      $event_type = "DM";

   } elsif ( $source eq "TMNT" ) {
      if ( substr($EVENT_CLASS, 0, 10) eq "HeartBeat_") { $event_type = "ITMHB" ; }
      else                                              { $event_type = "ITM"   ; }

   } elsif ( $EVENT_CLASS eq "TEC_ITS_NODE_STATUS" || $EVENT_CLASS eq "TEC_ITS_INTERFACE_STATUS" || $EVENT_CLASS eq "TEC_ITS_ROUTER_STATUS" ) {
      $event_type = "NETVIEW";

   } elsif ( $EVENT_CLASS eq "OV_Node_Down" ) {
      if    ( $source eq "NV6K")                        { $event_type = "NVOLD" ; }
      elsif ( $source eq "HPOV")                        { $event_type = "HPOV"  ; }

   }
}


# Trace $event_type since it dictates how the script handles the event
if ( $trace >= $trace_full ) {
   print TRACE_OUT "Event Type:             $event_type\n";
}

#-------------------------------------------------------------------------------
# Other environment variables we might need
#-------------------------------------------------------------------------------
$BINDIR             = $ENV{BINDIR};               # Note that TEC rules environment does not define BINDIR
$TEC_BIN_DIR        = $ENV{TEC_BIN_DIR};          # but does define TEC_BIN_DIR. EE needs one or the other.
$OS                 = $ENV{OS};                   # Used for Windows vs Unix EE executable location
$INTERP             = $ENV{INTERP};               # TEC interp type

#-------------------------------------------------------------------------------
# Set default values; initialize variables.
#-------------------------------------------------------------------------------
$RetCode            = 0;                          # Assume good return code
&setCmdPaths();                                   # Determine EE executable location and set BINDIR if necessary

#-------------------------------------------------------------------------------
# Do things common for all events
#-------------------------------------------------------------------------------

# Assume the hostname slot is the TCP hostname, and normalize it
# to help avoid event correlation issues due to variations
# in case and domain qualification. Typically we will always use the
# short TCP hostname in forwarded events unless the hostname slot
# was a dotted decimal. For that case we will use the full dotted
# decimal address in lieu of an actual short TCP hostname.
$tcp_hostname      =  $hostname_slot;
$tcp_hostname_orig =  $tcp_hostname;
$tcp_hostname      =~ tr/A-Z/a-z/;

if ( $tcp_hostname =~ /^[\d.]+$/ ) { # if all digits it is an ip address.
   $tcp_short  = $tcp_hostname;
   $ipaddr     = $tcp_hostname; # causes -q option instead of -h on ihstttec. EE resolves ipaddr to hostname if it can, uses ipaddr otherwise
#  $tcp_id     = "%HOSTNAME%";  # let TBSM EE fill in the instance id with the short TCP hostname
   $tcp_domain = "";
} else {
   ( $tcp_short, $tcp_domain ) =  split(/\./, $tcp_hostname, 2);
}

# The variable $tcp_id is used to hold a string representing the TCP hostname,
# and it is used when formatting events for TBSM. The default is to set it
# to the short TCP hostname where it has been converted to all lower case and
# had its domain qualification stripped off.
#
# If the script was invoked with one of the optional keywords HOSTNAME or
# FQHOSTNAME, then we will set $tcp_id to a keyword string that TBSM
# Event Enablement will replace after we forward the event to EE. This can be
# useful if the event has a dotted decimal address, but no TCP hostname, or
# if the event has a short TCP hostname when we want to use a domain qualified
# TCP hostname.
if ( $ee_hostname_kwd ) {
   $tcp_id = "%HOSTNAME%";
}
elsif ( $ee_fqhostname_kwd ) {
   $tcp_id = "%FQHOSTNAME%";
} elsif ( !$tcp_id) {
   $tcp_id = $tcp_short;
}

if ( $trace >= $trace_full ) {
   print TRACE_OUT "Original   Hostname:    $tcp_hostname_orig\n";
   print TRACE_OUT "Normalized Hostname:    $tcp_short\n";
   print TRACE_OUT "TCP Id String:          $tcp_id\n";
}

# Format the event and forward it to TBSM
&format_generic_event();

#-------------------------------------------------------------------------------
# Close trace file, reset variables and exit program.
#-------------------------------------------------------------------------------
if ( $trace > $trace_none) {
   close TRACE_OUT;
}

exit $RetCode;



#================================================================
#======================= end main program =======================
#================================================================



#-------------------------------------------------------------------------------
# Subroutine "format_generic_event" will format the event based on the
# keyword saved in variable $event_type.
#
# It is the instance id (-i option) that identifies the instance of the
# TBSM object class where the event will be posted. The TBSM object
# class is specified by the -b option.
#
# WARNING: For a given instance id, if the TCP hostname (-h option)
#          changes, then  TBSM will delete the old instance and create
#          it anew in the physical tree under the new TCP hostname.
#
#          While this is sort of cool it is potentially a very
#          bad thing from a practical standpoint because when the
#          original instance is deleted it will also be deleted
#          from all the business systems it was once in.
#
#          In TBSM case is respected, so that means the instance id
#          is case sensitive to.
#
# When TBSM posts an event to a given object the decision as to whether
# to update an existing exception under that object, or add a new
# exception under that object, is made by TBSM based on the name of the
# exception.
#
# If the name part of the new event matches the name field of an
# existing exception then that existing exception is updated and the
# count incremented.
#
# If the name part of the new event does not match the name field of
# an existing exception then the new exception is added as a new entry
# to the list of exceptions posted under that object.
#
# The following variables need to be set in this subroutine. Many have
# default values as noted, but not all of them do. There is a stanza
# in the following perl switch statement for each event family where
# some of these variables
# are adjusted:
#
#    $tcp_short           - TCP hostname passed to TBSM.
#    $prodname            - TBSM Product Name. Defaults to UnixSrv,
#                           WinSrv or NovSrv based on TCP hostname
#                           naming conventions.
#    $obj_class           - String to be passed on ihstttec -b option.
#    $instid              - Identifies the instance of the TBSM object
#                           class, and Unique across every instance of
#                           this object class. Defaults to short TCP
#                           hostname.
#    $inst_label          - Icon label to be used on GUI.
#    $alert_name          - Alert name seen on properties sheet of icon
#                           on TBSM. If there is an existing alert with
#                           an identical name then this alert will overlay
#                           the old one. The date, description, and repeat
#                           count are all updated.
#    $alert_desc          - Alert description seen on properties sheet
#                           of icon on TBSM.
#                           Carries supplemental information for the alert.
#    $severity            - Determines TBSM Alert State and Priority as
#                           defined by TBSM SQL table GEM_Severities.
#    $endpoint_os         - Can be used to set the object instance's
#                           parent object to NetworkUNIX, NTServer or
#                           NetworkNode. Typically this just confuses
#                           the alphabetical ordering on the physical
#                           tree, and we just let this default to 22.
#                           That sets the parent object to NetworkNode.
#
#    $ipaddr              - Optional dotted decimal address that can be
#                           used instead of $tcp_short.
#
# Note: When you add a new event family stanza remember to include the perl "last"
#       statement to cause the switch to exit, e.g.  "last etypes;"
#-------------------------------------------------------------------------------
sub format_generic_event {
   #----------------------------------------------------------------------------
   # Pick default values for some of the values passed to TBSM.
   # Typically some of these get updated in the "etypes" switch
   # stanza that corresponds to the event_type being processed.
   #----------------------------------------------------------------------------

   # Figure out default TBSM object class based on hostname
   # The optional second parameter specifies a default object class that will be used
   # if the hostname can not be decoded to determine the object class
   $prodname = &default_object_class ($tcp_short, "Server");

   $instid       = $tcp_id;     # $tcpid may have actual TCP hostname (short), or keyword "%HOSTNAME" or "%FQHOSTNAME%"

                                # Alert name and description are set to "unknown" rather than null or some slot value
                                # so that unexpected events will still result in an event being forwarded to TBSM
                                # rather than just discarded. If you change these defaults be sure to search this script
                                # for places where these variables are checked for the value "unknown"

   $alert_name   = "unknown";   # always replaced depending upon event type
   $alert_desc   = "unknown";   # always replaced depending upon event type


   # Uncomment the setting of "$endpoint_os" to 22 (default) to explicitly
   # pass an interp type of "default" to TBSM. That will ensure this object's
   # parent will always be "Network Node", and then all endpoints will be listed
   # on the physical tree in a single alphabetized list rather than split among
   # three alphabetized lists for "Network UNIX", "NT Server" and "Network Node".
   #$endpoint_os  = 22;

   $_ = $event_type;
   etypes:
   {
      #-------------------------------------------------------------------------
      # Event is a classic DM event
      #-------------------------------------------------------------------------
      /^DM$/ && do {
         if ( $monitor ) {
            $alert_name = $monitor . "(" . $probe_arg . ")";
         }
         else {
            $alert_name = $probe   . "(" . $probe_arg . ")";
         }

         if ( $value && $units ) {
            $alert_desc = $value . " " . $units;
         } elsif ($value) {
            $alert_desc = $value;
         } else {
            $alert_desc = $msg;
         }
         
         &set_obj_class_and_icon_label; # create values to pass TBSM on -b and -d parameters

         # The DM msg attribute may have many carriage returns
         # and line feeds. Change consecutive line feed(s)s to
         # a blank and delete carriage returns
         # $alert_desc =~ tr/\n\r/ /sd;

         last etypes;

      }; # end do DM

      #-------------------------------------------------------------------------
      # Event is an ITM event (non-heartbeat)
      #-------------------------------------------------------------------------
      /^ITM$/    && do {
         # Check to see if this is a PAC event. If it is, return
         # without forwarding to TBSM since PAC events are
         # forwarded by PAC-specific rules and scripts.
         #
         # Note: The following attributes appear in all PAC events
         # but not in non-PAC ITM events: application_version
         #                                application_oid
         #                                application_class
         #                                application_label
         if ($application_class) {
            &handleError ( 0x0080,
                           "ERROR: This is an ITM PAC event, and they are not forwarded by this script. application_class=\"$application_class\"\n",
                           "abend" );
         }

         # ITM clearing events have an event class of TMW_Clearing,
         # and the original event class in the "eventname" attribute.
         if ( $EVENT_CLASS eq "TMW_Clearing" ) {
            $EVENT_CLASS = $eventname;
            $severity    = "HARMLESS";
         }

         # ITM events save the ITM profilename in an event attribute
         # appropriately named "profilename". Unfortunately ITM appends
         # a "#" character, and then the region name.
         #
         # If the user wants to map ITM events to an object class via
         # some value that value would be passed on the PROF keyword.
         # That value could be the ITM profilename event attribute, or
         # any other value depending upon the customer environment.
         #
         # If the passed value is the actual profilename event attribute
         # then it will contain the actual profile name followed by a "#"
         # and then the region name. Here we will check for the actual 
         # profilename attribute, and if passed on the PROF keyword we will
         # strip off the trailing "#region_name".
         #
         # $profile      = value passed on the PROF keyword
         # $profilename  = actual profilename event attribute
         if ( $profile eq $profilename ) {
            ($profile, $region) = split ("#", $profilename, 2);
         }

         # Note: The following code was intended as a starting point for TMTP
         #       events forwarded via the ITM infrastructure. Unfortunately the 
         #       TMTP product dropped support for this after TMTP 5.1.1. 
         #
         #       Starting with TMTP 5.2 events are forwarded directly to TEC, and
         #       there are differences in event classes and event attribute names.
         #
         #       For TMTP 5.2 and beyond you need to add a new TEC rule (that 
         #       checks for source=TMTP, and then add stanza to this script
         #       to handle events of this type.

         # If this an ITM TMTP (5.1.1) event, then recompute the TBSM object class,
         # instance id, and icon label.
         if ( substr($EVENT_CLASS,0,11) eq "Ev_TMW_TMTP" ) {
            $tcp_short = "Transactions";
            $prodname  = "Tran";
            $instid    = $transactionname;

            if ($message) {
               $alert_name = $message;
            } else {
               $alert_name = $msg;
            }

            ($tmtp_userid, $rest) = split(/;/, $event_key, 2);  # take Userid=dddd from $event_key
            $alert_name = $alert_name . " @ " . $tcp_id . " " . $tmtp_userid;

            $alert_desc = "";
            &append_itm_slots_to_desc;   # Append non-standard ITM slots to the alert's description $alert_desc
         }

         # Build the TBSM exception name. The code here builds the 
         # exception name the same way it is has been historically built
         # if the ITM was forwarded with the ihstztec exit.
         #
         # Concatenate the TEC event class with "@" and the ITM resource
         # model name. Both are present in all ITM resource model events,
         # and this naming convention matches how the event is named in
         # common listener events (neglecting the inclusion of eventid in
         # common listener events.
         #
         # Append to that a single space and the "event_key" slot after
         # possibly adjusting the event_key slot. ITM uses the "event_key"
         # slot when a resource model needs to identify a particular
         # resource instance.  The event class may prefix the event_key,
         # if so then the event class is removed from the event_key prior
         # to appending it.

         if ( $alert_name eq "unknown" ) {
            $alert_name = $EVENT_CLASS . "@" . $modelname;

            $event_key_adj =  $event_key;            # keep original event_key slot in case we want it for messages
            $event_key_adj =~ s/^$EVENT_CLASS//;     # remove leading TEC event class name if present
            $event_key_adj =~ s/^\|//;               # remove leading | if present

            if ( $event_key_adj ) {
               $alert_name = $alert_name . " " . $event_key_adj;
            }
         }

         # Build the TBSM exception description. This is how TBSM 3.1 EE
         # does it for ITM events (source="TMNT") if you do not pass
         # an exception name (-p option) to ihstttec. We explicitly
         # do it here instead so that TBSM 2.1.1 and before will see
         # the same exception name.
         if ( $alert_desc eq "unknown" ) {
            if ($message) {
               $alert_desc = $message;
            } else {
               $alert_desc = $msg;
            }

            # Append non-standard ITM slots to the alert's description $alert_desc
            &append_itm_slots_to_desc;
         }

         &set_obj_class_and_icon_label; # create values to pass TBSM on -b and -d parameters
         
         last etypes;

      }; # end do ITM

      #-------------------------------------------------------------------------
      # Event is an ITM HeartBeat event
      #-------------------------------------------------------------------------
      /^ITMHB$/  && do {

         $alert_name = $EVENT_CLASS;
         $alert_desc = $msg;
         
         &set_obj_class_and_icon_label; # create values to pass TBSM on -b and -d parameters
         
         last etypes;

      }; # end do ITMHB

      #-------------------------------------------------------------------------
      # Event is a new NetView event using the new TEC classes TEC_ITS_*
      #-------------------------------------------------------------------------
      /^NETVIEW$/  && do {

         # Set up a hash table to convert netview numeric status enumeration to a string
         if ( $EVENT_CLASS eq "TEC_ITS_NODE_STATUS" ) {

            $nv_status      = $nodestatus;
            %nv_status_hash = (1, "UP"        ,
                               2, "DOWN"      ,
                               3, "MARGINAL",
                               4, "UNREACHABLE");

         } elsif ($EVENT_CLASS eq "TEC_ITS_INTERFACE_STATUS") {

            $nv_status = $ifstatus;
            %nv_status_hash = (1, "UP"        ,
                               2, "DOWN"      ,
                               3, "ADMIN_DOWN",
                               4, "UNREACHABLE");

         } elsif ( $EVENT_CLASS eq "TEC_ITS_ROUTER_STATUS" ) {

            $nv_status = $routerstatus;
            %nv_status_hash = (1, "UP"        ,
                               2, "DOWN"      ,
                               3, "MARGINAL",
                               4, "UNREACHABLE");
         }

         # Get a string value that corresponds to the numeric status enumeration in the TEC slot
         if ( $nv_status =~ /^[1-4]$/ ) {
            $nv_status_str = $nv_status_hash{$nv_status-1};
         } else {
            $nv_status_str = $nv_status;
         }

         # Explicitly assign TBSM severity rather than relying on TEC severity assigned by NetView
         # and default mapping of TEC severity to TBSM severity in TBSM table GEM_Severities
         if (      $nv_status eq "UP"       || $nv_status eq "1" ) {
            $severity = "HARMLESS";
         } elsif ( $nv_status eq "DOWN"     || $nv_status eq "2" ) {
            $severity = "CRITICAL";
         } elsif ( $nv_status eq "MARGINAL" || $nv_status eq "3" ) {
            $severity = "WARNING";
         } else {
            $severity = "WARNING";
         }

         # Set up alert name and description using the string equivalend of the status enumeration
         if ( $EVENT_CLASS eq "TEC_ITS_NODE_STATUS" ) {
            $alert_name = "Netview node status";
            $alert_desc = "Host $tcp_short is $nv_status_str";
         } elsif ($EVENT_CLASS eq "TEC_ITS_INTERFACE_STATUS") {
            $alert_name = "Netview interface status";
            if ($ifname) {
               $alert_name = $alert_name . " for $ifname";
            }
            $alert_desc = "Host $tcp_short interface $ifname is $nv_status_str";
         } elsif ( $EVENT_CLASS eq "TEC_ITS_ROUTER_STATUS" ) {
            $alert_name = "Netview router status";
            $alert_desc = "Router $tcp_short is $nv_status_str";
         }
         
         &set_obj_class_and_icon_label; # create values to pass TBSM on -b and -d parameters
         
         last etypes;

      }; # end do NETVIEW

      #-------------------------------------------------------------------------
      # Event is an old NetView event using the old TEC class OV_Node_Down
      #-------------------------------------------------------------------------
      /^NVOLD$/  && do {

         # Check for the three expected OV_status values and set the
         # severity accordingly. We do this to avoid HP OpenView's penchant for
         # using every severity in the book.
         if ( $OV_status eq "Normal" ) {
            $severity = "HARMLESS";
         } elsif ( $OV_status eq "Marginal" ) {
            $severity = "WARNING";
         } elsif ( $OV_status eq "Down" ) {
            $severity = "CRITICAL";
         }

         $alert_name = $EVENT_CLASS;

         $OV_status  =~tr/A-Z/a-z/;
         $alert_desc = "$msg Node $tcp_short ($origin) is $OV_status.";

         &set_obj_class_and_icon_label; # create values to pass TBSM on -b and -d parameters
         
         last etypes;

      }; # end do NVOLD

      #-------------------------------------------------------------------------
      # Event is an HP OpenView event
      #-------------------------------------------------------------------------
      /^HPOV$/  && do {

         # Check for the three expected OV_status values and set the
         # severity accordingly. We do this to avoid HP OpenView's penchant for
         # using every severity in the book.
         if ( $OV_status eq "Normal" ) {
            $severity = "HARMLESS";
         } elsif ( $OV_status eq "Marginal" ) {
            $severity = "WARNING";
         } elsif ( $OV_status eq "Down" ) {
            $severity = "CRITICAL";
         }

         $alert_name = $EVENT_CLASS;

         $OV_status  =~tr/A-Z/a-z/;
         $alert_desc = "$msg Node $tcp_short ($origin) is $OV_status.";

         &set_obj_class_and_icon_label; # create values to pass TBSM on -b and -d parameters
         
         last etypes;

      }; # end do HPOV

      #-------------------------------------------------------------------------
      # No matches found, do not send event to TBSM
      #-------------------------------------------------------------------------

      $errmsg = "ERROR: Un-implemented event type: \"$event_type\".\n";
      print ($errmsg);

      if ( $trace >= $trace_error ) {
         print TRACE_OUT ($errmsg);
      }

      return; # return prematurely since we couldn't figure out how to handle the message

   } # end etypes switch

   &sendGenericEventToTBSM(); # send the event to TBSM

} # end sub format_generic_event



#-------------------------------------------------------------------------------
# Append non-standard ITM slots to the alert description $alert_desc.
#-------------------------------------------------------------------------------
sub append_itm_slots_to_desc {
   # The SLOTS environment variable contains a blank delimited string
   # naming each of the slots in the event. We will scan it for unique
   # slots names not typically found in ITM events, and for each one found
   # append to the TBSM event description a new line, the slot name,
   # an "=" sign and the slot value.

   @slots = split(' ', $SLOTS); # Convert to array. Leading, trailing, and consecutive blanks ignored

   # These are the slots common to all ITM events
   %root_slots = ("acl",1,"adapter_host",1,"administrator",1,"cause_date_reception",1,
                  "cause_event_handle",1,"credibility",1,"date",1,"date_reception",1,
                  "duration",1,"event_handle",1,"event_key",1,"eventid",1,"eventname",1,
                  "hostname",1,"modelname",1,"msg",1,"msg_catalog",1,"msg_index",1,
                  "num_actions",1,"origin",1,"profilename",1,"repeat_count",1,
                  "server_handle",1,"server_path",1,"severity",1,"slotkey",1,"source",1,
                  "status",1,"sub_origin",1,"sub_source",1,"tmr",1);

   foreach $slotname (@slots) {                 # check each slot name
      if ( !($root_slots{$slotname}) ) {        # if it is not a common slot name
         $evalstr = '$ENV{' . $slotname . '}';  # build the string to get the slot value
         $slotval = eval ($evalstr);            # build a slot name=value pair

         $alert_desc = $alert_desc . "\n$slotname=$slotval";
      } last if (length($alert_desc) > 1023);
   }
} # end sub append_itm_slots_to_desc



#-------------------------------------------------------------------------------
# Send Generic Event to TBSM
#-------------------------------------------------------------------------------
sub sendGenericEventToTBSM {
   
   # If the script variable $closed_events_green_cfg has been set to enable it
   # status change events with a new status of CLOSED will have the severity
   # forced to HARMLESS. This allows us to avoid the default behaviour of deleting
   # the old event, and instead changing it to a green event in TBSM.
   if ( $closed_events_green_cfg && $status_chg && ($status eq "CLOSED") ) {
      $severity   = "HARMLESS";
      $status_chg = 0;
      $alert_desc = "Closed on TEC: " . $alert_desc;
   }

   # Build command. Always code the alert description last (-m option) because if something
   # gets truncated due to a long command line, that the most benign thing.
   $ee_cmd                         = "$GenCmd"                             ;
   if    ($obj_class   ) { $ee_cmd = $ee_cmd." -b '$obj_class'"            ; }
   if    ($instid      ) { $ee_cmd = $ee_cmd." -i '$instid'"               ; }
   if    ($alert_name  ) { $ee_cmd = $ee_cmd." -p '$alert_name'"           ; }
   if    ($severity    ) { $ee_cmd = $ee_cmd." -s '$severity'"             ; }
   if    ($status_chg  ) { $ee_cmd = $ee_cmd." -n "                        ; }
   if    ($inst_label  ) { $ee_cmd = $ee_cmd." -d '$inst_label' "          ; }
   if    ($endpoint_os ) { $ee_cmd = $ee_cmd." -o '$endpoint_os' "         ; }
   if    ($data1       ) { $ee_cmd = $ee_cmd." -1 '$data1' "               ; }
   if    ($data2       ) { $ee_cmd = $ee_cmd." -2 '$data2' "               ; }
   if    ($data3       ) { $ee_cmd = $ee_cmd." -3 '$data3' "               ; }
   if    ($data4       ) { $ee_cmd = $ee_cmd." -4 '$data4' "               ; }

   if    ($ipaddr      ) { $ee_cmd = $ee_cmd." -q '$ipaddr' "              ; }
   elsif ($tcp_short   ) { $ee_cmd = $ee_cmd." -h '$tcp_short' "           ; }

   if    ($eever >= 310) { $ee_cmd = $ee_cmd." -a '0' "                    ; } # do not append @hostname
   
   if    ($alert_desc  ) { $ee_cmd = $ee_cmd." -m '$alert_desc'"           ; }

   # If any required parameters are NULL don't send the event to tbsm
   $parm_err      = 0;
   $parm_err_desc = "";
   if ( !$obj_class ) {
      $parm_err++;
      $parm_err_desc = $parm_err_desc . " -b";
   }

   if ( !$instid ) {
      $parm_err++;
      $parm_err_desc = $parm_err_desc . " -i";
   }

   if ( !$alert_name ) {
      $parm_err++;
      $parm_err_desc = $parm_err_desc . " -p";
   }

   if ( !$tcp_id && !$ipaddr ) {
      $parm_err++;
      $parm_err_desc = $parm_err_desc . " (both -h and -q)";
   }

   if ( !$alert_desc ) {
      $parm_err++;
      $parm_err_desc = $parm_err_desc . " -m";
   }

   if ( !$severity ) {
      $parm_err++;
      $parm_err_desc = $parm_err_desc . " -s";
   }

   if ( $parm_err ) {
      &handleError ( 0x0010,
                     "ERROR   : Malformed EE command not sent to TBSM. Null value(s) for required ihstttec option(s):" . $parm_err_desc . ".   EventType=$event_type Cmd: $ee_cmd \n",
                     "abend");
   } else {
      # Required parameters are non-null, so send the event to tbsm

      print ("$ee_cmd\n"); # Make command visible from TEC console

      # If we are tracing each event, then put the command in the trace file
      # Note: If trace is set to limited (i.e. one trace line per event).
      #       then STDOUT already has pending output waiting for a \n linefeed.
      if ( $trace >= $trace_limited) {
         if ( $trace >= $trace_full) {
            print TRACE_OUT ("Date: $date   EventClass: $EVENT_CLASS   EventType: $event_type   TCP Short Hostname: $tcp_short\n");
         }
         $| = 1; # Flush TRACE_OUT. The NT TEC seems to take forever otherwise.
         print TRACE_OUT ("$ee_cmd\n");
      }

      # Make trace status visible from TEC console or if command line testing.
      if ( $trace >= $trace_limited) {
         print STDOUT ("Trace level $trace: $trace_fq\n");
      }

      # If debug mode then display message on TEC console or if testing on STDOUT.
      if ($debug) {
         print STDOUT ("Debug mode enabled: Event not sent to TBSM.\n");
      } else {
         # Execute the command and capture the rc.
         # The command's actual return code is above the first 2 bytes.
         `$ee_cmd`;
         $rc     = $? >> 8;
         $rc_str = sprintf("0x%04x", $rc),

         # Display on TEC console the rc for this command. If multiple commands
         # are sent this will show individual command rc values as opposed to
         # the overall RetCode rc returned by this script and automatically
         # displayed by the TEC console.
         print STDOUT ("rc = $rc_str\n");

         # Check for bad return code from ihstttec
         if ($rc != 0) {
            if ( $trace >= $trace_error) {
               &handleError ( $rc << 8,
                              "ERROR: rc = $rc_str returned from ihstttec.\n",
                              "abend");
            }
         }

      } # end else we are not in debug mode

   } # end else required parameters are non-null

} # end sub sendGenericEventToTBSM



#-------------------------------------------------------------------------------
# Subroutine "handleError":
#    1. Updates the return code
#    2. Prints the error message to STDOUT so it can be seen on the TEC Console
#       and when testing from the command line.
#    3. Prints the error message to the trace file if tracing.
#-------------------------------------------------------------------------------
sub handleError {
      local ($rc, $errmsg, $abend) = @_;
      $RetCode = $RetCode | $rc;

      print ($errmsg);   # Visible from TEC console and cmd line testing

      # $trace_str gets set up in subroutine setUpTrace with the local date,
      # Some values to correlate the event with TEC listings, and the TEC event class.
      # We don't print it to the trace file at that time if we are only tracing upon errors.
      # Even if we are tracing at a higher level we don't print a trailing \n so that we can add
      # more to the trace line for the case of tracing one line per event.
      if ( $trace == $trace_error ) {
         print TRACE_OUT ("$trace_str\n");
      } elsif ( $trace == $trace_limited ) {
         print TRACE_OUT ("\n");
      } 
         
      if ( $trace > $trace_none ) {
         print TRACE_OUT ($errmsg);
      }

      if ( $abend eq "abend" ) {

         if ( $trace > $trace_none) {
            close TRACE_OUT;
         }

         exit $RetCode;
      }
} # end sub handleError



#-------------------------------------------------------------------------------
# Subroutine "setUpTrace" determines:
#
#    1, Location of trace file and creates directory if necessary
#    2. Trace file name
#-------------------------------------------------------------------------------
sub setUpTrace {
   # If a trace directory is not specified we need to default it.
   if ( $trace_dir eq "" ) {
      # Check for the following possible temporary directories, and use
      # the first one found.
      $tmp_dir = "/tmp";
      if (! -d $tmp_dir ) {
         $tmp_dir = "/temp";
         if (! -d $tmp_dir ) {
            $tmp_dir = "/";
         }
      }

      $trace_dir = $tmp_dir;

      # Enable this code if you prefer to default to saving the trace
      # file to the directory pointed to by Tivoli framework's wtemp cmd.
      #$tmpdir = `wtemp 2>/dev/null`;  # send STDERR to null to avoid "not found" error when run on non-managed node
      #if ($tmpdir ne "") {
      #   chop $tmpdir;
      #   $trace_dir = $tmpdir;
      #} else {
      #   $trace_dir = "/";
      #}
   }

   # Make sure the trace directory exists, and create it if necessary
   if ( (! -d $trace_dir) && ( system ("mkdir -p $trace_dir") ) )
   {
     warn "Cannot create directory '$trace_dir'. Defaulting to root directory.\n";
     $trace_dir = "/";
     $RetCode = $RetCode | 0x0001;
   }

   # Set the fully qualified name of the trace file
   if ( substr($trace_dir, -1, 1) ne "/" ) {
      $trace_dir = $trace_dir . "/";
   }
   $trace_fq  = $trace_dir . $trace_fn;

   # We overwrite or append the trace file as appropriate
   if ( $trace_append == 0 || -s $trace_fq > $trace_max ) {
      open(TRACE_OUT, ">  $trace_fq");
   } else {
      open(TRACE_OUT, ">> $trace_fq");
   }

   # Get these values before the other environment variables so that they can be used 
   # early in the trace to correlate the trace entry with TEC wtdumprl and wtdumptr listings
   $EVENT_CLASS    = $ENV{EVENT_CLASS};    
   $server_handle  = $ENV{server_handle};  
   $date_reception = $ENV{date_reception}; 
   $event_handle   = $ENV{event_handle};   
   
   # When run from the command line, use current time.
   if (!$date_reception) {
     $date_reception = time;
   }

   # To correlate this script's trace entries with event stanzas in
   # TEC wtdumprl and wtdumptr listings we will extract from this 
   # event the following 3 values:
   #    server_handle
   #    event_handle
   #    date_reception
   #
   # These 3 values are guaranteed by TEC to uniquely identify an event in TEC.
   #
   # In the wtdumtr listing the correlation is easy because all 3 values are eplicitly 
   # identified on 3 lines in each event's stanza.
   #
   # It is a little more complicated in a wtdumprl listing because the three values
   # are not explicitly identified. in particular the event handle is merged with 
   # an internal TEC id that is not available to this script.
   #
   # Using this example line from the beginning on an event's stanza in a wtdumprl listing:
   #    1~823372~9~1073650509(Jan 09 13:15:09 2004)
   #
   # This is ho to interpret the fields:
   #    server_handle  is the first of the four numeric fields
   #    event_handle   is the third of the four numeric fields, last 16 bits (we think)
   #                         (third of the four numeric fields, first 16 bits is internal event id we think)
   #    date_reception is the last  of the four numeric fields
   # 
   
   # Include a formated version of $date_reception, formatted as it is in wtdumprl
   ($sec,$min,$hour,$dom,$mon,$year,$wday,$yday,$isdst) = localtime($date_reception);
   $mon_str = (Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec)[$mon];

   # if ($hour < 12 ) {
   #    $ampm="AM";
   # } else {
   #   $ampm="PM";
   #    if ($hour > 12) {
   #       $hour = $hour - 12;
   #    }
   # }

   $tec_correlate_str = sprintf("%d~%d~%d(%s %02d %02d:%02d:%02d %04d)", $server_handle, $event_handle, $date_reception, $mon_str, $dom, $hour, $min, $sec, $year+1900 );

   # Depending upon how verbose our tracing is we may or may not
   # add the following to the trace file:
   #
   # 1. Header line demarcating beginning of trace file stanza for the current event.
   # 2. Date on separate line unless we are performing minimal tracing at one line per event.
   # 3. Copy of all the environment variables. this is very verbose, but also very useful
   #    during development since this script gets event attribute information via
   #    environment variables.

   if ( $trace >= $trace_full ) {
      print TRACE_OUT "\n\n======================== Start Event ==========================\n";
   }

   $trace_str = "$date: TEC: $tec_correlate_str $EVENT_CLASS: "; # no \n line feed in case we are at trace level 1 with 1 line per normal event

   if ( $trace >= $trace_limited ) {
      print TRACE_OUT $trace_str;                                # no \n line feed in case we are at trace level $trace_limited meaning 1 trace line per normal event
   }

   if ( $trace >= $trace_full ) {
      print TRACE_OUT "\n";
   }

   if ( $trace >= $trace_full_env ) {
      &traceEnv("Initial");
   }
} # end sub setUpTrace



#-------------------------------------------------------------------------------
# Subroutine "traceEnv" traces environment variable settings.
#-------------------------------------------------------------------------------
sub traceEnv {
   $env_trace_header = $_[0];
   print TRACE_OUT "\n====== Environment Variables: $env_trace_header\n";
   $env=`env`;
   print TRACE_OUT "$env\n";
} # end sub traceEnv



#-------------------------------------------------------------------------------
# Subroutine "setCmdPaths determines where the TBSM Event Enablement executables
# are located and qualifies the event forwarding commands with a path prefix.
#
# Not only must we figure out where the Event Enablement binaries are installed,
# but we must also ensure that either BINDIR or TEC_BIN_DIR is defined because
# the event enablement binaries require that one or the other of these environment
# variables be defined.
#
# This code allows the script to work with the TBSM Event Enablement instance
# located on any of these possible places:   Unix TEC
#                                            Windows TEC
#                                            TBSM database server
#
# This code allows the script to be invoked from the TEC rules or manually from
# the command line on TEC or on the TBSM database server.
#-------------------------------------------------------------------------------
sub setCmdPaths {
   if ( $trace >= $trace_setup ) {
      print TRACE_OUT "BINDIR      from environment: $BINDIR\n";
      print TRACE_OUT "TEC_BIN_DIR from environment: $TEC_BIN_DIR\n";
   }

   if ( ! $BINDIR && $TEC_BIN_DIR ) {
      $TEC_BIN_DIR =~ tr/\\/\//;              # Windows TEC_BIN_DIR has backslashes
      $BINDIR      = "$TEC_BIN_DIR/../..";
   }

   # If BINDIR is still null then we must be getting invoked manually and the user
   # did not go to the trouble to ensure the environment was set up. If we are
   # running on a Unix box it will be in the standard place for a Unix Tivoli
   # Managed Node (TEC). On Windows it will be either in the standard place for a
   # Windows Tivoli Managed Node (TEC), or in the standard place for a TBSM
   # database server.
   #
   # The code below copies the expected environment setup file to $setenv,
   # and then extracts the line where BINDIR is defined.
   if (! $BINDIR) {
      # Windows managed node or TBSM database server
      if ($OS eq "Windows_NT") {
         $setenv_fn="C:/WINNT/system32/drivers/etc/Tivoli/setup_env.sh";
         if (-e $setenv_fn) {
            # windows managed node
            $setenv=`cat $setenv_fn | grep BINDIR=`;
         } else {
            $setenv_fn="C:/WINNT/system32/drivers/etc/Tivoli/setup_tbsm.sh";
            if (-e $setenv_fn) {
               # tbsm db server
               $setenv=`cat $setenv_fn | grep BINDIR=`;
            }
         }
      } else {
         # unix managed node
         $setenv=`cat /etc/Tivoli/setup_env.sh | grep BINDIR=`;
      }

      ($junk,$BINDIR) = split (/\=/, "$setenv");
      chop($BINDIR);
   } # end if (! $BINDIR)

   if ( $trace >= $trace_setup ) {
      print TRACE_OUT "BINDIR      after adjustment: $BINDIR\n";
   }

   if ("$BINDIR") {
      $ENV{BINDIR} = "$BINDIR";                              # make newly defined BINDIR available to EE in case we are on tbsm db server

      if ($OS eq "Windows_NT") {
         $DMCmd  = "ihstztec.exe";                           # command for sending event to tbsm as tbsm DM event
         $GenCmd = "ihstttec.exe";                           # command for sending event to tbsm as tbsm generic event
         $MWCmd  = "gtmmaint.sh";                            # command for sending event to tbsm as tbsm generic event
      } else {
         $DMCmd  = "ihstztec";                               # command for sending event to tbsm as tbsm DM event
         $GenCmd = "ihstttec";                               # command for sending event to tbsm as tbsm generic event
         $MWCmd  = "gtmmaint";                               # command for sending event to tbsm as tbsm generic event
      }

      $DMCmd  = $BINDIR . "/TDS/EventService/"     . $DMCmd; # command for sending event to tbsm as tbsm DM event
      $GenCmd = $BINDIR . "/TDS/EventService/"     . $GenCmd;# command for sending event to tbsm as tbsm generic event
      $MWCmd  = $BINDIR . "/TDS/EventService/bin/" . $MWCmd; # command for sending event to tbsm as tbsm generic event

   } else {
      &handleError ( 0x0020,
                     "ERROR: Could not figure out how to set BINDIR. Abending.\n",
                     "abend");
   }
} # end sub setCmdPaths


#-------------------------------------------------------------------------------
# Subroutine "default_object_class" will examine the passed hostname and 
# return a TBSM object name based on a decoding of the object class.
# For customers with strong naming conventions this can be an ideal
# way to determine TBSM object class.
#
# The naming conventions shown here are for example only. The customer
# will have to adjust the code in this routine to reflect their naming
# conventions.
#-------------------------------------------------------------------------------
sub default_object_class {
   local ($host, $default_obj) = @_;
   local ($obj_class);

   # If the optional second parameter was not supplies to specify default object class
   # assign a default here so that we can return a value even if the hostname can not be decoded.
   if ( $default_obj eq "" ) {
      $default_obj = "Unknown";
   }

   $obj_class = $default_obj;
   
   # Use naming conventions to determine object class.
   #
   # This is intended as a simplistic example of how TCP hostname naming conventions 
   # might be used to determine TBSM object class. In a real environment the logic here
   # would typically be more complex.

   # if (      substr($tcp_short, 0, 1) eq "u"  ) {
   #    $prodname = "UnixSrv";
   # } elsif ( substr($tcp_short, 0, 1) eq "w"  ) {
   #    $prodname = "WinSrv";
   # } elsif ( substr($tcp_short, 0, 2) eq "nw" ) {
   #    $prodname = "NovSrv";
   # } elsif ( substr($tcp_short, 0, 1) eq "r"  ) {
   #    $prodname = "Router";
   # } elsif ( substr($tcp_short, 0, 1) eq "h"  ) {
   #    $prodname = "Hub";
   # } elsif ( substr($tcp_short, 0, 1) eq "s"  ) {
   #    $prodname = "Switch";
   # } else {
   #    $prodname = "Server";
   # }

   return "$obj_class";
} # end sub default_object_class


#-------------------------------------------------------------------------
# Subroutine set_obj_class_and_icon_label is used to determine the 
# values passed to TBSM on the -b <obj_class> and -d <icon_label> parameters.
#
# By the time this routine is called the variable $prodname is typically 
# set to the expected object class for the event source being processed.
# Typically this default value was determined in routine default_object_class.
#
# If the TEC rule specified a keyword such as PROF or INTERP then the 
# default object class will be overridden in this routine.
#
# Examples of what can be passed to the -b variable include:
#
#    TBSM 2.1.1 or older:
#       ProdName;ProdVer
#       CID
#       cname;
#
#   TBSM 3.1 or newer:
#       Any of the previous possibilities
#       ProdName
#       Any name, such as a profile name, configured at the TBSM 
#          database server to map to the actual Prodname
#       It is also legal to imbedd the EE substitution variables 
#          "%INTERP%", "%HOSTNAME%" or "%FQHOSTNAME%" in the 
#          ihstttec -b value for subsequent substitution by EE.
#
# The icon label is also set in this routine, and the string used depends
# upon the setting of this script's constant variable $eever. For TBSM 3.1 or
# newer this script will not use the optional ihstttec paramater for the icon label.
# That means TBSM will default to using the instance id for the icon lable. This
# script uses some form of the TCP hostname for the instance id.
#
# The expectation is that for TBSM 3.1 
# and beyond that TBSM Event Enablement will be configured to suppress the 
# automatic addition of the @tcphostname suffix to all distributed labels.
#
# The labels are optimized for a TBSM console view setting of medium or short 
# icon labels. This is true regardless of the $eever setting. For TBSM 3.1 
# or newer the expectation is that TBSM will be configured to not automatically 
# append the TCP hostname to icon labels.
#-------------------------------------------------------------------------
sub set_obj_class_and_icon_label {
   if ($profile) {

      $obj_class  = "$profile";                # ihstttec -b option: profile name (mapped by TBSM SQL to ProdName)
                                               
      if ($eever < 310) {
         $inst_label = "PROF Illegal";         # For testing purposes. PROF keyword is not supported until TBSM 3.1
      }

   } elsif ($ee_interp_lookup) {

      $obj_class  = "MAP_%INTERP%_SERVER";     # ihstttec -b option: string to be mapped at TBSM to the appropraite ProdName
                                               # TBSM database pre-loaded to recognize MAP_solaris2_SERVER, MAP_win32-ix86_SERVER, etc
      if ($eever < 310) {
         $inst_label = "INTERP Illegal";       # For testing purposes. INTERP keyword is not supported until TBSM 3.1.
      }

   } elsif ($prodver) {

      $obj_class  = $prodname .";" . $prodver; # ihstttec -b option: traditional product name and product version
                                               
      if ($eever < 310) {
         $inst_label = $prodname;              # Icon label: ProdName@hostname
      }

   } else {

      $obj_class  = $prodname;

      if ($eever < 310) {
         $inst_label = $prodname;              # Icon label: ProdName@hostname
      }

   } # end 4 stanza if - elsif for determining $obj_class and $inst_label

} # end sub set_obj_class_and_icon_label

