#!/bin/sh
#**************************************************************** {COPYRIGHT-TOP-RM} ***
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#**************************************************************** {COPYRIGHT-END-RM} ***
#***********************************************************************
#
# Component Name: TGM task server
#
#***********************************************************************
#
# Package problem determination information
#
# NOTE: rc(5902) indicates that the etc/Tivoli/setup_env.sh file could
#       not be found.  Therefore the Tivoli environment could not be
#       established.
#***********************************************************************
TGM_ROOT="/TDS/EventService"
SETUPTBSM="setup_tbsm.sh"
SETUPTIV="setup_env.sh"

usage()
{
   "$BINDIR$TGM_ROOT/bin/usage.sh" "getpd" $TDS_OS
}

#
#Setup the environment
#
stopIt="0"

if [ -n "$TSERVER_ETC" ] ; then
  if [ -n "$windir" ] ; then
    if [ -z "$T_SERVT" ] ; then
      if [ -f "$TSERVER_ETC/$SETUPTBSM" ] ; then
        . "$TSERVER_ETC/$SETUPTBSM"
      elif [ -f "$TSERVER_ETC/$SETUPTIV" ] ; then
        . "$TSERVER_ETC/$SETUPTIV"
      else
        stopIt="1"
      fi
    else
      if [ "$T_SERVT" = "TW" ] ; then
        sPath=`egrep "^set BINDIR=" $windir/system32/drivers/etc/Tivoli/setup_tbsm.cmd | sed -e 's/^set BINDIR=//' | sed -e 's/\\\\/\//g'`
        export BINDIR="$sPath"
      fi
    fi
    TDS_OS="PC"
  else
    if [ -z "$T_SERVT" ] ; then
      if [ -f "$TSERVER_ETC/$SETUPTBSM" ] ; then
        . "$TSERVER_ETC/$SETUPTBSM"
      elif [ -f "$TSERVER_ETC/$SETUPTIV" ] ; then
        . "$TSERVER_ETC/$SETUPTIV"
      else
        stopIt="1"
      fi
    fi
    TDS_OS="RISC"
  fi
else
  if [ -n "$windir" ] ; then
    WINOS=`echo "$windir" | awk /:/'{ print substr($1,4) }'`
    WINDRIVE=`echo "$windir" | awk /:/'{ print substr($1,1,2) }'`
    if [ -n "$WINOS" ] ; then
      if [ -z "$T_SERVT" ] ; then
        if [ -f "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/$SETUPTBSM" ] ; then
          . "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/$SETUPTBSM"
        elif [ -f "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/$SETUPTIV" ] ; then
          . "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/$SETUPTIV"
        else
          stopIt="1"
        fi
      else
        if [ "$T_SERVT" = "TW" ] ; then
          sPath=`egrep "^set BINDIR=" $windir/system32/drivers/etc/Tivoli/setup_tbsm.cmd | sed -e 's/^set BINDIR=//' | sed -e 's/\\\\/\//g'`
          export BINDIR="$sPath"
        fi
      fi
      TDS_OS="PC"
    else
      stopIt="1"
    fi
  else
    if [ -z "$T_SERVT" ] ; then
      if   [ -f "/etc/Tivoli/$SETUPTBSM" ] ; then
        . "/etc/Tivoli/$SETUPTBSM"
      elif [ -f "/etc/Tivoli/$SETUPTIV" ] ; then
        . "/etc/Tivoli/$SETUPTIV"
      else
        stopIt="1"
      fi
    fi
    TDS_OS="RISC"
  fi
fi

if [ "$stopIt" = "1" ] ; then
  echo "rc(5902) setup_env.sh not found"
  exit 1
fi

if [ -z "$T_SERVP" ] ; then
  #
  # Set NLSPATH
  #
  if [ -n "$NLSPATH" ] ; then
    NLSPATH="$NLSPATH:$BINDIR$TGM_ROOT/msg_cat/%L/%N.cat"
  else
    NLSPATH="$BINDIR$TGM_ROOT/msg_cat/%L/%N.cat"
  fi
  export NLSPATH

  #
  # Set PATH
  #
  PATH="$PATH:$BINDIR$TGM_ROOT/bin"
  export PATH
fi

#
# Set PATH
#
if [ "`uname`" = "SunOS" ] ; then
  PATH="/bin:/usr/bin:/sbin:/usr/sbin:/usr/ucb:/usr/dt/bin:$PATH"
fi

#***********************************************************************
# You must be root inorder to start the TDS Server daemons
#***********************************************************************
if [ $TDS_OS = "RISC" ] ; then
  if [ "`whoami`" != "root" ]
   then
    command "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5901
    exit 1
   fi
fi

TGM_TEMP="$BINDIR$TGM_ROOT/tmp"

if [ "$1" = "" ] ; then
  echo "---------------------------------------------"
else
  usage
  exit 1
fi

# Show wait message
command "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5948

MY_OS=`"uname"`

# set error code
rc=0

# make a temporary directory for file storage
if [ ! -d "$TGM_TEMP" ] ; then
  mkdir "$TGM_TEMP" 2> /dev/null
  RC=$?
  if [ $RC -ne 0 ] ; then
    command "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5949
    echo "$TGM_TEMP"
    exit 1
  fi
else
  rm -fr "$TGM_TEMP/*"
  ## */
  RC=$?
  if [ $RC -ne 0 ] ; then
    command "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5850
    echo "$TGM_TEMP"
    exit 1
  fi
fi

#
# retrieve system unique data
#
if [ $MY_OS = "AIX" ] ; then
  lsps -a > "$BINDIR$TGM_ROOT/log/lsps.a"
  ps -ef > "$BINDIR$TGM_ROOT/log/psef.out"
  lsdev -C > "$BINDIR$TGM_ROOT/log/lsdev.C"
  df  > "$BINDIR$TGM_ROOT/log/df.out"
  set > "$BINDIR$TGM_ROOT/log/set.out"
  errpt '-a' > "$BINDIR$TGM_ROOT/log/errpt.out"
  cp -pr /var/adm/ras/errlog "$BINDIR$TGM_ROOT/log"  2> /dev/null
elif [ $MY_OS = "SunOS" ] ; then
  ps -ef > "$BINDIR$TGM_ROOT/log/psef.out"
  df  > "$BINDIR$TGM_ROOT/log/df.out"
  set > "$BINDIR$TGM_ROOT/log/set.out"
elif [ $MY_OS = "Windows_NT" ] ; then
  set > "$BINDIR$TGM_ROOT/log/set.out"
else
  ps -ef > "$BINDIR$TGM_ROOT/log/psef.out"
  df  > "$BINDIR$TGM_ROOT/log/df.out"
  set > "$BINDIR$TGM_ROOT/log/set.out"
fi


if [ $TDS_OS = "RISC" ] ; then
  if [ -f "/tmp/Tivoli/tbsm/ipc_rc.log" ] ; then
    cp "/tmp/Tivoli/tbsm/ipc_rc.log"  "$BINDIR$TGM_ROOT/log"
  fi

  if [ -f "/tmp/Tivoli/tbsm/ipc_rc.bak" ] ; then
    cp "/tmp/Tivoli/tbsm/ipc_rc.bak"  "$BINDIR$TGM_ROOT/log"
  fi
fi

#
# tar up the log files
#
cd "$BINDIR$TGM_ROOT"

tar -cvf "$TGM_TEMP/tbsm.$MY_OS.tar" log
RC=$?
if [ $RC -ne 0 ] ; then
  command "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5851
  echo "$RC"
  rc=1
fi

cd "$TGM_TEMP"

if [ $MY_OS != "Windows_NT" ] ; then
  compress "$TGM_TEMP/tbsm.$MY_OS.tar"
  RC=$?
  if [ $RC -ne 0 ] ; then
    command "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5852
    echo "$RC"
    rc=1
  fi
fi

#
# remove some temporary files
#
if [ -f "$BINDIR$TGM_ROOT/log/lsps.a" ] ; then
  rm "$BINDIR$TGM_ROOT/log/lsps.a"
fi
if [ -f "$BINDIR$TGM_ROOT/log/psef.out" ] ; then
  rm "$BINDIR$TGM_ROOT/log/psef.out"
fi
if [ -f "$BINDIR$TGM_ROOT/log/lsdev.C" ] ; then
  rm "$BINDIR$TGM_ROOT/log/lsdev.C"
fi
if [ -f "$BINDIR$TGM_ROOT/log/df.out" ] ; then
  rm "$BINDIR$TGM_ROOT/log/df.out"
fi
if [ -f "$BINDIR$TGM_ROOT/log/set.out" ] ; then
  rm "$BINDIR$TGM_ROOT/log/set.out"
fi
if [ -f "$BINDIR$TGM_ROOT/log/errlog" ] ; then
  rm "$BINDIR$TGM_ROOT/log/errlog"
fi
if [ -f "$BINDIR$TGM_ROOT/log/errpt.log" ] ; then
  rm "$BINDIR$TGM_ROOT/log/errpt.log"
fi

if [ $rc -eq 0 ] ; then
  command "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5853
 else
  command "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5854
fi

exit $RC
