#------------------------------------------------- {COPYRIGHT-TOP-RM}
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#------------------------------------------------- {COPYRIGHT-END-RM}

# File: archive/scripts/fo_config.ksh
# SID: 1.19.5.2
# Date modified: 07/03/08 14:47:56
#
# Module Description:
#
# fo_config.ksh - TBSM Failover configuration script
#
#   This script configure both the primary and secondary site for failover.  
#
#   Note:  
#   - A configuration file is required.
#   - Use -G option to generate configuration file template.
#   - This config script needs to be run before starting the log shipping, failover, or reverse failover. 
#   - Run this script from primary Database server.  Failover direction is not an issue at this time.
#
#set -x
#


usage()
{
  if [ "$NLSMSG" == TRUE ]; then
    usage2
    return
  fi

  print "Usage: $0 [ -G | -f<config_file> | -T | -h ]"
  print "This script performs Failover configuration.  It should only be invoked from "
  print "the primary Database Server. A configuration file is required to successfully "
  print "perform failover configuration. "
  print "The following list shows the supported Server Roles with their associated Windows services "
  print "
  Required Roles:
    ROLE_DatabaseServer(Database Server Role):
      ASIDBValidater, ASIStagedEventLoader, ASIMVSIPListenerSvc, ASIEventEnablement, ASITaskServer
    ROLE_WASServer(WebSphere Application Server Role), accept multiple hosts.

  Optional Roles:
    ROLE_AgentListener(Agent Listener Role): 
      ASIAgentListenerSvc  
    ROLE_CommonListener(Common Listener Role):
      ASICommonlistener 
    ROLE_EhServer(Event Handler Server Role), accept multiple hosts:
      ASIDBValidater, ASIFileReceiver-<svc_name>, ASIMVSEventHandler-<svc_name>, 
      ASIMVSIPOSListenerSvc, ASIEnqueueProxyServer, ASIRemoteExecutionServer
    ROLE_HistServer(History Server Role)
    ROLE_HmServer(Health Monitor Server Role):
      ASIHealthMonitor
    ROLE_SnaServer(Host Integration Server Role):
      SnaServr
    ROLE_SsServer(Sender Service Server Role), accept multiple hosts:  
      ASIDBValidater, ASIEnqueueProxyServer, ASIMVSIPSender-<svc_name>, ASIMVSSender-<svc_name>
    ROLE_EIFSender(EIF State Change Sender Server Role):
      ASIEIFSender
    ROLE_TECListener(TEC Listener Role):
      ASITECListenerSvc 
    ROLE_TGMTaskServer(TGM Task Server Role), accept multiple hosts:
      ASIEventEnablement, ASITaskServer
    ROLE_UpLoadRuleServer(MVS Upload Rule Server Role):
      ASIMVSUploadRuleSvc 
    ROLE_WebServer(Reporting Server Role, where IIS is running):
      W3SVC
    ROLE_XMLToolkit(XML Toolkit Server Role), accept multiple hosts:
      ASIXMLToolkitSvc
"
  print "  -G          Generates Failover Configuration file template and exit"
  print "  -f<file>    Specifies the configuration file to use"
  print "  -T          Turns on Trace mode"
  print "  -h          Displays help info"
}

usage2()
{
  ASIShellMsg $FOMSG 800 $0
  ASIShellMsg $FOMSG 827
  ASIShellMsg $FOMSG 830
  ASIShellMsg $FOMSG 822
  ASIShellMsg $FOMSG 819
  ASIShellMsg $FOMSG 823
}

. load_fo_env


while getopts "Gf:Th?" c; do
  case $c in

  G)  GenConfigFile=TRUE;;
  f)  ConfigFile=$OPTARG;;
  T)  Trace_flag=TRUE;;
  h) usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

# define failover.upg file 
#----------------------------
FailoverUPG=" \n\
SQL| 2|backup_movement_plans|             0|tables/backup_movement_plans.sql \n\
SQL| 2|backup_movement_plan_databases|    0|tables/backup_movement_plan_databases.sql \n\
SQL| 2|backup_movement_plan_history|      0|tables/backup_movement_plan_history.sql \n\
SQL| 2|asisp_create_backup_movement_plan| 0|procs/asisp_create_backup_movement_plan.sql \n\
SQL| 2|sp_add_db_to_backup_movement_plan| 0|procs/sp_add_db_to_backup_movement_plan.sql \n\
SQL| 2|log_ship_status|                   0|procs/log_ship_status.sql \n\
SQL| 2|log_ship_status_hms|               0|procs/log_ship_status_hms.sql \n\
SQL| 2|log_ship_entity_log|               0|procs/log_ship_entity_log.sql \n\
SQL| 2|log_ship_history_purge|            0|procs/log_ship_history_purge.sql \n\
SQL| 2|log_ship_alert|                    0|procs/log_ship_alert.sql \n\
SQL| 2|asisp_recover_databases|           0|procs/asisp_recover_databases.sql \n\
SQL| 2|asisp_create_directory|            0|procs/asisp_create_directory.sql \n\
SQL| 2|asisp_split_file_path|             0|procs/asisp_split_file_path.sql \n\
SQL| 2|asisp_get_restore_move|            0|procs/asisp_get_restore_move.sql \n\
SQL| 2|asisp_enabledisable_job|           0|procs/asisp_enabledisable_job.sql \n\
SQL| 2|asisp_disable_logship_jobs|        0|procs/asisp_disable_logship_jobs.sql \n\
SQL| 2|asisp_create_backup_restore_purge| 0|procs/asisp_create_backup_restore_purge.sql \n\
SQL| 2|asisp_create_logship_trans_backup| 0|procs/asisp_create_logship_trans_backup.sql \n\
SQL| 2|asisp_create_logship_backups|      0|procs/asisp_create_logship_backups.sql \n\
SQL| 2|asisp_restore_logship_backups|     0|procs/asisp_restore_logship_backups.sql \n\
SQL| 2|asisp_create_copy_load_jobs|       0|procs/asisp_create_copy_load_jobs.sql \n\
SQL| 2|asisp_create_syn_eventcachetime|   0|procs/asisp_create_syn_eventcachetime.sql \n\
SQL| 2|asisp_create_export_jobs|          0|procs/asisp_create_export_jobs.sql
"


#
#  routine to generate a config file template
#
generate_configfile()
{
  print '
# ***  This is a generated TBSM Version 3.2 Failover Configuration file  *** 
# ***  Please edit the hostnames and parameters to match your TBSM environment. 
# 
# ---Primary SERVER ROLE---       -----HOSTNAME------ 
'
  for host in $All_Server_Names; do
    print "ROLE_"$host"_Primary \t| $HOSTNAME" 
  done

  print '
# 
# ---Secondary SERVER ROLE---     -----HOSTNAME------ 
'
  for host in $All_Server_Names; do
    print "ROLE_"$host"_Secondary \t| $HOSTNAME" 
  done
}


#
# routine to read and parse input config file
#
parse_configfile()
{
  # validate the config file
  #-------------------------
  validateFile $ConfigFile
  if [ $? != 0 ]; then
    msgprint $WARN "Configuration file $ConfigFile is invalid!"
    msgprint2 135 $WARN $ConfigFile 
    return 1
  fi
 
  #
  # generate awk script for config file parsing
  #
  awkscript="$LOGDIR/$0.awk" 

  print '{ gsub(/  /, " ") }' > "$awkscript"
  print '{ gsub(/^ /, "", $2); gsub(/ $/, "", $2); gsub(/,/, " ", $2) }' >> "$awkscript"

  for host in $All_Server_Names; do
    print '/ROLE_'$host'_Primary/ {printf("HOST_'$host'_Primary=%s\\n", toupper($2))}' >> "$awkscript"
  done

  for host in $All_Server_Names; do
    print '/ROLE_'$host'_Secondary/ {printf("HOST_'$host'_Secondary=%s\\n", toupper($2))}' >> "$awkscript"
  done

  # read the input config file and generated internal format of the file 
  #---------------------------------------------------------------------
  awk -F"|" -f $awkscript $ConfigFile | awk -F"=" '{ if ($2 != "") print $0 }' | 
    crypt -e $SERVER_ROLE_KEY > "$SERVER_ROLE_FILE" 

  rm -rf "$awkscript"
  return 0
}


#
# If EhServer or SsServer is defined, UpLoadRuleServer should be defined
# Each TBSM site can have only one UpLoadRuleServer defined 
#   Extra UpLoadRuleServer will be disabled
#
chk_extra_ASIMVSUploadRuleSvc()
{
  if [ "$HOST_EhServer_Primary" != "" ] ||  [ "$HOST_SsServer_Primary" != "" ]; then
    if [ "$HOST_UpLoadRuleServer_Primary" == "" ]; then
      msgprint $WARN "Primary UpLoadRuleServer is not defined!"
      msgprint2 286 $WARN 
    fi
  fi 

  if [ "$HOST_EhServer_Secondary" != "" ] ||  [ "$HOST_SsServer_Secondary" != "" ]; then
    if [ "$HOST_UpLoadRuleServer_Secondary" == "" ]; then
      msgprint $WARN "Secondary UpLoadRuleServer is not defined!"
      msgprint2 316 $WARN 
    fi
  fi 

  # check if there is more than one host assigned to this role
  #-------------------------------------------------------------
  if [ "$HOST_UpLoadRuleServer_Primary" != "" ]; then
    for host in $(hostsin $All_Primary_Hosts)
    do
      if [ "$host" == "$HOST_UpLoadRuleServer_Primary" ]; then
        continue
      fi

      sclist \\\\$host | grep -q -e " ASIMVSUploadRuleSvc " >nul:
      if [ $? == 0 ]; then
        msgprint $WARN "Stopping and disabling extra ASIMVSUploadRuleSvc on $host!"
        msgprint2 332 $WARN $host
        sc \\\\$host stop ASIMVSUploadRuleSvc >nul:
        sc \\\\$host config ASIMVSUploadRuleSvc start= disabled >nul:
      fi 
    done
  fi

  if [ "$HOST_UpLoadRuleServer_Secondary" != "" ]; then
    for host in $(hostsin $All_Secondary_Hosts)
    do
      if [ "$host" == "$HOST_UpLoadRuleServer_Secondary" ]; then
        continue
      fi

      sclist \\\\$host | grep -q -e " ASIMVSUploadRuleSvc " >nul:
      if [ $? == 0 ]; then
        msgprint $WARN "Stopping and disabling extra ASIMVSUploadRuleSvc on $host!"
        msgprint2 332 $WARN $host
        sc \\\\$host stop ASIMVSUploadRuleSvc >nul:
        sc \\\\$host config ASIMVSUploadRuleSvc start= disabled >nul:
      fi 
    done
  fi
  
  return 0 
}


#
# validate server role configuration
#  check required roles on primary site
#  check multiple UpLoadRuleServer
#  check paired roles
#  database server and history server cannot be the same
#  get WAS configuration confirmation
#  TGMTask server mapping, if more than 1 specified. 
#
validate_configinfo()
{
  # Required roles check
  #----------------------

  # Primary Database Server is required
  #-----------------------------
  if [ "$HOST_DatabaseServer_Primary" == "" ]; then
    msgprint $WARN "Primary Database Server is not specified!"
    msgprint2 282 $WARN 
    return 1
  fi

  # Primary WAS Server is required
  #---------------------------------
  if [ "$HOST_WASServer_Primary" == "" ]; then
    msgprint $WARN "Primary WebSphere Application Server is not specified!"
    msgprint2 287 $WARN 
    return 1
  fi

  let p_count=$(echo $HOST_UpLoadRuleServer_Primary | awk '{printf NF}')
  let s_count=$(echo $HOST_UpLoadRuleServer_Secondary | awk '{printf NF}')

  if [ "$p_count" -gt 1 -o "$s_count" -gt 1 ]; then
    msgprint $WARN "More than one Primary or Secondary UpLoadRuleServer is specified!"
    msgprint2 254 $WARN 
    return 1
  fi

  # paired role check
  #  if primary site is specified, secondary site has to specified also
  #  same hostname cannot be assigned to both sites except WAS 
  #--------------------------------------------------------------------
  for server in $All_Server_Names; do
    eval p_server='$HOST_'$server'_Primary'
    eval s_server='$HOST_'$server'_Secondary'
    
    if [ "$p_server" != "" -a "$s_server" == "" ]; then
      msgprint $WARN "Primary $server is specified, but not the Secondary!"
      msgprint2 280 $WARN $server 
      return 1
    fi

    if [ "$p_server" == "" -a "$s_server" != "" ]; then
      msgprint $WARN "Secondary $server is specified, but not the Primary!"
      msgprint2 311 $WARN $server 
      return 1
    fi

    if [ "$p_server" != "" -a "$p_server" == "$s_server" ]; then
      msgprint $WARN "Primary and Secondary $server cannot be the same!"
      msgprint2 288 $WARN $server 
      return 1
    fi

  done

  # Primary Database Server and History Server cannot be the same
  #-------------------------------------------------------
  if [ "$HOST_DatabaseServer_Primary" == "$HOST_HistServer_Primary" ]; then
    msgprint $WARN "The Primary Database Server and History Server cannot be the same!"
    msgprint2 362 $WARN 
    return 1
  fi

  # Secondary Database Server and History Server cannot be the same
  #-------------------------------------------------------
  if [ "$HOST_DatabaseServer_Secondary" == "$HOST_HistServer_Secondary" ]; then
    msgprint $WARN "The Secondary Database Server and History Server cannot be the same!"
    msgprint2 363 $WARN 
    return 1
  fi

  # check equal number of TGMTask Server on both sites
  # return error if number in primary and secondary are different
  #-----------------------------------------------------------------

  let p_tgm_count=$(echo $HOST_TGMTaskServer_Primary | awk '{print NF}')
  let s_tgm_count=$(echo $HOST_TGMTaskServer_Secondary | awk '{print NF}')

  if [ "$p_tgm_count" -ne "$s_tgm_count" ]; then
    msgprint $WARN "Number of Primary/Secondary TGM Task Server does not match($p_tgm_count/$s_tgm_count)."
    msgprint2 268 $WARN $p_tgm_count $s_tgm_count
    exit 1
  fi

  if [ "$p_tgm_count" -gt 1 ]; then
    # get confirm from user on the primary to secondary mapping
    #----------------------------------------------------------
    set +A p_tgm $HOST_TGMTaskServer_Primary
    set +A s_tgm $HOST_TGMTaskServer_Secondary

    beep
    print "The Primary and Secondary TGM Task Server will have the following mapping:"

    idx=0
    until [ "$idx" -ge "$p_tgm_count" ]
    do
      print "Primary TGM Task Server ${p_tgm[$idx]} <--> Secondary TGM Task Server ${s_tgm[$idx]}"
      let idx="$idx + 1"
    done

    print -n "Continue [y|n]?  "
    read response

    if [ ! "$response" == "y" ] && [ ! "$response" == "Y" ]; then
      return 1
    fi
  fi

  return 0
}


#
# create out-of-synch notification in msdb database
#
sql_OOSAlert()
{
  host=$1
  alertMsg=$2

  # clean up previous operator and alert if any
  # create administrators' operator
  # create Out of Synch Alert
  # create administrators notification
  #-----------------------------------------------
  sql_cmd="
    IF (EXISTS (SELECT \"*\" FROM sysoperators WHERE name = N'TBSM Administrator'))
      EXEC sp_delete_operator @name = 'TBSM Administrator'
    IF (EXISTS (SELECT \"*\" FROM sysalerts WHERE name = N'TBSM Out of Synch Alert'))
      EXEC sp_delete_alert @name = 'TBSM Out of Synch Alert'

    EXEC sp_add_operator @name = 'TBSM Administrator', @enabled = 1, @netsend_address = '$host'
    EXEC sp_add_alert @name = 'TBSM Out of Synch Alert', @message_id = 90051, @severity = 0,
      @notification_message = '$alertMsg'

    EXEC sp_add_notification 'TBSM Out of Synch Alert', 'TBSM Administrator', 4
  "
  dbquery $host msdb $sql_cmd
  if [ $? != 0 ]; then
    msgprint $WARN "Failed to create OOS Alert on $host."
    msgprint2 164 $WARN $host
    return 1
  fi

  msgprint $TRACE "Successfully created OOS Alert on $host."
  msgprint2 339 $TRACE $host

  # install necessary stored proc and tables
  #------------------------------------------
  sqldir="$(find_tbsm_install_dir $HOST_DatabaseServer_Primary | sed 's/\$/:/g')/sql"
  cd "$sqldir"

  print $FailoverUPG > "failover.upg"
  msgprint $TRACE "Generated failover.upg in $sqldir"

  dbuser=""
  dbpassword=""
  eval access=$(print '$DB_ACCESS_'$host)
  eval $(print $access | awk '{printf("dbuser=%s dbpassword=%s", $1, $2)}')

  if [ "$dbuser" == "" ]; then
    ./ApplyUpgrade -S$host -E failover.upg
  else
    ./ApplyUpgrade -S$host -U$dbuser -P$dbpassword failover.upg
  fi

  if [ $? != 0 ]; then
    msgprint $WARN "Failed to create $sqlfile on $host."
    msgprint2 100 $WARN "Failed to create $sqlfile on $host."
    return 1
  fi

  msgprint $INFO "Successfully apply failover.upg to database on $host."
  return 0
}


#
# create DatabaseServer and History Server out-of-synch notification 
#
create_OOSAlert()
{
  for host in $(hostsin $HOST_DatabaseServer_Primary $HOST_DatabaseServer_Secondary)
  do
    sql_OOSAlert $host "TBSM Primary and Secondary Database Server Databases are Out of Synch!" 
    if [ $? != 0 ]; then
      return 1
    fi

  done

  for host in $(hostsin $HOST_HistServer_Primary $HOST_HistServer_Secondary)
  do
    sql_OOSAlert $host "TBSM Primary and Secondary History Server Databases are Out of Synch!" 
    if [ $? != 0 ]; then
      return 1
    fi

  done
}


propagate_server_role_file()
{
  host=$1

  if [ "$host" == "" ]; then
    msgprint $TRACE "Server role is empty!" 
    msgprint2 892 $TRACE 
    return 0
  fi

  # no need to copy to itself
  #---------------------------
  if [ "$host" == "${COMPUTERNAME%%.*}" ]; then
    return 0
  fi

  # propagate server_role_file to host
  #---------------------------------------------------------
  instdir=$(find_tbsm_install_dir $host)
  if [ "$instdir" == "" ]; then
    msgprint $ERROR "Failed to find INSTALLDIR of $host. Cannot continue!" 
    msgprint2 171 $ERROR $host
    rm -f "$SERVER_ROLE_FILE"
    return 1
  fi

  cp -pv "$SERVER_ROLE_FILE" "//$host/$instdir" >nul: 2>&1 
  if [ $? != 0 ]; then
    msgprint $ERROR "Failed to store configuration file on $host. Cannot continue!" 
    msgprint2 202 $ERROR $host
    rm -f "$SERVER_ROLE_FILE"
    return 1
  fi

  msgprint $TRACE "Successfully copied configuration file to $host!" 
  msgprint2 338 $TRACE $host
  
  return 0
}


#
# routine to collect History Server database user and password info.
#
getHistServer_dbinfo()
{
  host=$1

  dbuser=""
  dbpassword=""
  db_passwd_2=""

  print " ------  For History Server $host Database Access ------ "
  beep
  print "Please enter Database User for the History Server $host "
  print -n "(Hit Enter key if trusted mode is used for database access): "
  read dbuser

  if [ "$dbuser" == "" ]; then
    return;
  fi

  while TRUE; do
    print -n "Please enter Database Password for the user $dbuser: "
    stty -echo
    read dbpassword
    stty echo
    print
    print -n "Please re-enter the Password again: "
    stty -echo
    read db_passwd_2
    stty echo
    print

    if [ "$dbpassword" == "$db_passwd_2" ]; then
      break
    else
      msgprint $WARN "Database Passwords entered are not the same, please try again!"
      msgprint2 141 $WARN
    fi
  done
}


#
# add the following host DB access info to the tbsm_server_role file
#   $HOST_DatabaseServer_Primary
#   $HOST_DatabaseServer_Secondary
#   $HOST_HistServer_Primary
#   $HOST_HistServer_Secondary
#-------------------------------------------------------------------
add_db_access_info()
{
  cat "$SERVER_ROLE_FILE" | crypt -d "$SERVER_ROLE_KEY" > "$SERVER_ROLE_FILE.tmp"

  for host in $(hostsin $HOST_DatabaseServer_Primary $HOST_DatabaseServer_Secondary)
  do
    dbuser=""
    dbpassword=""
    # find the db access info
    result=$(regdmp -m \\\\$host \
      "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" |
      awk ' /User/ {printf("dbuser=%s\n",$3)}
            /Password/ {printf("dbpassword=%s\n",$3)}')
    eval $result

    #if [ "$dbuser" == "" ]; then
    #  msgprint $WARN "Blank database user is defined for Database Server $host."
    #  msgprint2 121 $WARN $host
    #  return 1
    #fi

    print "DB_ACCESS_$host=$dbuser $dbpassword" >> "$SERVER_ROLE_FILE.tmp"
  done

  for host in $(hostsin $HOST_HistServer_Primary $HOST_HistServer_Secondary)
  do
    getHistServer_dbinfo $host
    #if [ "$dbuser" == "" ]; then
    #  msgprint $WARN "Blank database user is defined for History Server $host."
    #  msgprint2 122 $WARN $host
    #  return 1
    #fi

    print "DB_ACCESS_$host=$dbuser $dbpassword" >> "$SERVER_ROLE_FILE.tmp"
  done

  cat "$SERVER_ROLE_FILE.tmp" | crypt -e "$SERVER_ROLE_KEY" > "$SERVER_ROLE_FILE"
  rm -f "$SERVER_ROLE_FILE.tmp"
  return 0
}



#--------------   MAIN   ------------------
main()
{
  # generate config file and exit if the -G option is specified 
  # no need to continue
  #------------------------------------------------------------
  if [ "$GenConfigFile" == TRUE ]; then
     generate_configfile | expand -17
     exit 0
  fi

  # read the input config file and parse it
  #-----------------------------------------
  parse_configfile 
  if [ $? != 0 ]; then
    msgprint $ERROR "Failed to parse configuration file $ConfigFile. Cannot continue!" 
    msgprint2 186 $ERROR $ConfigFile
    exit 1
  fi

  # load config info into memory 
  #-----------------------------
  load_configinfo 
  if [ $? != 0 ]; then
    msgprint $ERROR "Failed to load configuration information. Cannot continue!" 
    msgprint2 182 $ERROR 
    rm -f "$SERVER_ROLE_FILE"
    exit 1
  fi

  msgprint $INFO "Successfully loaded configuration file $ConfigFile." 
  msgprint2 348 $INFO $ConfigFile

  # Check to see if we are on the Primary Database Server
  #-------------------------------------------------
  if [ "$HOSTNAME" != "$HOST_DatabaseServer_Primary" ]; then
    msgprint $ERROR "Please run this script from the Primary Database Server '$HOST_DatabaseServer_Primary'!" 
    msgprint2 273 $ERROR $HOST_DatabaseServer_Primary
    rm -f "$SERVER_ROLE_FILE"
    exit 1
  fi

  # validate config info
  #----------------------
  validate_configinfo
  if [ $? != 0 ]; then
    msgprint $ERROR "Configuration file validation failed!"
    msgprint2 136 $ERROR 
    rm -f "$SERVER_ROLE_FILE"
    exit 1
  fi

  # update server_role_file with DB access info
  #---------------------------------------------
  add_db_access_info
  if [ $? != 0 ]; then
    msgprint $ERROR "Failed to add Database access info. Cannot continue!"
    msgprint2 153 $ERROR
    rm -f "$SERVER_ROLE_FILE"
    exit 1
  fi

  # reload just DB access info
  #-----------------------------------
  load_dbaccess
  if [ $? != 0 ]; then
    msgprint $ERROR "Failed to load Database access info. Cannot continue!"
    msgprint2 181 $ERROR
    rm -f "$SERVER_ROLE_FILE"
    exit 1
  fi

  # chk Database Server version on both sites, hostname will be return if cannot access db 
  # check the version either 7 or 2000
  #-------------------------------------
  PD_VERSION=$(get_sqlversion $HOST_DatabaseServer_Primary)
  SD_VERSION=$(get_sqlversion $HOST_DatabaseServer_Secondary)

  if [ "$PD_VERSION" == "$HOST_DatabaseServer_Primary" ]; then
    msgprint $ERROR "Problem in reading Primary Database Servers SQL Server version!" 
    msgprint2 295 $ERROR 
    rm -f "$SERVER_ROLE_FILE"
    exit 1
  fi

  if [ "$SD_VERSION" == "$HOST_DatabaseServer_Secondary" ]; then
    msgprint $ERROR "Problem in reading Secondary Database Servers SQL Server version!" 
    msgprint2 297 $ERROR 
    rm -f "$SERVER_ROLE_FILE"
    exit 1
  fi

  if [ "$PD_VERSION" != "$SD_VERSION" ]; then
    msgprint $WARN "Primary Database Servers SQL has version $PD_VERSION!" 
    msgprint2 283 $WARN $PD_VERSION
    msgprint $WARN "Secondary Database Servers SQL has version $SD_VERSION!" 
    msgprint2 313 $WARN $SD_VERSION
    msgprint $ERROR "Primary and Secondary Database Servers are at different SQL Server version!" 
    msgprint2 289 $ERROR 
    rm -f "$SERVER_ROLE_FILE"
    exit 1
  fi
    
  if [ "$HOST_HistServer_Primary" != "" ]; then
    PH_VERSION=$(get_sqlversion $HOST_HistServer_Primary)
    SH_VERSION=$(get_sqlversion $HOST_HistServer_Secondary)

    if [ "$PH_VERSION" == "$HOST_HistServer_Primary" ]; then
      msgprint $ERROR "Problem in reading Primary History Servers SQL Server version!" 
      msgprint2 296 $ERROR 
      rm -f "$SERVER_ROLE_FILE"
      exit 1
    fi

    if [ "$SH_VERSION" == "$HOST_HistServer_Secondary" ]; then
      msgprint $ERROR "Problem in reading Secondary History Servers SQL Server version!" 
      msgprint2 298 $ERROR 
      rm -f "$SERVER_ROLE_FILE"
      exit 1
    fi

    if [ "$PH_VERSION" != "$SH_VERSION" ]; then
      msgprint $WARN "Primary History Servers SQL has version $PH_VERSION!" 
      msgprint2 285 $WARN $PH_VERSION
      msgprint $WARN "Secondary History Servers SQL has version $SH_VERSION!" 
      msgprint2 315 $WARN $SH_VERSION
      msgprint $ERROR "Primary and Secondary History Servers are at different SQL Server version!" 
      msgprint2 290 $ERROR 
      rm -f "$SERVER_ROLE_FILE"
      exit 1
    fi
  fi

  # check all servers connectivity
  #---------------------------------
  conn_err=0
  for host in $(hostsin $All_Primary_Hosts $All_Secondary_Hosts)
  do
    msgprint $INFO "Checking network connectivity to host $host ..." 
    msgprint2 130 $INFO $host 

    connectivity_chk $host
    if [ $? != 0 ]; then
      msgprint $ERROR "Connectivity check to host $host failed!" 
      msgprint2 137 $ERROR $host
      let conn_err="$conn_err + 1"
    else
      msgprint $INFO "Connectivity check to host $host is successful!" 
      msgprint2 139 $INFO $host 
    fi
  done

  if [ "$conn_err" -gt 0 ]; then
    msgprint $ERROR "Failed to connect to $conn_err servers. Cannot continue!" 
    msgprint2 158 $ERROR $conn_err 
    exit 1
  fi

  # propagate server_role_file to all servers
  #---------------------------------------------------------
  for host in $(hostsin $All_Primary_Hosts $All_Secondary_Hosts)
  do
    msgprint $INFO "Copying server_role_file to host $host ..." 
    msgprint2 890 $INFO 
    propagate_server_role_file $host
    if [ $? != 0 ]; then
      exit 1
    fi

    msgprint $TRACE "Successfully copied configuration file to $host!"
    msgprint2 338 $TRACE $host
  done

  # chk extra ASIMVSUploadRuleSvc
  #-----------------------------
  chk_extra_ASIMVSUploadRuleSvc 
  if [ $? != 0 ]; then
    msgprint $ERROR "Failed to disable extra ASIMVSUploadRuleSvc. Cannot continue!" 
    msgprint2 165 $ERROR 
    exit 1
  fi

  # create out-of-sync-alert
  #-------------------------
  create_OOSAlert
  if [ $? != 0 ]; then
    msgprint $ERROR "Failed to create Database Out-Of-Synch Alert. Cannot continue!" 
    msgprint2 163 $ERROR 
    exit 1
  fi

  msgprint $INFO "Successfully configured TBSM Servers for Failover." 
  msgprint2 334 $INFO 

  exit 0
}

#--------------------------------------------------------------------------------------
main

