#------------------------------------------------- {COPYRIGHT-TOP-RM}
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#------------------------------------------------- {COPYRIGHT-END-RM}

# File: archive/scripts/load_fo_env.ksh
# SID: 1.18.1.3
# Date modified: 07/03/08 14:47:55
#
# Module Description
#
# used by all scripts

# INSTALLDIR is where local TBSM has been installed
#-----------------------------------------------------
INSTALLDIR=$(reg query "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" 2>nul: | awk '/InstallDirectory/ { gsub(/\\/, "/"); print $3 }')

# LOGDIR is where the local log files located
#-----------------------------------------------------
LOGDIR=$(reg query "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\Log" 2>nul: | awk '/Directory/ { gsub(/\\/, "/"); print $3 }')

# HOSTNAME is the name of current local host
#-----------------------------------------------------
HOSTNAME=$(echo `hostname` | awk '{print toupper($0)}')

which osql.exe >&2 >nul:
if [ $? != 0 ]; then
  echo "Could not find osql.exe.  Please install SQL Client Utilities." 
  exit 1
fi

SQLCMD="osql -n -w3000 -h-1 -l60 "

# list of unreachable hosts
#---------------------------
HOST_DOWN=""

# FO_LOG_FILE is where to write the log message
#-----------------------------------------------------
now=$(date "+"%Y""%m""%d""%H""%M""%S"")
FO_LOG_FILE="$LOGDIR/$(basename "$0" .ksh).$now.log"

# FO_CMD_FILE is where the failover command generated
#-----------------------------------------------------
FO_CMD_FILE="$LOGDIR/$(basename "$0" .ksh).$now.cmd"

# SERVER_ROLE_FILE where the local configuration file is  
#-----------------------------------------------------
SERVER_ROLE_FILE="$INSTALLDIR/tbsm_server_role"
SERVER_ROLE_KEY="FAILOVER"

# the output file created by the database query
#-------------------------------------------------
DBQUERY_OUT="$LOGDIR/DBQUERY_OUT.tmp"

#
# All_Server_Names defines all TBSM server role names
#-----------------------------------------------------
All_Server_Names="DatabaseServer
  WASServer
  AgentListener
  CommonListener
  EhServer
  EIFSender
  HistServer
  HmServer
  SnaServer
  SsServer
  TECListener
  TGMTaskServer
  UpLoadRuleServer
  WebServer
  XMLToolkit
  "

#
# BATCHSVC_START_TIMER defines the number of seconds the failover will wait 
#   for a batch of Ss or Eh services to start 
# BATCHSVC_STOP_TIMER defines the number of seconds the failover will wait 
#   for a batch of Ss or Eh services to stop
#---------------------------------------------------------------------------------------
BATCHSVC_START_TIMER=120
BATCHSVC_STOP_TIMER=120

# WAIT_JOBDONE - wait at most 3600 seconds for scheduled task to complete running
# WAIT_TLFSTART - wait at least 900 seconds for 2 invocation of Log Backup
#---------------------------------------------------------------------------------
WAIT_JOBDONE=3600
WAIT_TLFSTART=900

# wait for WAS to start/stop
PARM_WAS_WaitTime=120

ALLTBSM_DBNAME_SQL="ASIRuleSvc Meta Object ObjectEvents ObjectQueues RODM RODMLoad OPCLoad WebServer"
ALLTBSM_DBNAME_HIST="History WebServer"

mksSort="$ROOTDIR/mksnt/sort"

# check see if we need to do NLS message
#-----------------------------------------
FOMSG=""
NLSMSG=FALSE
ls "$INSTALLDIR"/bin/failovermsg*.dll 2>nul: | grep -iq -e "failovermsg"
if [ $? == 0 ]; then
  FOMSG="failovermsg"
fi
  
if [ "$FOMSG" != "" -a "$(uname -s)" = "Windows_NT" -a "$(which ASIShellMsg.exe)" ]; then
  NLSMSG=TRUE
fi

#
# defines the severity level of the msgprint 
#-----------------------------------------------------
ERROR="E"
INFO="I"
WARN="W"
TRACE="T"
WARN_COUNT=0
ERROR_COUNT=0

#
# Non-NLS message print
#
# routine to write message to a log file and stdout
# syntax : msgprint <severity> <msgtext>
#
msgprint()
{
  if [ $NLSMSG == TRUE ]; then
    return 0
  fi

  if (! [ $Trace_flag ] && [ "$1" == "$TRACE" ]); then
    return 0
  fi

  now=$(date "+"%Y"/"%m"/"%d" "%H":"%M":"%S"")

  print -r $now "$*" >> "$FO_LOG_FILE"
  print -r $now "$*" >&2

  if [ "$1" == "$WARN" ]; then
    let WARN_COUNT="$WARN_COUNT + 1"
  fi

  if [ "$1" == "$ERROR" ]; then
    let ERROR_COUNT="$ERROR_COUNT + 1"
  fi
}


msgprint2()
{
  if [ $NLSMSG == FALSE ]; then
    return 0
  fi

  msgnum=$1
  t_flag=$2
  #parms=$(shift 2; print -r "$*")

  if (! [ $Trace_flag ] && [ "$t_flag" == "$TRACE" ]); then
    return 0
  fi

  now=$(date "+"%Y"/"%m"/"%d" "%H":"%M":"%S"")

  shift 2
  ASIShellMsg $FOMSG $msgnum "$now $t_flag" "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9" >> "$FO_LOG_FILE"
  ASIShellMsg $FOMSG $msgnum "$now $t_flag" "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9" 

  if [ "$t_flag" == "$WARN" ]; then
    let WARN_COUNT="$WARN_COUNT + 1"
  fi

  if [ "$t_flag" == "$ERROR" ]; then
    let ERROR_COUNT="$ERROR_COUNT + 1"
  fi
}


msgprint $TRACE "Tivoli Business Systems Manager V3.2 Script $0 "
msgprint2 371 $TRACE $0 

#
# routine to return sorted unique host names from input parameters
#
hostsin()
{
  print $@ | xargs -n1 | $mksSort -u 
}


#
#  connectivity_chk - routine performs the following 
#    - ping check
#    - SCLIST check
#    - SC Query check
#    - directory share check
#
#
connectivity_chk()
{
  hostname=$1

  if [ "$hostname" == "" ]; then
    msgprint $ERROR "Hostname is not specified. Cannot continue!"
    msgprint2 228 $ERROR 
    return 1
  fi

  # ping check
  #-----------
  rc=$(ping -n 1 $hostname)
  if [ $? != 0 ]; then
    msgprint $WARN "Ping $hostname failed!" 
    msgprint2 270 $WARN $hostname 
    return 1
  else
    if $(print "$rc" | grep -q -e "Request timed out"); then
      msgprint $WARN "Ping $hostname failed!" 
      msgprint2 270 $WARN $hostname 
      return 1
    fi
  fi

  # SCLIST check
  #-------------
  sclist \\\\$hostname >nul: 2>&1 
  if [ $? != 0 ]; then
    msgprint $WARN "Problem in running SCLIST on host $hostname"
    msgprint2 300 $WARN $hostname
    return 1
  fi

  # SC QUERY check
  # the following not true in 2003
  # SC return 1 when OK
  # SC return 0 when not OK
  # change code to cope with different return code from 2000/2003
  #-------------------------
  rc=$(SC \\\\$hostname QUERY NetLogon)

  if $(print $rc | grep -iq -v -e "SERVICE_NAME: NetLogon"); then
    return 1
  fi

  return 0
}


#
# routine load DB access info into memory
#
load_dbaccess()
{
  if [ "$DB_ACCESS_LOADED" == "TRUE" ]; then
    return 0
  fi

  validateFile "$SERVER_ROLE_FILE"
  if [ $? != 0 ]; then
    msgprint $WARN "File $SERVER_ROLE_FILE is invalid!"
    msgprint2 214 $WARN "$SERVER_ROLE_FILE" 
    return 1
  fi

  eval $(cat "$SERVER_ROLE_FILE" | crypt -d "$SERVER_ROLE_KEY" |
    awk -F"=" '/DB_ACCESS.*/ {printf("%s=\"%s\" ", $1, $2) }')

  DB_ACCESS_LOADED=TRUE
}


#
# Load the configuration info into memory
# also set up variables: All_Primary_Hosts and All_Secondary_Hosts
#
load_configinfo()
{
  validateFile "$SERVER_ROLE_FILE"
  if [ $? != 0 ]; then
    msgprint $WARN "File $SERVER_ROLE_FILE is invalid!"
    msgprint2 214 $WARN "$SERVER_ROLE_FILE" 
    return 1
  fi

  eval $(cat "$SERVER_ROLE_FILE" | crypt -d "$SERVER_ROLE_KEY" |
    awk -F"=" ' {printf("%s=\"%s\" ", $1, $2) } ')

  #dumpinfo
 
  All_Primary_Hosts=""
  for host in $All_Server_Names; do
    eval $(print 'h=$HOST_'$host'_Primary')
    All_Primary_Hosts="$All_Primary_Hosts $h"
  done

  All_Secondary_Hosts=""
  for host in $All_Server_Names; do
    eval $(print 'h=$HOST_'$host'_Secondary')
    All_Secondary_Hosts="$All_Secondary_Hosts $h"
  done

  if [ "$(echo $All_Primary_Hosts)" == "" ] || [ "$(echo $All_Secondary_Hosts)" == "" ]; then
    msgprint $WARN "File $SERVER_ROLE_FILE has invalid content!"
    msgprint2 212 $WARN "$SERVER_ROLE_FILE" 
    return 1
  fi

  return 0
}


#
# Dump the config info to the log
#
dumpinfo()
{
  for host in $All_Server_Names; do
    eval "tag=HOST_"$host"_Primary"
    eval "value=\$HOST_"$host"_Primary"
    msgprint $TRACE "$tag=$value"
    msgprint2 102 $TRACE $tag $value
  done

  for host in $All_Server_Names; do
    eval "tag=HOST_"$host"_Secondary"
    eval "value=\$HOST_"$host"_Secondary"
    msgprint $TRACE "$tag=$value"
    msgprint2 102 $TRACE $tag $value
  done
}


#
# check if the Object database on host is read-only
# will check twice
# this check is performed on the database server not history server
#
# return 0 read-only
# return 1 if not read-only
# return 2 if not be able to determine
#
is_readonly_db()
{
  host=$1

  # read the dboption from the Object database
  #--------------------------------------------------
  dbquery $host msdb "EXEC master..sp_dboption 'Object'"
  if [ $? != 0 ]; then
    # failed first attempt
    sleep 1

    # try again
    dbquery $host msdb "EXEC master..sp_dboption 'Object'"
    if [ $? != 0 ]; then
      # give up
      return 2
    fi
  fi

  rc=$(cat "$DBQUERY_OUT")

  msgprint $TRACE "Object database option on $host: $rc."
  msgprint2 269 $TRACE $host "$rc"

  # check if SQL Object database is read-only
  if $(echo $rc | grep -iq -e "read only"); then
    return 0 
  fi
  
  return 1
}


#
# set source and target server variable  
# syntax: set_source_target $REVERSE 
#
set_source_target()
{
  fo_direction=$1

  if [ "$fo_direction" != TRUE ]; then

    # --- for FORWARD failover ---
    msgprint $INFO "Failover direction: Forward!"
    msgprint2 325  $INFO 

    # - Source -
    All_Source_Hosts=$All_Primary_Hosts
    for host in $All_Server_Names; do
      eval `print 'HOST_'$host'_Source=$HOST_'$host'_Primary'`
    done

    # - Target -
    All_Target_Hosts=$All_Secondary_Hosts
    for host in $All_Server_Names; do
      eval `print 'HOST_'$host'_Target=$HOST_'$host'_Secondary'`
    done

  else

    # --- for Reversed failover ---
    msgprint $INFO "Failover direction: Reversed!"
    msgprint2 326  $INFO 

    # - Source -
    All_Source_Hosts=$All_Secondary_Hosts
    for host in $All_Server_Names; do
      eval `print 'HOST_'$host'_Source=$HOST_'$host'_Secondary'`
    done
  
    # - Target -
    All_Target_Hosts=$All_Primary_Hosts
    for host in $All_Server_Names; do
      eval `print 'HOST_'$host'_Target=$HOST_'$host'_Primary'`
    done

  fi
}

WRT_STATE=""

#
#  use write_cmd begin <module> to start command writing process
#  use write_cmd text <text> to write text if the writing process begin and not end.
#  use write_cmd dash to write a dash line
#  use write_cmd blank to write a blank line
#  use write_cmd end to end the command writing process
#
#  will not write anything if type is text and text field is empty
#
write_cmd()
{
  type=$1
  shift 1
  text=$*

  if [ "$type" == "begin" ]; then
    WRT_STATE="begin"
    now=$(date "+"%Y"/"%m"/"%d" "%H":"%M":"%S"")
    print -r "#----------------------------------------------------------------" > "$FO_CMD_FILE"
    print -r "#  TBSM Version 3.2 $text Command File - $now ">> "$FO_CMD_FILE"
    print -r "#----------------------------------------------------------------" >> "$FO_CMD_FILE"
    print -r "#  --- Start of $text Process ---  " >> "$FO_CMD_FILE"
    print -r "#------------------------------------------------------------" >> "$FO_CMD_FILE"
  fi

  if [ "$type" == "end" ]; then
    WRT_STATE="end"
    print -r "#------------------------------------------------------------" >> "$FO_CMD_FILE"
    print -r "#  --- End of $text Process ---  " >> "$FO_CMD_FILE"
    print -r "#------------------------------------------------------------" >> "$FO_CMD_FILE"
  fi

  if [ "$type" == "dash" ]; then
    print -r "#------------------------------------------------------------" >> "$FO_CMD_FILE"
  fi

  if [ "$type" == "blank" ]; then
    print "" >> "$FO_CMD_FILE"
  fi

  if [ "$type" == "text" ]; then
    if [ "$WRT_STATE" != "begin" ]; then
      msgprint $WARN "Command file $FO_CMD_FILE is not in writing state, cannot write!"
      msgprint2 132 $WARN "$FO_CMD_FILE" 
      return 1
    fi

    if [ "$WRT_STATE" == "end" ]; then
      msgprint $WARN "Command file $FO_CMD_FILE has been closed, cannot write!"
      msgprint2 131 $WARN "$FO_CMD_FILE" 
      return 1
    fi

    if [ "$text" == "" ]; then
      # no text to write
      return 1
    fi
    print -r $text >> "$FO_CMD_FILE"
  fi

  return 0
}


# 
# routine to check the file is valid
# 
validateFile()
{
  file=$1

  if [ "$file" == "" ]; then
    msgprint $WARN "No file name specified!"
    msgprint2 262 $WARN 
    return 1
  fi

  if [ ! -e $file ]; then
    msgprint $WARN "File $file does not exist!"
    msgprint2 211 $WARN $file 
    return 1
  fi

  if [ ! -r $file ]; then
    msgprint $WARN "File $file is not readable!"
    msgprint2 215 $WARN $file 
    return 1
  fi

  if [ ! -s $file ]; then
    msgprint $WARN "File $file is empty!"
    msgprint2 213 $WARN $file 
    return 1
  fi
  
  return 0
}


#
# issue database query cmd on the database on a host
# syntax: dbquery <host> <dbname> <dbcmd>
#----------------------------------------------
dbquery()
{
  host=$1
  db=$2
  cmd=$(echo "$(shift 2; echo "$*")" | sed 's/"//g')

  # find dbuser and dbpassword based on dbserver
  #--------------------------------------------- 
  load_dbaccess
  dbuser=""
  dbpassword=""
  eval access=$(print '$DB_ACCESS_'$host)
  eval $(print $access | awk '{printf("dbuser=%s dbpassword=%s", $1, $2)}')

  if [ "$dbuser" == "" ]; then
    $SQLCMD -S$host -E -d$db -Q"$cmd" > "$DBQUERY_OUT"
  else
    $SQLCMD -S$host -U$dbuser -P$dbpassword -d$db -Q"$cmd" > "$DBQUERY_OUT"
  fi

  # issue the db command
  #----------------------

  # check the result
  #----------------------
  if $(tail -500 "$DBQUERY_OUT" | grep -iq \
    -e "failure" \
    -e "failed" \
    -e "Cannot open" \
    -e "terminating abnormally" \
    -e "network error" \
    -e "access denied" \
    -e "not created" \
    -e "Incorrect syntax" \
    -e "unknown option" \
    -e "could not find" \
    -e "SQLServerAgent Error" \
    -e "does not exist" \
    -e "usage: osql" \
    -e "Invalid object name" \
    ); then

    msgprint $WARN "SQL command failed => $cmd"
    msgprint2 309 $WARN "SQL $cmd"
    return 1
  fi   
  return 0
}


# look up SQL Server Version
#---------------------------
get_sqlversion()
{
  host=$1
  dbquery $host master "SELECT @@VERSION" 2>nul:

  if [ $? != 0 ]; then
    # print something 
    print $host
    return 1
  fi

  ver=$(cat "$DBQUERY_OUT" | awk '/Microsoft SQL Server / {print $4}')
  ##    -----   should not generate any string in this routine -----
  ##msgprint $TRACE "Host $host has SQL Server Version $ver."
  ##msgprint2 227 $TRACE $host $ver
  print $ver
}


beep()
{
  print "\007"
}


#
# look for the active site
#   if the primary site is active, then REVERSE is set to FALSE
#   if the secondary site is active, then REVERSE is set to TRUE
#   return 0 if REVERSE is set
#   return 1 if cannot set RESVERSE
# is_readonly_db returns 0 if read-only, 1 if not read-only, 2 if not determine
#
which_is_active()
{
  is_readonly_db $HOST_DatabaseServer_Primary
  readonly_on_Primary=$?

  is_readonly_db $HOST_DatabaseServer_Secondary
  readonly_on_Secondary=$?

  if [ "$readonly_on_Primary" == 2 ] || [ "$readonly_on_Secondary" == 2 ]; then 
    msgprint $WARN "Unknown Primary or Secondary Object database state."
    msgprint2 376 $WARN 
    return 1
  fi

  if [ "$readonly_on_Primary" == 0 ] && [ "$readonly_on_Secondary" == 0 ]; then
    msgprint $WARN "Both Primary and Secondary sites are read-only."
    msgprint2 124 $WARN 
    return 1
  fi

  if [ "$readonly_on_Primary" == 1 ] && [ "$readonly_on_Secondary" == 1 ]; then
    msgprint $WARN "Both Primary and Secondary sites are not read-only."
    msgprint2 123 $WARN 
    return 1
  fi

  # secondary is read only, therefore the primary is active
  if [ "$readonly_on_Secondary" == 0 ]; then
    REVERSE=FALSE
  else
    REVERSE=TRUE
  fi

  return 0
}


# 
# copy TEC server properties file from source to target
# the properties file is located in
#   $TivoliManager/AgentListener/config/<tecserver>.properties
# Syntax: do_copytecproperties <source> <x> <x> <target>
#
do_copytecproperties()
{
  s_host=$1
  t_host=$4

  s_instdir=$(find_tbsm_install_dir $s_host)
  if [ "$s_instdir" == "" ]; then
    msgprint $WARN "Failed to find INSTALLDIR of $s_host."
    msgprint2 170 $WARN $s_host
    return 1
  fi

  t_instdir=$(find_tbsm_install_dir $t_host)
  if [ "$t_instdir" == "" ]; then
    msgprint $WARN "Failed to find INSTALLDIR of $t_host."
    msgprint2 170 $WARN $t_host
    return 1
  fi

  if [ ! -e "//$s_host/$s_instdir/AgentListener/config" ]; then
    msgprint $INFO "No AgentListener configuration files to copy on $s_host."
    msgprint2 886 $INFO $s_host
    return 0
  fi

  cp -p "//$s_host/$s_instdir/AgentListener/config/"* "//$t_host/$t_instdir/AgentListener/config" 2>nul:
  if [ $? != 0 ]; then
    msgprint $WARN "Failed to copy AgentListener configuration files to $t_host."
    msgprint2 885 $WARN $t_host
    return 1
  fi

  msgprint $INFO "Successfully copied AgentListener configuration file from //$s_host/$s_instdir"
  msgprint2 887 $INFO "//$s_host/$s_instdir"

  return 0
}


#
# setAllHost - routine to set the server role name to HOST_<role>
#   when working only on Target or Source site 
#
setAllHost()
{
  if [ "$1" == "Target" ]; then
    for host in $All_Server_Names; do
      eval `print 'HOST_'$host'=$HOST_'$host'_Target'`
    done
  elif [ "$1" == "Source" ]; then
    for host in $All_Server_Names; do
      eval `print 'HOST_'$host'=$HOST_'$host'_Source'`
    done
  else
    msgprint $WARN "Unknown site $1.  Should either be Target or Source."
    msgprint2 378 $WARN $1  
    return 1
  fi

  return 0
}


#
# usage: gen_stopsvcs <site> either Target or Source
# if NoSync flag is turned on all the steps here will be generated as Optional
#
gen_stopsvcs()
{
  setAllHost $1
  if [ $? != 0 ]; then
    return 1
  fi

  if [ "$NoSync" == TRUE ]; then
    req_step=Opt
  else
    req_step=Req
  fi

  msgprint $INFO "Generating commands to stop $1 services."
  msgprint2 224 $INFO $1 

  write_cmd text "# Stop $1 services"
  write_cmd dash

  # stop Health Monitor
  #---------------------------------------
  write_cmd text ${HOST_HmServer:+$HOST_HmServer Opt svcstop ASIHealthMonitor} 

  # stop XML Toolkit 
  #---------------------------------------
  for host in $(hostsin $HOST_XMLToolkit)
  do
    write_cmd text "$host Opt svcstop ASIXMLToolkitSvc" 
  done

  # stop IIS
  #---------------------------------------
  write_cmd text ${HOST_WebServer:+$HOST_WebServer Opt svcstop W3SVC} 

  # stop SNA Server 
  #---------------------------------------
  write_cmd text ${HOST_SnaServer:+$HOST_SnaServer $req_step svcstop SnaServr} 

  # stop MVS IPListener service
  #---------------------------------------
  write_cmd text "$HOST_DatabaseServer Opt svcstop ASIMVSIPListenerSvc " 

  # stop MVS IP OS Listener service
  #---------------------------------------
  for host in $(hostsin $HOST_EhServer)
  do
    write_cmd text "$host Opt svcstop ASIMVSIPOSListenerSvc " 
  done

  # stop Sender Service Server 
  #---------------------------------------
  for host in $(hostsin $HOST_SsServer)
  do
    write_cmd text "$host Opt svc_Ss stop $BATCHSVC_STOP_TIMER" 
  done

  # stop Event Handler Server 
  #---------------------------------------
  for host in $(hostsin $HOST_EhServer) 
  do
    write_cmd text "$host Opt svc_Fr stop $BATCHSVC_STOP_TIMER" 
    write_cmd text "$host Opt svc_Eh stop $BATCHSVC_STOP_TIMER" 
  done

  # stop AgentListener Server 
  #---------------------------------------
  write_cmd text ${HOST_AgentListener:+$HOST_AgentListener Opt svcstop ASIAgentListenerSvc} 
  write_cmd text ${HOST_AgentListener:+$HOST_DatabaseServer Opt svcstop ASIEventEnablement}
  write_cmd text ${HOST_AgentListener:+$HOST_DatabaseServer Opt svcstop ASITaskServer} 

  for host in $(hostsin $HOST_AgentListener $HOST_TGMTaskServer)
  do
    write_cmd text "$host Opt svcstop ASITaskServer " 
  done

  # stop TGM Task Server 
  #---------------------------------------
  for host in $(hostsin $HOST_TGMTaskServer)
  do
    write_cmd text "$host Opt svcstop ASIEventEnablement " 
  done

  # stop Common Listener 
  #---------------------------------------
  write_cmd text ${HOST_CommonListener:+$HOST_CommonListener Opt svcstop ASICommonlistener} 

  # stop TEC Listener 
  #---------------------------------------
  write_cmd text ${HOST_TECListener:+$HOST_TECListener Opt svcstop ASITECListenerSvc} 

  # stop EIF Sender 
  #---------------------------------------
  write_cmd text ${HOST_EIFSender:+$HOST_EIFSender Opt svcstop ASIEIFSender} 

  # shut down feed data processing
  #---------------------------------------
  write_cmd text "$HOST_DatabaseServer $req_step svcstop ASIStagedEventLoader " 

  # stop MVS UpLoadRule Server 
  #---------------------------------------
  write_cmd text ${HOST_UpLoadRuleServer:+$HOST_UpLoadRuleServer Opt svcstop ASIMVSUploadRuleSvc} 

  # stop ASIEnqueueProxyServer
  #---------------------------------------
  for host in $(hostsin $HOST_EhServer $HOST_SsServer)
  do
    write_cmd text "$host Opt svcstop ASIEnqueueProxyServer " 
  done

  # stop ASIRemoteExecutionServer
  #---------------------------------------
  for host in $(hostsin $HOST_EhServer)
  do
    write_cmd text "$host Opt svcstop ASIRemoteExecutionServer " 
  done

  # stop WAS last
  #-------------------
  for host in $(hostsin $HOST_WASServer)
  do
    write_cmd text "$host Opt stopwas $PARM_WAS_WaitTime" 
  done

  # stop ASIDBValidater
  #---------------------------------------
  for host in $(hostsin $HOST_DatabaseServer $HOST_EhServer $HOST_SsServer)
  do
    write_cmd text "$host Req svcstop ASIDBValidater" 
  done

  return 0
}


#
#  usage: gen_configsvcs <site> <state> 
#    <site> should be either Target or Source
#    <state> can be auto disabled 
#
gen_configsvcs()
{
  setAllHost $1
  if [ $? != 0 ]; then
    return 1
  fi

  if [ "$2" != "auto" ] && [ "$2" != "disabled" ] && [ "$2" != "demand" ]; then
    msgprint $WARN "Unsupported state $2.  Should either be auto, demand or disabled."
    msgprint2 381 $WARN $2
    return 1
  else
    svc_state=$2
  fi
  
  write_cmd text "# Configure $1 services to $svc_state"
  write_cmd dash

  msgprint $INFO "Generating commands to configure $1 services to $svc_state."
  msgprint2 222 $INFO $1 $svc_state

  for host in $(hostsin $HOST_DatabaseServer $HOST_EhServer $HOST_SsServer)
  do
    write_cmd text "$host Opt svcconfig ASIDBValidater $svc_state" 
  done

  for host in $(hostsin $HOST_WASServer)
  do
    write_cmd text "$host Opt configwas $svc_state"
  done

  write_cmd text "$HOST_DatabaseServer Opt svcconfig ASIStagedEventLoader $svc_state" 
  write_cmd text "$HOST_DatabaseServer Opt svcconfig ASIMVSIPListenerSvc $svc_state" 
  write_cmd text ${HOST_AgentListener:+$HOST_DatabaseServer Opt svcconfig ASIEventEnablement $svc_state} 
  write_cmd text ${HOST_AgentListener:+$HOST_DatabaseServer Opt svcconfig ASITaskServer $svc_state} 

  for host in $(hostsin $HOST_EhServer $HOST_SsServer)
  do
    write_cmd text "$host Opt svcconfig ASIEnqueueProxyServer $svc_state" 
  done

  for host in $(hostsin $HOST_EhServer)
  do
    write_cmd text "$host Opt svcconfig ASIRemoteExecutionServer $svc_state" 
  done

  for host in $(hostsin $HOST_TGMTaskServer)
  do
    write_cmd text "$host Opt svcconfig ASITaskServer $svc_state" 
    write_cmd text "$host Opt svcconfig ASIEventEnablement $svc_state" 
  done

  for host in $(hostsin $HOST_SsServer)
  do
    for svc in $(sclist \\\\$host | awk '/ASIMVS(|[I][P])Sender/ { print $2 }')
    do
      write_cmd text "$host Opt svcconfig $svc $svc_state" 
    done
  done

  for host in $(hostsin $HOST_EhServer)
  do
    for svc in $(sclist \\\\$host | awk '/ASIMVSEventHandler/ { print $2 }')
    do
      write_cmd text "$host Opt svcconfig $svc $svc_state" 
    done

    if [ "$svc_state" == "disabled" ]; then
      for svc in $(sclist \\\\$host | awk '/ASIFileReceiver/ { print $2 }')
      do
        write_cmd text "$host Opt svcconfig $svc $svc_state" 
      done
    fi
  done

  for host in $(hostsin $HOST_EhServer)
  do
    write_cmd text "$host Opt svcconfig ASIMVSIPOSListenerSvc $svc_state" 
  done

  write_cmd text ${HOST_SnaServer:+$HOST_SnaServer Opt svcconfig SnaServr $svc_state} 
  write_cmd text ${HOST_UpLoadRuleServer:+$HOST_UpLoadRuleServer Opt svcconfig ASIMVSUploadRuleSvc $svc_state} 
  write_cmd text ${HOST_WebServer:+$HOST_WebServer Opt svcconfig W3SVC $svc_state} 
  write_cmd text ${HOST_XMLToolkit:+$HOST_XMLToolkit Opt svcconfig ASIXMLToolkitSvc $svc_state} 
  write_cmd text ${HOST_HmServer:+$HOST_HmServer Opt svcconfig ASIHealthMonitor $svc_state} 
  write_cmd text ${HOST_AgentListener:+$HOST_AgentListener Opt svcconfig ASIAgentListenerSvc $svc_state} 
  write_cmd text ${HOST_CommonListener:+$HOST_CommonListener Opt svcconfig ASICommonlistener $svc_state} 
  write_cmd text ${HOST_TECListener:+$HOST_TECListener Opt svcconfig ASITECListenerSvc $svc_state} 
  write_cmd text ${HOST_EIFSender:+$HOST_EIFSender Opt svcconfig ASIEIFSender $svc_state} 
  return 0
}


find_tbsm_install_dir()
{
  host=$1
  i=$(reg query "\\\\$host\HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" 2>nul: | awk '/InstallDirectory/ { gsub(/\\/, "/"); print $3 }' | sed 's/:/$/g')

  print $i
}



find_logs_dir()
{
  host=$1
  l=$(reg query "\\\\$host\HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\Log" 2>nul: | awk '/Directory/ { gsub(/\\/, "/"); print $3 }' | sed 's/:/$/g')

  print $l
}


find_queues_dir()
{
  host=$1
  q=$(reg query "\\\\$host\HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\EnqueueProxyServer\QueueConfig" 2>nul: | awk '/Directory/ { gsub(/\\/, "/"); print $3 }' | sed 's/:/$/g')

  print $q
}


find_was_install_dir()
{
  host=$1

  w=$(reg query "\\\\$host\HKEY_LOCAL_MACHINE\SOFTWARE\IBM\WebSphere Application Server\6.0.0.0" 2>nul: | awk '/InstallLocation/ { gsub(/\\/, "/"); sub(/^ /, "", $3); print $3, $4, $5, $6}'| sed 's/:/$/g')

  wd="$(print "$w" | awk '{gsub(/ $/, ""); print $0}')" 
  print $wd
}


findWasNode()
{
  host=$1

  wasInstallDir="$(find_was_install_dir $host)"
  if [ "$wasInstallDir" == "" ]; then
    print ""
    return
  fi

  w=$(cat "//$host/$wasInstallDir/properties/profileRegistry.xml" | grep " name=\""tbsm\"" path=")
  pfilePath=$(print -r $w | awk -F"\"" '{print $6}' | sed 's/:/$/g')
  if [ "$pfilePath" == "" ]; then
    print ""
    return
  fi

  nodeName=$(cat "//$host/$pfilePath"/bin/setupCmdLine.bat | grep WAS_NODE | awk -F"=" '{print $2}')
  print $nodeName
}


findWasService()
{
  host=$1

  name="$(sclist \\\\$host | awk '/IBMWAS61Service - tbsm31/ {print $2,$3,$4}' )"
  if [ "$name" != "" ]; then
    print "$name"
    return 0
  fi

  name="$(sclist \\\\$host | awk '/IBMWAS6Service - tbsm31/ {print $2,$3,$4}' )"
  if [ "$name" != "" ]; then
    print "$name"
    return 0
  fi

  # for backward compatible
  name="$(sclist \\\\$host | awk '/IBMWAS5Service - server1/ {print $2,$3,$4}' )"

  print "$name"
}
