#------------------------------------------------- {COPYRIGHT-TOP-RM}
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
#  Copyright IBM Corp. 1999,2007 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#------------------------------------------------- {COPYRIGHT-END-RM}

# File: archive/scripts/makemvscomponentsdetail.ksh, tbsm.sql, tbsm_base31e
# SID: 1.11.2.6
# Date modified: 07/05/10 04:00:15
#
# Module Description:
#
# MakeMVSComponents.ksh
#
# Goal: Setup a new installation of TMOS390 with Event Handler, Listener, and File Receiver for OS specified.
#
# usage: see MakeMVSComponents.ksh -?
#
# OA20684: For non-English OS, use sc.exe return codes instead of Error strings
#  1056: Already running
#  1060: Service does not exist
#  1062: Already stopped
#  1072: The specified service has been marked for deletion
#  1722: Open SC Manager Failed
#  1078: CreateService Failed. Name already exists
#  1073: CreateService Failed. Service already exists
#    87: CreateService Failed. Parameter incorrect.

num_params=$#;
component_listing='ALL'
bInstall_EH=0;
bInstall_LS=0;
bInstall_SS=0;
bInstall_FR=0;
bRestart=0;
os_id="";
cellsize=1024;
maxentries=32000;
cellsizeu=4096;
maxentriesu=32768;

SenderSNA=1;  # Assume already installed
SenderIP=1;  # Assume already installed
ListenerSNA=1;  # Assume already installed
ListenerIP=1;  # Assume already installed

# Get Database Server, User, Password from Registry
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

# default values
bTrusted=0
bRemoveComponents=0
bForceDatabaseSettings=0
TMP_LOCALDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\\\\\\\\/g' | sed 's/\\\\\\\\$//g')
TMP_INSTALLDIR='//'$COMPUTERNAME'/'$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g' | sed 's?:?$?g' | sed 's/\/$//g')
UPLOAD_MODENAME='LU62PS';
local_host="$(print "$COMPUTERNAME" | awk '{print toupper($0);}')";
mvs_host="$local_host"
hms_host="$local_host"

usage   ()
{
  sNameScript="MakeMVSComponents.ksh"

  print ''
  print 'Usage: '$sNameScript' -B<base_name> -O<os_path> [options...]'
  print ''
  print '  Required Parameters(s) - All Components:'
  print '  -B<base_name>           Base name of created NT Services.  e.g. \047Tivoli BSM MVSEventHandlerSvc-<base_name>\047'
  print '  -O<os_path>             MVS Operating System Path (<os_name> or full/partial:\047<ENT/COMP/MACH/LPAR/OS>\047)'
  print ''
  print '  Additional Required Parameters(s) for Removing All Components:'
  print '  -r                      Remove all MVS components specified by <base_name> and <os_path>'
  print ''
  print '  Required SNA Parameters'
  print '    Additional Required Parameters(s) for Listener Services:'
  print '    -d<download_LU_name>    MVS Listener Service Partner LU (download).'
  print '    -t<local_LU_name>       Local LU name'
  print ''
  print '    Additional Required Parameters(s) for Sender Services:'
  print '    -x<upload_LU_name>      MVS Sender Service Partner LU (upload).'
  print ''
  print '  Required IP Parameters'
  print '    -v<download_ip_port>       MVS Listener Service IP port (download).'
  print '                               port must be the same value as defined in the TCPIP_PORT '
  print '                               control card of the MVS Object Server'
  print '                               (this value should be the same for all -v entries)'
  print ''
  print '    -w<upload_ip_address:port> MVS Sender Service IP address and port (upload).'
  print '                               port must be the same value as defined in the TCPIP_SERVER_PORT '
  print '                               control card of the MVS Object Server.  This is a required'
  print '                               parameter for TCP/IP even when not installing the Sender'
  print '                               because the address is required by download too.'
  print ''
  print '  Optional Parameters(s):'
  print '  -S<db_server>           Specifies server name where database resides.  Default=['$db_server'].'
  print '  -U<db_user>             Specifies database user name.  Default=['$db_user'].'
  print '  -P<db_password>         Specifies password for database user.  Default=['$db_password'].'
  print '  -E                      Use trusted database connection. Use this instead of -U<db_username> and -P<db_password>'
  print '  -I<install_dir>         UNC Remote Directory Name.  Default=['$TMP_INSTALLDIR']'
  print '  -L<local_dir>           Local Directory Name.  Default=['$TMP_LOCALDIR']'
  print '  -H<mvs_host>            Hostname of MVSEventHandlers.  Default=['$mvs_host']'
  print '  -m<upload_mode>         MVS Sender Service Modename (upload only).  Default=['$UPLOAD_MODENAME']'
  print '  -f                      Force the inclusion of a Database Settings registry key under'
  print '                          each component instead of using the default database settings'
  print '                          on the MVS Host ('$mvs_host').'
  print '  -c<component_listing>   Comma-separated list of components to install.  Default is "ALL".'
  print '                          See list below for valid component codes.  Note that the [-r] option'
  print '                          is not affected by this option.'
  print '  -A                      Activate services for SNA or IP listener, EventHandler, and Sender Services'
  print '  -g<download_cellsize>   override the download queue cellisze. default is 1024'
  print '  -j<download_maxentries> override the download queue maxentries. default is 32000'
  print '  -k<upload_cellsize>     override the upload queue cellisze. default is 4096'
  print '  -n<upload_maxentries>   override the upload maxentries. default is 32768'
  print ''
  print '  -y<systemid>            The netid.sscpname of the Host basename. TBSM provides a program named'
  print '                          GTMINFO that may be run from TSO to obtain the netid.sscpname.'
  print '  -M<HealthMonitor_host>  Hostname of Health Monitor server.  Default=['$hms_host']'
  print '  -W<Windows account>     Windows account for trusted access by Event Handler or ' 
  print '                          File Receiver services. Required for trusted connection.'
  print '  -D<Windows password>    Windows account password for trusted access by Event Handler or'
  print '                          File Receiver services. Required for trusted connection.'
  print ''
  print '  Component Codes:'
  print '  ALL - All 4 components  (EH,LS,SS,FR)'
  print '  EH  - Event Handlers    (Tivoli BSM MVSEventHandlerSvc)'
  print '  LS  - Listener Services (Tivoli BSM MVSListenerSvc)'
  print '  SS  - Sender Services   (Tivoli BSM MVSSenderSvc)'
  print '  FR  - File Receivers    (Tivoli BSM FileReceiverSvc)'
  print ''
  print ''
  print '  Examples:'
  print ''
  print '  1) Create all SNA-based MVS components for an OS named E003:'
  print '      '$sNameScript' -BE003 -OE003 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390'
  print ''
  print '     Create all TCP/IP-based MVS components for an OS named E003:'
  print '      '$sNameScript' -BE003 -OE003 -wHOST01.RALEIGH.IBM.COM:1023 -v1022'
  print ''
  print '  2) Create all SNA-based MVS components for two OS instances named CPUA (same SMFID) located on different MVS host machines, M1 and M2:'
  print '      '$sNameScript' -BM1CPUA -O\047M1/LPARA/CPUA\047 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390'
  print '      '$sNameScript' -BM2CPUA -O\047M2/LPARA/CPUA\047 -HASIMVS -L\047C:\\\\TivoliManager\047 -I\047//ASIMVS/C$/TivoliManager\047 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390' -mLU62PS
  print ''
  print '  3) Create SNA-based MVS components for an OS named with partially qualified (MACH/LPAR/OS) path \047M1/LPARA/CPUA\047:'
  print '      '$sNameScript' -BE003 -O\047M1/LPARA/CPUA\047 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390'
  print ''
  print '  4) Create SNA-based MVS components for an OS named with fully qualified (ENT/COMP/MACH/LPAR/OS) path \047Tivoli Systems/Eastern/M1/LPARA/CPUA\047:'
  print '      '$sNameScript' -BE003 -O\047Tivoli Systems/Eastern/M1/LPARA/CPUA\047 -dUSASI01.NJ1TM390 -tNJUS47A -xNJ1TM390'
  print ''
  print '  5) Remove all MVS components for an OS named E003:'
  print '      '$sNameScript' -BE003 -OE003 -r'
  print ''
  print '  6) Remove all MVS components for an OS named M2CPUA on M2/LPARA/CPUA:'
  print '      '$sNameScript' -BM2CPUA -O\047M2/LPARA/CPUA\047 -r'
  print ''
  print '  7) Install only SNA-based Event Handler and Listener Services for an OS named M2CPUA on M2/LPARA/CPUA:'
  print '      '$sNameScript' -BM2CPUA -O\047M2/LPARA/CPUA\047 -cEH,LS -dUSASI01.NJ1TM390 -tNJUS47A'
  print ''
  print '  8) Install only TCP/IP-based Event Handler and Listener Services for an OS named M2CPUA on M2/LPARA/CPUA:'
  print '      '$sNameScript' -BM2CPUA -O\047M2/LPARA/CPUA\047 -cEH,LS -wHOST01.RALEIGH.IBM.COM:1023 -v1022'
  print ''
  print '  9) Install only SNA-based Sender Service for an OS named M2CPUA on M2/LPARA/CPUA:'
  print '      '$sNameScript' -BM2CPUA -O\047M2/LPARA/CPUA\047 -cSS -xNJ1TM390'
  print ''
  print '  10) Install only TCP/IP-based Sender Service for an OS named M2CPUA on M2/LPARA/CPUA:'
  print '      '$sNameScript' -BM2CPUA -O\047M2/LPARA/CPUA\047 -cSS -wHOST01.RALEIGH.IBM.COM:1023'
  print ''
}

while getopts "H:I:L:U:S:B:O:P:M:W:D:x:d:m:t:c:v:w:g:j:k:n:y:AErf?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  H)  mvs_host=$(echo $OPTARG | awk '{print toupper($0);}');;
  L)  LOCALDIR=$OPTARG;;
  I)  INSTALLDIR=$OPTARG;;
  B)  BASENAME=$OPTARG;;
  O)  OSPATH=$OPTARG;;
  t)  LOCAL_LUNAME=$(echo $OPTARG | awk '{print toupper($0);}');;
  x)  UPLOAD_LUNAME=$(echo $OPTARG | awk '{print toupper($0);}');;
  d)  if [ "$c" = "d" ]; then
        DOWNLOAD_LUNAME=$(echo $OPTARG | awk '{print toupper($0);}')
      elif [ "$c" = "D" ]; then
        wPassword_opt="$OPTARG"
      fi;;
  m)  if [ "$c" = "m" ]; then
        UPLOAD_MODENAME=$(echo $OPTARG | awk '{print toupper($0);}')
      elif [ "$c" = "M" ]; then
        hms_host=$(echo $OPTARG | awk '{print toupper($0);}')
      fi;;
  v)  DOWNLOAD_IPPORT=$OPTARG;;
  w)  if [ "$c" = "w" ]; then
        UPLOAD_IPNAME=$(echo $OPTARG | awk -F":" '{print toupper($1);}');
        UPLOAD_IPPORT=$(echo $OPTARG | awk -F":" '{print toupper($2);}');
      elif [ "$c" = "W" ]; then
        wUser_opt="$OPTARG"
      fi;;
  E)  bTrusted=1;;
  r)  bRemoveComponents=1;;
  f)  bForceDatabaseSettings=1;;
  c)  component_listing=$OPTARG;;
  A)  bRestart=1;;
  g)  cellsize=$OPTARG;;
  j)  maxentries=$OPTARG;;
  k)  cellsizeu=$OPTARG;;
  n)  maxentriesu=$OPTARG;;
  y)  systemid=$OPTARG;;
 \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done
shift $OPTIND-1

# Use trusted connection if db_user is empty
if [ -z "$db_user" ]; then
  bTrusted=1
fi

process_component_listing()
{
  if [ "$component_listing" = "ALL" ]; then;
    component_listing="EH,LS,SS,FR"
  fi
  echo "Validating component code(s): \c"
  for component_type in $(print "${component_listing}" | awk 'BEGIN {RS=","} {print}'); do
  {
    echo "[$component_type] \c";
    case $component_type in
    EH) bInstall_EH=1;;
    LS) bInstall_LS=1;;
    SS) bInstall_SS=1;;
    FR) bInstall_FR=1;;
    *)  print >&2 "\nERROR: invalid component type [$component_type] in component listing [$component_listing]"; usage; exit 1;;
    esac
  }
  done;
  print "";
}


check_nt_res_kit()
{
  filename="${1?"ERROR: check_nt_res_kit() missing required field <filename>"}"
  
  # check for utility
  rc=`which "$filename"`; rc=$?
  if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find "$filename".  Please install Windows NT Resource Kit Supplement 3" >&2
  echo "       or later and ensure "$filename" is in the path." >&2
    exit 1
  fi
}

check_for_required_parameters()
{
  # check for required parameters
  if [ -z "$BASENAME" ]; then
    echo "ERROR: Missing required parameter -B<base_name>" >&2
    usage >&2
    exit 2
  fi
  if [ -z "$OSPATH" ]; then
    echo "ERROR: Missing required parameter -O<os_path>" >&2
    usage >&2
    exit 2
  fi

  rc=$(sc \\\\$mvs_host query "ASIMVSSenderSvc-$BASENAME" 2>&1)
  if $(print "$rc" | grep -iq -e "1060:" -e "1722:"); then;
    SenderSNA=0
  fi

  rc=$(sc \\\\$mvs_host query "ASIMVSIPSenderSvc-$BASENAME" 2>&1)
  if $(print "$rc" | grep -iq -e "1060:" -e "1722:"); then;
    SenderIP=0
  fi

  ListenerSNAX=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components\ASIMVSListenerSvcInstances$BASENAME" -n "PartnerLU"  2>nul: | grep PartnerLU) 
  if [ -z "${ListenerSNAX}" ]; then
    ListenerSNA=0
  fi
  ListenerIPX=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\ComponentsASIMVSIPOSListenerSvc\Instances\$BASENAME" -n "PartnerID" 2>nul: | grep PartnerID) 
  if [ -z "${ListenerIPX}" ]; then
    ListenerIP=0
  fi

  if [ $bRemoveComponents -eq 0 ]; then;
    if [ $bInstall_LS -eq 1 ]; then
      # check if we are defining a SNA or IP connection
      if [ -z "${DOWNLOAD_LUNAME}" -a -z "${LOCAL_LUNAME}" -a -z "${DOWNLOAD_IPPORT}" ]; then
        echo "ERROR: Missing required parameters SNA: -d<download_LU_name> -t<local_LU_name>" >&2
        echo "ERROR: Missing required parameters IP:  -v<download_ip_port>" >&2
        usage >&2
        exit 2
      fi

      if [ ! -z "${DOWNLOAD_LUNAME}"  -o  ! -z "${LOCAL_LUNAME}"  ]; then
        if  [ ! -z "${DOWNLOAD_IPPORT}" ]; then
          echo "ERROR: Ambiguous parameters -d<download_LU_name>, -t<local_LU_name> and -v<download_ip_port>" >&2
          usage >&2
          exit 2
        fi
      fi

      # check to see if either SNA parameter is defined - if yes lets assume SNA
      if [ ! -z "${DOWNLOAD_LUNAME}"  -o  ! -z "${LOCAL_LUNAME}" ]; then
        if [ -z "${DOWNLOAD_LUNAME}" ]; then
          echo "ERROR: Missing required parameter -d<download_LU_name>" >&2
          usage >&2
          exit 2
        fi
        if [ -z "${LOCAL_LUNAME}" ]; then
          echo "ERROR: Missing required parameter -t<local_LU_name>" >&2
          usage >&2
          exit 2
        fi
        if [ $ListenerIP -eq 1 ]; then   #  IP is already defined
          echo "ERROR: $BASENAME already has an IP Listener defined"
          exit 2
        fi
      fi

      if [ ! -z "${DOWNLOAD_IPPORT}" ]; then
        if [ -z "${UPLOAD_IPPORT}" -o -z "${UPLOAD_IPNAME}" ]; then
          echo "ERROR: Missing required parameter -w<upload_ip_address:port>" >&2
          usage >&2
          exit 2
        fi
        if [ $ListenerSNA -eq 1 ]; then   #  SNA is already defined
          echo "ERROR: $BASENAME already has an SNA Listener defined"
          exit 2
        fi
      fi
    fi


    if [ $bInstall_SS -eq 1 ]; then
      # check if we are defining a SNA or IP connection
      if [ -z "${UPLOAD_LUNAME}"  -a  -z "${UPLOAD_IPNAME}" -a  -z "${UPLOAD_IPPORT}" ]; then
        echo "ERROR: Missing required parameters SNA: -x<upload_LU_name>" >&2
        echo "ERROR: Missing required parameters IP:  -w<upload_ip_address:port>" >&2
        usage >&2
        exit 2
      fi
      # check to see if SNA parameter is defined - if yes lets assume SNA
      if [ ! -z "${UPLOAD_LUNAME}" ]; then 
        if [ ! -z "${UPLOAD_IPNAME}" -o ! -z "${UPLOAD_IPPORT}" ]; then
          echo "ERROR: Ambiguous parameters -x<upload_LU_name> and -w<upload_ip_address:port>" >&2
          usage >&2
          exit 2
        fi
        if [ $SenderIP -eq 1 ]; then   #  IP is already defined
          echo "ERROR: $BASENAME already has an IP Sender defined"
          exit 2
        fi

      fi
      if [ ! -z "${UPLOAD_IPNAME}"  -o  ! -z "${UPLOAD_IPPORT}" ]; then
        if [ -z "${UPLOAD_IPNAME}" ]; then
          echo "ERROR: Missing required parameter (upload_IPAddr) -w<upload_ip_address:port>" >&2
          usage >&2
          exit 2
        fi
        if [ -z "${UPLOAD_IPPORT}" ]; then
          echo "ERROR: Missing required parameter (port) -w<upload_ip_address:port>" >&2
          usage >&2
          exit 2
        fi
        if [ $SenderSNA -eq 1 ]; then   #  SNA is already defined
          echo "ERROR: $BASENAME already has an SNA Sender defined"
          exit 2
        fi
      fi
    fi
  else
    if [ $num_params -lt 3 ]; then
      echo "ERROR: Missing required parameter(s)" >&2
      usage >&2
      exit 5
    fi
  fi
  check_nt_res_kit REGFIND.EXE
  check_nt_res_kit REGINI.EXE
  check_nt_res_kit SC.EXE
  check_nt_res_kit REGDMP.EXE
  check_nt_res_kit REG.EXE

  # if local or install directory not specified, look in the registry of host <mvs_host>
  if [ -z "$LOCALDIR" ]; then;
    LOCALDIR=$(REGFIND -m '\\'"$mvs_host" -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\\\\\\\\/g' | sed 's/\\\\\\\\$//g')
  fi
  if [ -z "$INSTALLDIR" ]; then;
    INSTALLDIR='//'"$mvs_host"'/'$(REGFIND -m '\\'"$mvs_host" -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g' | sed 's?:?$?g' | sed 's/\/$//g')
  fi

  # get rid of potential extra backslash ('\')
  LOCALDIR=$(print -r "$LOCALDIR" | sed 's/\\\\/\\/g')

  # check for OSQL utility
  rc=`which OSQL.exe`; rc=$?
  if [ $rc -ne 0 ]; then;
    echo "ERROR: Could not find OSQL.EXE.  Please install SQL Client Utilities." >&2
    exit 1
  fi

  if [ -z "$INSTALLDIR" ]; then
    echo "ERROR: Could not access directory [$INSTALLDIR]" >&2
    usage >&2
    exit 3
  fi

  if [ ! -d "$INSTALLDIR" ]; then
    echo "ERROR: Could not access directory [$INSTALLDIR]" >&2
    exit 4
  fi

  if [ -z "$LOCALDIR" ]; then
    echo "ERROR: Could not access directory [$LOCALDIR]" >&2
    usage >&2
    exit 3
  fi

  if [ ! -d "$LOCALDIR" ]; then
    echo "ERROR: Could not access directory [$LOCALDIR]" >&2
    exit 4
  fi
  
  # For trusted mode, the EventHandler and FileReceiver require the Windows account
  # and password for their services configuration
  if [ $bRemoveComponents -eq 0 ]; then;
    if [ $bTrusted -eq 1 ]; then
      if [ $bInstall_EH -eq 1 ] || [ $bInstall_FR -eq 1 ]; then;
        if [ -z "$wUser_opt" ] || [ -z "$wPassword_opt" ]; then
          echo "ERROR:  Windows account and password required to configure services with trusted mode"
          exit 2
        fi
      fi
    fi    
  fi
}


validate_os_path()
{
  # Lookup the OS id based on the path
  echo "Using Database Host=[$db_server], MVS Host=[$mvs_host]"
  print "$OSPATH" | awk -F'/' 'BEGIN {
    tables[1]="OS_C";
    tables[2]="LPAR_C";
    tables[3]="MACHINE_C";
    tables[4]="COMPLEX_C";
    tables[5]="ENT_C";

    cids[1]="OS";
    cids[2]="LPAR";
    cids[3]="MACH";
    cids[4]="COMP";
    cids[5]="ENT";
  }
  {
    if (NF > 5) {
      print "ERROR: OS path cannot have more than 5 fields. It must be of the form"
      print "  ENT/COMPLEX/MACHINE/LPAR/OS"
      print "The path does not have to be fully specified. If the OS name is unique"
      print "then only the OS name is required. If OS names are unique by machine"
      print "then the path may be specified as MACHINE/LPAR/OS."
    }
    sql="SET NOCOUNT ON\nDECLARE @rc INT\nSELECT CONVERT(BINARY(3),t1.id)\nFROM OS_V AS t1";
    if ( NF == 1 ) {
      sql=sprintf("%s\n  JOIN link as link0 on link0.dst_cid = '\''OS'\'' AND link0.dst_id = t1.id AND link_type='\''PHYC'\'' AND link0.deleted = 0",
                  sql);
    }
    for (i = 2; i <= NF; i++) {
      idx=i-1;
      table=tables[i];
      sql=sprintf("%s\n  JOIN link AS link%d ON link%d.src_cid='\''%s'\'' AND link%d.dst_cid='\''%s'\'' AND link%d.dst_id=t%d.id AND link%d.deleted=0",
                  sql, idx, idx, cids[i], idx, cids[i-1], idx, idx, idx);
      sql=sprintf("%s\n  JOIN %s AS t%d ON t%d.id = link%d.src_id AND t%d.name=N'\''%s'\''\n",
                  sql, tables[i], i, i, idx, i, $(NF-(idx)));
    }
    sql = sprintf("%s\nWHERE t1.name = '\''%s'\''", sql, $NF);
    print sql;
    print "SELECT @rc=@@ROWCOUNT"
    print "IF @rc=0"
    print "  RAISERROR(\047ERROR: no matching OS id found\047, 11, 1)"
    print "ELSE IF @rc>1"
    print "  RAISERROR(\047ERROR: duplicate non-deleted OS ids found\047, 11, 1)"
  }
  ' > "$0.IN.SQL"
  if $(cat "$0.IN.SQL" | grep -v RAISERROR | grep -q ERROR:); then
    echo '' >&2
    echo 'Error: processing OS path' >&2
    cat "$0.IN.SQL" | grep -v RAISERROR | grep ERROR: >&2
    echo '' >&2
    echo '*** SQL CODE ********************************' >&2
    cat "$0.IN.SQL" >&2
    echo '*** SQL CODE ********************************' >&2
    exit 1
  fi

  # obtain the OS id from the database
  if [ $bTrusted -eq 1 ]; then
    SQL_CMD_LINE="OSQL -S$db_server -E -n -w3000 -dObject";
  else
    SQL_CMD_LINE="OSQL -S$db_server -U$db_user -P$db_password -n -w3000 -dObject";
  fi

  OSID=$($SQL_CMD_LINE -i"$0.IN.SQL" |
    awk '
    /^[ ]*0x/ {
      gsub(/[ ]*/,"");
      sub(/^0x/,"");
      os_id=sprintf("%s0006\n", toupper($0));
    }
    /ERROR/ {
      err_str=$0;
    }
    END {
      if (err_str=="")
        print os_id
      else
        print err_str
    }
  ')
  rm -f "$0.IN.SQL"

  # check for errors in isolating OSID
  if $(print "$OSID" | grep -iq ERROR); then
    if [ $bRemoveComponents -eq 1 ]; then;
      OSID="0000000006"
    else
      print "$OSID" for [$OSPATH] >&2
      exit 5
    fi
  fi

  # check for missing OSID
  if [ -z "$OSID" ]; then
    echo "ERROR: Could not find id from OS $OSNAME" >&2
    exit 6
  fi
  echo 'Found OS id='$OSID' for ['$OSPATH']'
  os_id="0x"${OSID%0006}
}

delete_service()
{
  svc_name="${1?"ERROR: delete_service() missing required parameter <svc_name>"}"

  echo 'Deleting NT service ['$svc_name']...\c'
  rc=$(sc \\\\$mvs_host delete "$svc_name" 2>&1)
  if $(print "$rc" | grep -iq  ":"); then; #All Errors return the : character.

    if $(print "$rc" | grep -iq "1060:"); then #1060=The specified service does not exist as an installed service.
      echo 'OK (already deleted)'
    elif $(print "$rc" | grep -iq "1072:"); then #1072 - The specified service has been marked for deletion.
      echo 'OK (already marked for deletion; delete will occur on next re-boot)'
    else
      print "FAILURE: $rc"
    fi
  else
    echo 'OK'
  fi
}

delete_registry_entry()
{
  svc_type="${1?"ERROR: delete_registry_entry() - missing required parameter <svc_type> "}"
  svc_base="${2?"ERROR: delete_registry_entry() - missing required parameter <svc_base> "}"

  print 'Deleting registry entry for ['$svc_type'-'$svc_base']...\c'

  if [ "$svc_type" = "ASIMVSIPOSListenerSvc" ]; then;
      partid=$(regfind -m '\\'"$mvs_host" -p "HKEY_LOCAL_MACHINE\SOFTWARE\\Accessible Software, Inc.\Access1\1.0\Components\\$svc_type\Instances\\$svc_base" -n "PartnerID" 2>nul: | awk '/PartnerID/ {print $3;}')
  fi

  # enhanced REG DELETE to work with 2000 Reskit. Removed older reg.exe usage.
  delstr="\\\\$mvs_host\HKEY_LOCAL_MACHINE\SOFTWARE\\Accessible Software, Inc.\Access1\1.0\Components\\$svc_type\Instances\\$svc_base"
  rc=$(REG DELETE "$delstr" /f 2>&1)

  if [ "$svc_type" = "ASIMVSIPOSListenerSvc" ]; then;
    delstr="\\\\$mvs_host\HKEY_LOCAL_MACHINE\SOFTWARE\\Accessible Software, Inc.\Access1\1.0\Components\\$svc_type\Settings\\ValidClients"
    if [ -n "$partid" ]; then
       rc=$(REG DELETE "$delstr" /v $partid /f 2>&1)
    fi
  fi  

  if $(print "$rc" | grep -q "completed successfully"); then
    print "OK"
  elif $(print "$rc" | grep -q "The system was unable to find the specified registry key"); then
    print "OK (already deleted)"
  else
    print "ERROR: $rc"
  fi
}

delete_file()
{
  filepath="${1?"ERROR: delete_file() - missing required parameter <filepath>}"
  
  echo 'Deleting file ['"${filepath}"']...\c'
  if [ -f "${filepath}" ]; then; 
    rc=$(rm -f "${filepath}" 2>&1);
    if $(print "$rc" | grep -q "Access is denied"); then;
      which DeleteFileDelayed.exe 2>&1 > nul:
      if [ $? -eq 0 ]; then;
        DeleteFileDelayed -dr "${filepath}"
        print 'OK (file is in use; delete will occur on next re-boot)'
      else
        print 'WARNING (file is in use and cannot be deleted)'
      fi
    elif [ -z "$rc" ]; then
      print "OK"
    else
      print "ERROR: $rc"
    fi
  else
    echo 'OK (already deleted)'
  fi
}

delete_target_queue_registry_entries()
{
  basename="${1?"ERROR: delete_target_queue_registry_entry() - missing required parameter <basename>}"

  echo 'Deleting target download queue registry entry for ['$basename']...\c'
  locdir=$(echo "$LOCALDIR" | awk '{print toupper($0);}' | sed 's/\\/\//1')

  # enhanced REG DELETE to work with 2000 Reskit. Removed older reg.exe usage.
  delstr="\\\\$mvs_host\HKEY_LOCAL_MACHINE\SOFTWARE\\Accessible Software, Inc.\Access1\1.0\Queues\Instances\\$locdir/DATA/QUEUES/$basename.que"
  rc=$(REG DELETE "$delstr" /f 2>&1)

  if $(print "$rc" | grep -q "completed successfully"); then
    print "OK"
  else $(print "$rc" | grep -q "The system was unable to find the specified registry key");
    print "OK (already deleted)"
  fi

  echo 'Deleting target upload queue registry entry for ['$basename']...\c'

  # enhanced REG DELETE to work with 2000 Reskit. Removed older reg.exe usage.
  delstr="\\\\$mvs_host\HKEY_LOCAL_MACHINE\SOFTWARE\\Accessible Software, Inc.\Access1\1.0\Queues\Instances\\$locdir/DATA/QUEUES/"$basename"_UPLOAD.QUE"
  rc=$(REG DELETE "$delstr" /f 2>&1)

  if $(print "$rc" | grep -q "completed successfully"); then
    print "OK"
  else $(print "$rc" | grep -q "The system was unable to find the specified registry key");
    print "OK (already deleted)"
  fi
}

delete_queue()
{
  filepath="${1?"ERROR: delete_queue() - missing required parameter <filepath>}"
  
  echo 'Deleting queue ['"${filepath}"']...\c'
  if [ -f "${filepath}" ]; then; 
    rc=$(rm -f "${filepath}" 2>&1);
    if $(print "$rc" | grep -q "Access is denied"); then;
      which DeleteFileDelayed.exe 2>&1 > nul:
      if [ $? -eq 0 ]; then;
        DeleteFileDelayed -d "${filepath}"
        print 'OK (queue file is in use; delete will occur on next re-boot)'
      else
        print 'WARNING (file is in use and cannot be deleted)'
      fi
    elif [ -z "$rc" ]; then
      print "OK"
    else
      print "ERROR: $rc"
    fi
  else
    echo 'OK (already deleted)'
  fi
}

remove_mvs_components()
{
  echo 'Removing MVS Components ['$BASENAME']: ['$OSPATH'('$OSID')]'

  echo 'Stop services for components'
  stop_service "ASIMVSEventHandlerSvc-$BASENAME"
  stop_service "ASIFileReceiverSvc-$BASENAME"
      
  # we will try and determine if this is a sna removal.
  REG QUERY "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components\ASIMVSListenerSvc\Instances\\$BASENAME" > "$0.reg" 2> "$0.reg"
  rcsna=$(cat "$0.reg" | awk ' BEGIN { result = 0 } END { print result } /ERROR:/ { result = 1 } /Error:/ { result = 1 } // { next } ')
  rm -f "$0.reg"  2> nul:

  if [ "$rcsna" -eq 0 ]; then
    stop_service "ASIMVSSenderSvc-$BASENAME"
  else
    stop_service "ASIMVSIPSenderSvc-$BASENAME"
    stop_service "ASIMVSIPOSListenerSvc"
  fi
  
  echo 'Deleting each of the affected NT services...'
  delete_service "ASIMVSEventHandlerSvc-$BASENAME"
  delete_service "ASIFileReceiverSvc-$BASENAME"
  delete_service "ASIMVSSenderSvc-$BASENAME"
  delete_service "ASIMVSIPSenderSvc-$BASENAME"

  echo 'Deleting the service bootstrap executable for each of the affected services...'
  delete_file "$INSTALLDIR/bin/services/ASIEH_$BASENAME.exe"
  delete_file "$INSTALLDIR/bin/services/ASIFR_$BASENAME.exe"
  delete_file "$INSTALLDIR/bin/services/ASISS_$BASENAME.exe"

  # Removing registry entries for affected services
  delete_registry_entry ASIMVSSenderSvc "${BASENAME}"
  delete_registry_entry ASIMVSEventHandlerSvc "${BASENAME}"
  delete_registry_entry ASIMVSListenerSvc "${BASENAME}"
  delete_registry_entry ASIFileReceiverSvc "${BASENAME}"
  delete_registry_entry ASIMVSIPSenderSvc "${BASENAME}"
  delete_registry_entry ASIMVSIPOSListenerSvc "${BASENAME}"

  print 'Deleting row from database table MVSSenderSvcMap for os_id='$OSID', svc_host=['$mvs_host'], svc_name=['$BASENAME']...\c'
  isql_cmd="SET NOCOUNT ON
            DELETE FROM MVSSenderSvcMap 
              WHERE os_id=$os_id OR (svc_host=N'$mvs_host' AND svc_name=N'$BASENAME')
            IF @@ROWCOUNT=0
              RAISERROR('ERROR: Row not found', 0, 1)
            ELSE
              RAISERROR('OK', 0, 1)"
  if [ $bTrusted -eq 1 ]; then
    rc=$($SQL_CMD_LINE -Q"$isql_cmd" 2>&1)
  else
    rc=$($SQL_CMD_LINE -Q"$isql_cmd" 2>&1)
  fi
  if $(print "$rc" | grep -q "OK"); then
    print "OK"
  elif $(print "$rc" | grep -q "ERROR: Row not found"); then
    print "OK (already deleted)"
  else
    print "ERROR: $rc"
  fi

  # Delete queue for this basename
  delete_queue "$INSTALLDIR/Data/Queues/"$BASENAME".que"
  delete_queue "$INSTALLDIR/Data/Queues/"$BASENAME"_Upload.que"

  # Delete the target queue entries from registry
  delete_target_queue_registry_entries "${BASENAME}"

  # start the ASIMVSIPOSListenerSvc if there is only at least 1 ip instance sub key
  REG QUERY "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components\ASIMVSIPOSListenerSvc\Instances" > "$0.reg" 2> "$0.reg"
  rc=$(cat "$0.reg" | awk ' BEGIN { result = 0 } END { print result } /ERROR:/ { result = 1 } /Error:/ { result = 1 } // { next } ')
  rm -f "$0.reg"  2> nul:
  if [ "$rc" -eq 0 ]; then ;
    ic=`regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components\ASIMVSIPOSListenerSvc\Instances" | wc -l`
    if [ "$ic" -gt 1 -a "$rcsna" -eq 1 ]; then
       restart_service "ASIMVSIPOSListenerSvc"
    fi
  fi

  print "Done."
}

populate_MVSSenderSvcMap_table()
{
  if [ $bInstall_SS -ne 1 ]; then;
    return
  fi

# Check if we are defing a SNA connection
  if [ ! -z "${os_id}" ]; then
    # populate the MVSSenderSvcMap table in the database 
    # to avoid problems with duplicate OS names
    echo 'Populating database table MVSSenderSvcMap for os_id='$os_id', svc_host=['$mvs_host'], svc_name=['$BASENAME']'
    isql_cmd="SET NOCOUNT ON DELETE FROM MVSSenderSvcMap WHERE os_id=$os_id OR (svc_host=N'$mvs_host' AND svc_name=N'$BASENAME')
                INSERT MVSSenderSvcMap(os_id, svc_host, svc_name)
                  SELECT $os_id, N'$mvs_host', N'$BASENAME'"
    if [ $bTrusted -eq 1 ]; then
      $SQL_CMD_LINE -Q"$isql_cmd"
    else
      $SQL_CMD_LINE -Q"$isql_cmd"
    fi
  fi

}

update_systemid()
{
  # update the _SystemId column for the basename
  echo 'Updating systemid=['$systemid'] for svc_name=['$BASENAME']' 
  
  osid=$(echo "$os_id" | sed 's/\''///g')
  isql_cmd="UPDATE OS_V SET _SystemId = N'$systemid' WHERE name = N'$BASENAME' AND id = $osid AND deleted = 0"
      
  if [ $bTrusted -eq 1 ]; then
    $SQL_CMD_LINE -Q"$isql_cmd"
  else
    $SQL_CMD_LINE -Q"$isql_cmd"
  fi
}

create_registry_settings()
{
  rm -f "$0.reg" 2> nul:

  if [ $bInstall_EH -eq 1 ]; then;
    echo 'Creating registry settings for the Event Handler...'
    print -r "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSEventHandlerSvc
            Instances
                $BASENAME
                    Hosts
                        $mvs_host
                            Settings
                                Log
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\$BASENAME.que
									CellSize = REG_DWORD $cellsize
									MaxEntries = REG_DWORD $maxentries
                            Status
                                DesiredState = STOPPED
                    Settings
                        CreateBusinessObjects = REG_DWORD 0x00000000
                        OSID = $OSID
                        DB
                            Application = $BASENAME
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|$BASENAME|%i|%F|%L|
                            Prefix = MVSE_${BASENAME}_
                        Model
                            Params
                                APPLICATION = ASIMVSEventHandlerSvc-$BASENAME
    " | sed 's/\\\\/\\/g' >> "$0.reg"
  fi

  if [ $bInstall_LS -eq 1 ]; then;
    if [ ! -z "${DOWNLOAD_LUNAME}"  ]; then
      echo 'Creating registry settings for the Listener...'
      print -r "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSListenerSvc
            Instances
                $BASENAME
                    PartnerLU = ${DOWNLOAD_LUNAME}
                    Hosts
                        $mvs_host
                            Settings
                                Log        
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\$BASENAME.que
									CellSize = REG_DWORD $cellsize
									MaxEntries = REG_DWORD $maxentries
                    Settings
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|$BASENAME|%i|%F|%L|
                            Prefix = MVSL_${BASENAME}_
      " | sed 's/\\\\/\\/g' >> "$0.reg"
    fi

    if [ ! -z "${UPLOAD_IPNAME}"  ]; then
      echo 'Creating registry settings for the IP Listener...'
      print -r "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSIPOSListenerSvc
            Instances
                $BASENAME
                    PartnerID = ${UPLOAD_IPNAME}
                    Hosts
                        $mvs_host
                            Settings
                                Log        
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\$BASENAME.que
									CellSize = REG_DWORD $cellsize
									MaxEntries = REG_DWORD $maxentries
                    Settings
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|$BASENAME|%i|%F|%L|
                            Prefix = MVSL_${BASENAME}_
      " | sed 's/\\\\/\\/g' >> "$0.reg"

      echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSIPOSListenerSvc
            Settings
			    ValidClients
				    ${UPLOAD_IPNAME} = ""
      " >> "$0.reg"

      echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSIPOSListenerSvc
            Settings
                Port = REG_DWORD ${DOWNLOAD_IPPORT}
      " >> "$0.reg"
    fi
  fi


  if [ $bInstall_SS -eq 1 ]; then;
    if [ ! -z "${UPLOAD_LUNAME}"  ]; then
      echo 'Creating registry settings for the Sender Service...'
      print -r "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSSenderSvc
            Instances
                ${BASENAME}
                    Hosts
                        $mvs_host
                            Settings
                                Log
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\"${BASENAME}"_Upload.que
									CellSize = REG_DWORD $cellsizeu
									MaxEntries = REG_DWORD $maxentriesu
                            Status
                                DesiredState = RUNNING
                    Settings
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|${BASENAME}|%i|%F|%L|
                            Prefix = MVSS_${BASENAME}_
                        Partner
                            Luname = ${UPLOAD_LUNAME}
                            Modename = ${UPLOAD_MODENAME}
      " | sed 's/\\\\/\\/g' >> "$0.reg"
    fi

    if [ ! -z "${UPLOAD_IPNAME}"  ]; then
      echo 'Creating registry settings for the IP Sender Service...'
      print -r "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSIPSenderSvc
            Instances
                ${BASENAME}
                    Hosts
                        $mvs_host
                            Settings
                                Log
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\"${BASENAME}"_Upload.que
									CellSize = REG_DWORD $cellsizeu
									MaxEntries = REG_DWORD $maxentriesu
                            Status
                                DesiredState = RUNNING
                    Settings
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|${BASENAME}|%i|%F|%L|
                            Prefix = MVSS_${BASENAME}_
                        Partner
                            PartnerID = ${UPLOAD_IPNAME}
                            Port = REG_DWORD ${UPLOAD_IPPORT}
      " | sed 's/\\\\/\\/g' >> "$0.reg"
    fi
  fi

  if [ $bInstall_FR -eq 1 ]; then;
    echo 'Creating registry settings for the File Receiver...'
    print -r "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIFileReceiverSvc
            Instances
                $BASENAME
                    Hosts
                        $mvs_host
                            Settings
                                InputFile = $LOCALDIR\Logs\\$BASENAME.dat
                                Log
                                Queue
                                    QueuePath = $LOCALDIR\Data\Queues\\$BASENAME.que
									CellSize = REG_DWORD $cellsize
									MaxEntries = REG_DWORD $maxentries
                    Settings
                        Log
                            LogHeading = %Y/%m/%d %H:%M:%S|%t|%O|%o|$BASENAME|%i|%F|%L|
                            Prefix = FR_${BASENAME}_
    " | sed 's/\\\\/\\/g' >> "$0.reg"
  fi

  # apply the generated file
  if [ -f "$0.reg" ]; then;
    regini -m \\\\$mvs_host "$0.reg" > nul:
    rm -f "$0.reg" 2> nul:
  fi
  

  # Find ReocnfigureHMS script location
  rescript=""
  if [ -e "$INSTALLDIR/mgmt/HMS/ShellScripts/ReconfigureHMS.ksh" ]; then
    rescript="$INSTALLDIR/mgmt/HMS/ShellScripts/ReconfigureHMS.ksh"
  fi  
  if [ -e "$INSTALLDIR/bin/ReconfigureHMS.ksh" ]; then
    rescript="$INSTALLDIR/bin/ReconfigureHMS.ksh"
  fi
    
  if [ ! -z "$rescript" ]; then
    # Add mvs_host to Health Monitor list to monitor
    if [ $bInstall_EH -eq 1 -o $bInstall_LS -eq 1 ]; then;
      sh "$rescript" -aMVSHost=$mvs_host -k$hms_host
    fi  
  
    if [ $bInstall_SS -eq 1 ]; then;
      sh "$rescript" -aSenderServiceHost=$mvs_host -k$hms_host
    fi  
  fi  

}


create_db_registry_settings()
{
  rm -f "$0.reg" 2> nul:

  # Event Handler
  if [ $bInstall_EH -eq 1 ]; then;
    echo 'Forcing Event Handler component-level database settings into the registry in response to [-f] parameter...'
    echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
        ASIMVSEventHandlerSvc
            Instances
                ${BASENAME}
                    Settings
                        DB
                            Server = ${db_server}
                            User = ${db_user}
                            Password = ${db_password}
    " >> "$0.reg"
  fi

  # Sender Service
  if [ $bInstall_SS -eq 1 ]; then;
    if [ ! -z "${UPLOAD_LUNAME}"  ]; then
      echo 'Forcing Sender Service component-level database settings into the registry in response to [-f] parameter...'
      echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
          ASIMVSSenderSvc
              Instances
                  ${BASENAME}
                      Settings
                          DB
                              Server = ${db_server}
                              User = ${db_user}
                              Password = ${db_password}
    " >> "$0.reg"
    fi
    if [ ! -z "${UPLOAD_IPNAME}"  ]; then
      echo 'Forcing Sender Service component-level database settings into the registry in response to [-f] parameter...'
      echo "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components
          ASIMVSIPSenderSvc
              Instances
                  ${BASENAME}
                      Settings
                          DB
                              Server = ${db_server}
                              User = ${db_user}
                              Password = ${db_password}
    " >> "$0.reg"
    fi
  fi

  # apply the generated file
  if [ -f "$0.reg" ]; then;
    regini -m \\\\$mvs_host "$0.reg" > nul:
    rm -f "$0.reg" 2> nul:
  fi
}

create_service()
{
  svc_name="${1?"ERROR: create_service() missing required param <svc_name>"}"
  display_name="${2?"ERROR: create_service() missing required param <display_name>"}"
  bin_path="${3?"ERROR: create_service() missing required param <bin_path>"}"
  start_type="${4?"ERROR: create_service() missing required param <start_type>"}"
  dependencies="${5?"ERROR: create_service() missing required param <dependencies>"}"

  if [ "$svc_name" = "ASIMVSIPOSListenerSvc" ]; then;
     rc=$(sc query $svc_name | awk ' BEGIN { result = 0 } END {print result} /SERVICE_NAME/ { result = 1 } // { next } ')
     if [ $rc -eq 0 ]; then;
	echo 'Creating service ['$svc_name']...'
	if [ -z "$LOCALDIR" ] ; then
	   LOCALDIR=$TMP_LOCALDIR
        fi
	"$LOCALDIR"'\\bin\\'"$svc_name" '/service'
     else
        echo "$svc_name" 'service already installed'
     fi
     return
  fi

  echo 'Creating service ['$svc_name']...'
  rc=$(sc \\\\$mvs_host create "$svc_name" binPath= \""$bin_path"\" depend= "$dependencies" start= $start_type DisplayName= "$display_name" type= own 2>&1)
  if $(print "$rc" | grep -iq ":"); then; #All Errors return the : character.

    if $(print "$rc" | grep -iq "1073:"); then; #1073= The specified service already exists.
      echo 'Updating ['$svc_name'] as already existed'
      rc=$(sc \\\\$mvs_host config "$svc_name" binPath= \""$bin_path"\" depend= "$dependencies" start= $start_type DisplayName= "$display_name" type= own 2>&1)

      if $(print "$rc" | grep -iq ":"); then; #All Errors return the : character.
        print "FAILURE: $rc"
      fi
    else
      print "FAILURE: $rc"
    fi
  fi
  
  # For trusted mode, update the EventHandler(s) and FileReceiver(s), ASIDBValidater, and ASIMVSUploadRuleSvc
  # with the Windows account as the LOGON id for remote access to the database
  if [ $bTrusted -eq 1 ]; then
    # Check for EventHandler
    if [ "$svc_name" = "ASIMVSEventHandlerSvc-$BASENAME"  -o "$svc_name" = "ASIFileReceiverSvc-$BASENAME" ]; then
      setserv -v -h $mvs_host -s "$svc_name" -u "$wUser_opt" -p "$wPassword_opt" 
      retCode=$?
      if [ "$retCode" -ne 0 ]; then
        exit 2
      fi
    # Update ASIDBValidater and ASIMVSUploadRuleSvc
     rc=$(sc qc ASIDBValidater | awk ' BEGIN { result = 0 } END {print result} /LocalSystem/ { result = 1 } // { next } ')
     if [ $rc -eq 1 ]; then;
       setserv -v -h $mvs_host -s "ASIDBValidater" -u "$wUser_opt" -p "$wPassword_opt" 
       retCode=$?
       if [ "$retCode" -ne 0 ]; then
         exit 2
       fi
       setserv -v -h $mvs_host -s "ASIMVSUploadRuleSvc" -u "$wUser_opt" -p "$wPassword_opt" 
       retCode=$?
       if [ "$retCode" -ne 0 ]; then
         exit 2
       fi
     fi
    fi   
  fi     
}
                                 
create_TPNAME_on_SNABASE_service()
{
  if [ $bInstall_LS -ne 1 ]; then;
    return
  fi
# if SNA conection information not defined do not continue
  if [ -z "${DOWNLOAD_LUNAME}"  ]; then
    return
  fi

  MVS_HOST_LOCALDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n -m '\\'$mvs_host "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\$//g')
  print 'Local Install directory of MVS host = ['"$MVS_HOST_LOCALDIR"']'

  # check for existance to SNABASE
  sclist $mvs_host | grep -iq SNABASE;
  if [ $? -eq 1 ]; then
    echo 'SNABASE service is not installed; not configuring TPSTART.'
    return
  fi

  rm -f "$0.reg" 2> nul:

  # SNABASE service registry key
  if [ $bInstall_LS -eq 1 ]; then;
    echo 'Adding TPSTART configuration settings to SNABASE service...'
    echo 'HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\SnaBase\Parameters\TPs\ACC1RCV
      Parameters
        SNAServiceType = REG_DWORD 0x00000006
        ConversationSecurity = no
        AlreadyVerified = no
        LocalLU = REG_SZ "'"${LOCAL_LUNAME}"'"
        Timeout = REG_DWORD 0xFFFFFFFF
        Parameters = REG_SZ "-nASIMVSListenerSvc -c"
        PathName = REG_SZ "'"${MVS_HOST_LOCALDIR}"\\\\'bin\ASIServiceApp.exe"
    ' >> "$0.reg"
  fi

  # apply the generated file
  if [ -f "$0.reg" ]; then;
    regini -m \\\\$mvs_host "$0.reg" > nul:
    rm -f "$0.reg" 2> nul:
  fi
}

stop_service() {
   svc_name="${1?"ERROR: stop_service() missing required parameter <svc_name>"}"
   
   # 0-NOT STOPPED  1-STOPPED
   rc=$(sc query $svc_name | awk ' BEGIN { result = 0 } END {print result} /STOPPED/ { result = 1 } /1060:/ { result = 1 } // { next } ')
   if [ $rc -eq 1 ] ; then
      print "$svc_name is stopped"
   else
      rc=$(sc stop "$svc_name" | awk ' BEGIN { result = 0 } END {print result} /STOPPED/ { result = 1 } /1060:/ { result = 1 } // { next } ')
      while [ $rc -eq 0 ]; do
	    sleep 3
		rc=$(sc query "$svc_name" | awk ' BEGIN { result = 0 } END {print result} /STOPPED/ { result = 1 } /1060:/ { result = 1 } // { next } ')
	  done
	  print "$svc_name is stopped"
   fi
}

restart_service() {
   svc_name="${1?"ERROR: restart_service() missing required parameter <svc_name>"}"
   
   # 0-NOT STOPPED  1-STOPPED
   rc=$(sc query $svc_name | awk ' BEGIN { result = 0 } END {print result} /STOPPED/ { result = 1 } /1060:/ { result = 2 } // { next } ')
   if [ $rc -eq 1 ] ; then
      rc=$(sc start $svc_name)
	  print "$svc_name is started"
   elif [ $rc = 2 ] ; then
      return
   else
      rc=$(sc stop "$svc_name" | awk ' BEGIN { result = 0 } END {print result} /STOPPED/ { result = 1 } /1060:/ { result = 2 } // { next } ')
      while [ $rc -eq 0 ]; do
	    sleep 3
		rc=$(sc query "$svc_name" | awk ' BEGIN { result = 0 } END {print result} /STOPPED/ { result = 1 } /1060:/ { result = 2 } // { next } ')
	  done
	  
	  rc=$(sc start $svc_name)
	  print "$svc_name is restarted" 
   fi
}

main()
{
  process_component_listing

  check_for_required_parameters

  validate_os_path

  # remove components if specified
  if [ $bRemoveComponents -eq 1 ]; then;
    remove_mvs_components "$BASENAME" "$OSPATH"
    exit
  fi

  populate_MVSSenderSvcMap_table

  if [ ! -z "${systemid}" ]; then
    update_systemid
  fi

  create_TPNAME_on_SNABASE_service

  # Install the registry settings
  create_registry_settings
  if [ $bForceDatabaseSettings -eq 1 ]; then;
    create_db_registry_settings
  fi

  # Copy binaries
  if [ $bInstall_EH -eq 1 ]; then;
    echo 'Copying the Event Handler service bootstrap executable to the proper location/name...'
    cp -v "$INSTALLDIR/bin/ASIServiceApp.exe" "$INSTALLDIR/bin/services/ASIEH_$BASENAME.exe"
  fi
  if [ $bInstall_SS -eq 1 ]; then;
    echo 'Copying the Sender Service service bootstrap executable to the proper location/name...'
    cp -v "$INSTALLDIR/bin/ASIServiceApp.exe" "$INSTALLDIR/bin/services/ASISS_$BASENAME.exe"
  fi
  if [ $bInstall_FR -eq 1 ]; then;
    echo 'Copying the File Receiver service bootstrap executable to the proper location/name...'
    cp -v "$INSTALLDIR/bin/ASIServiceApp.exe" "$INSTALLDIR/bin/services/ASIFR_$BASENAME.exe"
  fi

  # Install services
  if [ $bInstall_EH -eq 1 ]; then;
    if $(filever "$(which reg.exe)" 2>&1 | grep -q "version" >nul:); then
       create_service "ASIMVSEventHandlerSvc-$BASENAME" "Tivoli BSM MVSEventHandlerSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASIEH_$BASENAME.exe" "auto" "ASIDBValidater/RpcSs/RpcLocator"
    else
       create_service "ASIMVSEventHandlerSvc-$BASENAME" "Tivoli BSM MVSEventHandlerSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASIEH_$BASENAME.exe" "auto" "ASIDBValidater RPCSS RPCLOCATOR"
    fi
  fi
  if [ $bInstall_SS -eq 1 ]; then;
    if [ ! -z "${UPLOAD_LUNAME}"  ]; then
      if $(filever "$(which reg.exe)" 2>&1 | grep -q "version" >nul:); then
         create_service "ASIMVSSenderSvc-$BASENAME" "Tivoli BSM MVSSenderSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASISS_$BASENAME.exe" "auto" "SnaBase/RpcSs/RpcLocator"
      else
         create_service "ASIMVSSenderSvc-$BASENAME" "Tivoli BSM MVSSenderSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASISS_$BASENAME.exe" "auto" "SNABASE RPCSS RPCLOCATOR"
      fi
    fi
    if [ ! -z "${UPLOAD_IPNAME}"  ]; then
      create_service "ASIMVSIPSenderSvc-$BASENAME" "Tivoli BSM MVSIPSenderSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASISS_$BASENAME.exe" "auto" ""
    fi
  fi
  if [ $bInstall_FR -eq 1 ]; then;
    if $(filever "$(which reg.exe)" 2>&1 | grep -q "version" >nul:); then
       create_service "ASIFileReceiverSvc-$BASENAME" "Tivoli BSM FileReceiverSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASIFR_$BASENAME.exe" "demand" "RpcSs/RpcLocator"
    else
       create_service "ASIFileReceiverSvc-$BASENAME" "Tivoli BSM FileReceiverSvc-${BASENAME}" "${LOCALDIR}\bin\Services\ASIFR_$BASENAME.exe" "demand" "RPCSS RPCLOCATOR"
    fi
  fi
  if [ -n "${DOWNLOAD_IPPORT}" -a -n "${UPLOAD_IPNAME}" -a -n "${UPLOAD_IPPORT}" ]; then;
	create_service "ASIMVSIPOSListenerSvc" "" "" "" ""
  fi

  # start the ASIMVSIPOSListenerSvc
  if [ -z "${DOWNLOAD_LUNAME}" ] ; then;
     restart_service "ASIMVSIPOSListenerSvc"
  fi
  
  # restart service(s) if -A specified
  if [ $bRestart = 1 ] ; then ;
     restart_service "ASIMVSEventHandlerSvc-$BASENAME"
     if [ -z "${DOWNLOAD_LUNAME}" ] ; then
        restart_service "ASIMVSIPSenderSvc-$BASENAME"
     else
        restart_service "ASIMVSSenderSvc-$BASENAME"
     fi
  fi

  echo 'Done.'
}


main



















