-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_deleteoldopcplans.sql, tbsm_base, tbsm_base31e
-- SID: 1.1
-- Date Modified: 07/05/02 17:52:19
--
-- Module Description:
--   Defines the asisp_deleteoldopcplans stored procedure
--

USE OPCLoad
GO

---------------------------------------------------------------------------------------------------
-- asisp_deleteoldopcplans
PRINT 'Creating procedure asisp_deleteoldopcplans'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_deleteoldopcplans' AND type = 'P')
  DROP PROCEDURE asisp_deleteoldopcplans
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_deleteoldopcplans.sql, tbsm_base, tbsm_base31e
-- SID: 1.1
-- Date Modified: 07/05/02 17:52:19
-- Cleans up OPCLoad database tables based on dates of daily plans
CREATE PROCEDURE asisp_deleteoldopcplans
  @daystokeep INT = 60,
  @limit INT = 500000,
  @verbose INT = 0
AS

  SET NOCOUNT ON

  DECLARE @tcount      INT
  DECLARE @oldest      DATETIME
  DECLARE @err         INT
  DECLARE @now         NVARCHAR(30)
  DECLARE @rc          INT
  DECLARE @totalrc     INT
  DECLARE @alltablesrc INT

  -- Set the oldest plan to keep
  SET @oldest = DATEADD(day, -@daystokeep, CURRENT_TIMESTAMP)

  IF @verbose >= 1
  BEGIN
     SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
     RAISERROR('%s: asisp_deleteoldopcplans: Starting', 0, 1, @now) WITH NOWAIT
  END

  -- Temp table to hold the plan ids for the plans to be deleted
  CREATE TABLE #planidstodelete 
  (
    plan_id INT PRIMARY KEY CLUSTERED
  )

  INSERT INTO #planidstodelete
    SELECT id FROM opc_daily_plan WHERE end_date < @oldest
  SET @rc = @@ROWCOUNT

  IF @verbose >= 1
  BEGIN
    SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldopcplans: inserted %d rows into #planidstodelete', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Because of various foreign key constraints in the OPCLoad database, the delete from the
  -- tables have to be done in a certain order
  -- opc_daily_plan is the master table, it is referenced by opc_application, which is referenced
  -- by opc_operation, which is referenced by opc_predecessor

  SET ROWCOUNT @limit

  -- Cleanup opc_predecessor first
  SET @rc = @limit
  SET @totalrc = 0

  WHILE @rc >= @limit
  BEGIN
    DELETE FROM pred 
    FROM opc_predecessor AS pred
      JOIN #planidstodelete AS plan_delete ON plan_delete.plan_id = pred.plan_id
    SET @rc = @@ROWCOUNT
    SET @totalrc = @totalrc + @rc
    IF @verbose >= 2
    BEGIN
      SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldopcplans: deleted %d rows from opc_predecessor', 0, 1, @now, @rc) WITH NOWAIT
    END
    WAITFOR DELAY '00:00:02'
  END

  IF @verbose >= 1
  BEGIN
    SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldopcplans: deleted total of %d rows from opc_predecessor', 0, 1, @now, @totalrc) WITH NOWAIT
  END

  -- Cleanup opc_operation next
  SET @rc = @limit
  SET @totalrc = 0

  WHILE @rc >= @limit
  BEGIN
    DELETE FROM oper 
    FROM opc_operation AS oper
      JOIN #planidstodelete AS plan_delete ON plan_delete.plan_id = oper.plan_id
    SET @rc = @@ROWCOUNT
    SET @totalrc = @totalrc + @rc
    IF @verbose >= 2
    BEGIN
      SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldopcplans: deleted %d rows from opc_operation', 0, 1, @now, @rc) WITH NOWAIT
    END
    WAITFOR DELAY '00:00:02'
  END

  IF @verbose >= 1
  BEGIN
    SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldopcplans: deleted total of %d rows from opc_operation', 0, 1, @now, @totalrc) WITH NOWAIT
  END

  -- Cleanup opc_application next
  SET @rc = @limit
  SET @totalrc = 0

  WHILE @rc >= @limit
  BEGIN
    DELETE FROM app 
    FROM opc_application AS app
      JOIN #planidstodelete AS plan_delete ON plan_delete.plan_id = app.plan_id
    SET @rc = @@ROWCOUNT
    SET @totalrc = @totalrc + @rc
    IF @verbose >= 2
    BEGIN
      SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldopcplans: deleted %d rows from opc_application', 0, 1, @now, @rc) WITH NOWAIT
    END
    WAITFOR DELAY '00:00:02'
  END

  IF @verbose >= 1
  BEGIN
    SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldopcplans: deleted total of %d rows from opc_application', 0, 1, @now, @totalrc) WITH NOWAIT
  END

  -- Cleanup opc_app_path_aggr
  SET @rc = @limit
  SET @totalrc = 0

  WHILE @rc >= @limit
  BEGIN
    DELETE FROM aggr 
    FROM opc_app_path_aggr AS aggr
      JOIN #planidstodelete AS plan_delete ON plan_delete.plan_id = aggr.plan_id
    SET @rc = @@ROWCOUNT
    SET @totalrc = @totalrc + @rc
    IF @verbose >= 2
    BEGIN
      SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldopcplans: deleted %d rows from opc_app_path_aggr', 0, 1, @now, @rc) WITH NOWAIT
    END
    WAITFOR DELAY '00:00:02'
  END

  IF @verbose >= 1
  BEGIN
    SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldopcplans: deleted total of %d rows from opc_app_path_aggr', 0, 1, @now, @totalrc) WITH NOWAIT
  END

  -- Cleanup master plan table, opc_daily_plan
  SET @rc = @limit
  SET @totalrc = 0

  WHILE @rc >= @limit
  BEGIN
    DELETE FROM oplan 
    FROM opc_daily_plan AS oplan
      JOIN #planidstodelete AS plan_delete ON plan_delete.plan_id = oplan.id
    SET @rc = @@ROWCOUNT
    SET @totalrc = @totalrc + @rc
    IF @verbose >= 2
    BEGIN
      SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldopcplans: deleted %d rows from opc_daily_plan', 0, 1, @now, @rc) WITH NOWAIT
    END
    WAITFOR DELAY '00:00:02'
  END

  IF @verbose >= 1
  BEGIN
    SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldopcplans: deleted total of %d rows from opc_daily_plan', 0, 1, @now, @totalrc) WITH NOWAIT
  END

  SET ROWCOUNT 0
  
  SET NOCOUNT OFF

GO
