-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/hist_lob_objtree.sql, tbsm_base, tbsm_base31e
-- SID: 1.5.4.6
-- Date Modified: 07/03/27 02:44:01
--

/* PROCEDURE FOR LOB Object Tree Report */

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'hist_lob_objtree' AND type = 'P')
  DROP PROCEDURE hist_lob_objtree
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/hist_lob_objtree.sql, tbsm_base, tbsm_base31e
-- SID: 1.5.4.6
-- Date Modified: 07/03/27 02:44:01
--
CREATE PROCEDURE hist_lob_objtree
  @id ObjID
AS

  SET NOCOUNT ON

  CREATE TABLE #lobtree
  (
    generation INT,
    parent_ident INT,
    ident INT identity,
    name ObjName,
    lobid ObjID,
    cid ClassID,
    id ObjID NULL,
    nodeid NVARCHAR(255),
    shortnodeid NVARCHAR(255),
    deleted TINYINT,
    resourcepath NVARCHAR(1375) NULL
  )

  DECLARE @gen INT, @lobdesc NVARCHAR(255), @rc INT
  DECLARE @lvl SMALLINT, @bDone TINYINT, @pad SMALLINT, @holdpid ObjID 
  DECLARE @dot NCHAR(1), @parnodeid NVARCHAR(255), @shortparnodeid NVARCHAR(255)

-- Build #lobtree using containment table.  Initially, all rows
-- are loaded with a cid of LOB.
-- Any LOBs with LOBL links to phisical objects are updates below.

  SELECT @gen=0
  INSERT INTO #lobtree (generation, parent_ident, name, lobid, cid, id, nodeid, shortnodeid, deleted)
    SELECT @gen, 0, '', @id, 'LOB', NULL, '', '', 0 FROM LineOfBusiness_V WHERE id=@id
  SET @rc = @@ROWCOUNT
  WHILE @rc > 0
  BEGIN
    SELECT @gen=@gen + 1
    INSERT INTO #lobtree (generation, parent_ident, name, lobid, cid, id, nodeid, shortnodeid, deleted)
      SELECT @gen, t.ident, '', c.dst_id, 'LOB', NULL, '', '', 0
        FROM containment c, #lobtree t
        WHERE t.generation=@gen - 1 AND c.src_cid='LOB' AND c.src_id=t.lobid
        AND c.dst_cid='LOB' AND c.deleted=0
      UNION
      SELECT @gen, t.ident, '', c.dst_id, 'LOB', NULL, '', '', 0
        FROM containment c, #lobtree t, link ln
        WHERE t.generation=@gen - 1 AND c.src_cid='LOB' AND c.src_id=ln.dst_id
        AND c.dst_cid='LOB' AND c.deleted=0
        AND ln.link_type='LOBL' AND ln.src_cid='LOB' AND ln.dst_cid='LOB' AND ln.src_id=t.lobid
    SET @rc = @@ROWCOUNT
  END

-- For any LOBs with LOBL links to physical objects,
-- overwrite the cid, id and deleted flag

  UPDATE #lobtree SET cid=l.dst_cid, id=l.dst_id, deleted=l.deleted FROM #lobtree t, link l
    WHERE l.link_type='LOBL' AND l.src_cid='LOB' AND t.lobid=l.src_id

-- Use report_addclassdata to get name of all physical objects

  EXEC report_addclassdata '#lobtree', 'name', 'name'

-- The following sets the name column with the name of the LOB and the name of the
-- physical object in parens if it differs from the lob name.

  UPDATE #lobtree SET
    name=CASE t.name WHEN '' THEN l.name WHEN l.name THEN l.name ELSE LEFT( l.name + ' (' + t.name + ')', 54 ) END
--    name=case t.name when '' then l.name when l.name then l.name else l.name + ' (' + t.name + ')' end
  FROM #lobtree t, LineOfBusiness_V l WHERE t.lobid=l.id

--Added for OA19911 to get the path to sort the report on

CREATE INDEX Ilobtree on #lobtree(lobid)

UPDATE t SET t.resourcepath =o.resourcepath from #lobtree t join ObjPathCache o on t.lobid=o.id and o.cid='LOB'



-- The following builds the nodeid varchar column in the form 00001.00001....etc
-- This represents the indented structure of the tree sorted in the same order
-- as is used in the Access1 product.  The #lvlcounter table, by use of an
-- identity column, is used to create the numbered fields of this nodeid.

-- This next section is commented out as it no longer works in 3.1 - it always ends up 
-- just putting NULL in the fields

/**  SELECT @lvl=0, @bDone=0, @pad=5

  CREATE TABLE #lvlcounter (lid INT, ident INT identity)
  SELECT @dot=''

  WHILE @bDone=0
  BEGIN
    DECLARE cc INSENSITIVE CURSOR FOR
    SELECT DISTINCT parent_ident FROM #lobtree WHERE generation=@lvl
    OPEN cc
    FETCH NEXT FROM cc INTO @holdpid
    IF @@FETCH_STATUS=-1
      SELECT @bDone=1
    WHILE @@FETCH_STATUS<>-1 AND @bDone=0
    BEGIN
      TRUNCATE TABLE #lvlcounter
      SELECT @parnodeid=LTRIM(nodeid), @shortparnodeid=LTRIM(shortnodeid) FROM #lobtree WHERE ident=@holdpid
      INSERT INTO #lvlcounter (lid) SELECT d.ident FROM #lobtree d WHERE d.parent_ident=@holdpid ORDER BY d.name
      UPDATE #lobtree SET
        nodeid=LTRIM(@parnodeid + @dot + right(replicate("0",@pad) + LTRIM(str(c.ident)),@pad)),
        shortnodeid=LTRIM(@shortparnodeid + @dot + LTRIM(str(c.ident)))
      FROM #lvlcounter c, #lobtree d
      WHERE c.lid=d.ident
      FETCH NEXT FROM cc INTO @holdpid
    END
    DEALLOCATE cc
    SELECT @lvl=@lvl + 1
    SELECT @dot='.'
  END   **/

--changed for OA19911
  SELECT l.generation, l.shortnodeid, l.cid, o.label, l.id, l.name FROM #lobtree l, obj_class o WHERE o.cid=l.cid ORDER BY UPPER(l.resourcepath)
GO
