-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_oledb_bsv.sql, tbsm_base, tbsm_base31e
-- SID: 1.4
-- Date Modified: 07/03/23 08:07:40
--
-- Module Description
--   Defines the tep_oledb_bsv stored procedure

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tep_oledb_bsv' AND type = 'P')
  DROP PROCEDURE tep_oledb_bsv
GO

PRINT 'Creating procedure tep_oledb_bsv'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_oledb_bsv.sql, tbsm_base, tbsm_base31e
-- SID: 1.4
-- Date Modified: 07/03/23 08:07:40
-- Query TEPS database to gather TEP Business View groupings.
-- This proc populates tep_bsv and tep_disc_bsv locates/creates the related LOB objects
-- This proc is invoked by tep_disc
CREATE PROCEDURE tep_oledb_bsv
  @verbose   TINYINT = 0
AS

  SET ANSI_DEFAULTS OFF
  SET CONCAT_NULL_YIELDS_NULL OFF
  SET NOCOUNT ON

  DECLARE @now NVARCHAR(40)
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tep_oledb_bsv: started', 0, 1, @now) WITH NOWAIT
  END

  DECLARE @start DATETIME, @elapsed_msecs INT, @log_text NVARCHAR(200), @log_detail NVARCHAR(3500)

  CREATE TABLE #teps_tables
  (
     table_name   ObjName,
     col_names    NVARCHAR(200),
     where_clause NVARCHAR(300),
     orderno      INT IDENTITY(1,1)
  )
    DECLARE @table_name ObjName, @col_names NVARCHAR(200), @where_clause NVARCHAR(300)


  CREATE TABLE #tep_bsv
  (
    parent_rowno      INT,
    rowno             INT IDENTITY(1,1) PRIMARY KEY,
    lob_id            ObjID NULL,
    phy_cid           ClassID NULL,
    phy_id            ObjID NULL,
    name              NVARCHAR(64),
    tep_name          NVARCHAR(64),
    teps_objname      NVARCHAR(96),
    teps_busview      NVARCHAR(48),
    teps_server       NVARCHAR(64) ,
    namepath	      NVARCHAR(2000),
    lev               INT,
    topo_id           INT,
  )  

/*
KFWTOPO
ID          OBJECTID                                         DISPLAYNAME                                      TEXT                                                             ROOTMOBJ                                                                                         FLAGS       
----------- ------------------------------------------------ ------------------------------------------------ ---------------------------------------------------------------- ------------------------------------------------------------------------------------------------ ----------- 
0           MOPHYSICAL                                       Physical                                         Physical Node View                                               PHYSICAL_ENTERPRISE                                                                              2
1           MOLOGICAL                                        Business                                         CMW Business View                                                Enterprise                                                                                       1
7000000     KCF-CONFIGURATION                                Configuration                                    MQ Series Configuration                                          KCF-01-ROOT                                                                                      1
7000001     TOP-105.02.24-12.38.01-00001                     IMSV8_Plex                                       all IMN* components                                              OBJ-105.02.24-12.38.01-00001                                                                     1

Meaning of FLAGS columnn in KFWTOPO

Bit 0 (Editable): The topology can be seen in the TEP client Navigator Editor
Bit 1 (Protected): The topology can not be modified in Navigator Editor
Bit 2 (Prototype): The topology is a prototype, used for instantiation of special sub-topologies dynamically.  Not currently implemented fully, and not in GA product.
Bit 3 (Product Provided): The topology is special to an application, and should not be handled as a normal "STATEFUL" navigator (shows red/yellow etc. events).

Protected, product provided views should not be discovered for TBSM. 
When Bit 1 (Protested) and bit 3 (Product Provided) as set, FLAGS has a value of 10.
*/
  CREATE TABLE #KFWTOPO
  (
	ID		INT NULL,
	OBJECTID	NVARCHAR(48) NULL,
	DISPLAYNAME	NVARCHAR(48) NULL,
	TEXT		NVARCHAR(64) NULL,
	ROOTMOBJ	NVARCHAR(96) NULL,
	FLAGS       	INT NULL
  )
  INSERT INTO #teps_tables ( table_name, col_names, where_clause )
  SELECT 'KFWTOPO', 'ID, OBJECTID, DISPLAYNAME, TEXT, ROOTMOBJ, FLAGS',' WHERE FLAGS < 10 '

/*
KFWEDGE
NAME                                                                                             CHILD                                                                                            TOPOLOGY    
------------------------------------------------------------------------------------------------ ------------------------------------------------------------------------------------------------ ----------- 
Enterprise                                                                                       CICSPLEX                                                                                         1
OBJ-105.02.24-12.38.01-00001                                                                     p@DFSIMN0:TIC1.SYSPLEXC:SQGROUP                                                                  7000001
*/
  CREATE TABLE #KFWEDGE
  (
	NAME		NVARCHAR(96) NULL,
	CHILD		NVARCHAR(96) NULL,
	TOPOLOGY 	INT NULL
  )
  INSERT INTO #teps_tables SELECT 'KFWEDGE', 'NAME, CHILD, TOPOLOGY',''
  CREATE NONCLUSTERED INDEX #i_#KFWEDGE_TOPOLOGY_NAME ON #KFWEDGE (TOPOLOGY, NAME)

/*
KFWMOBJ
NAME                                                                                             TEMPLATE                                                         NODE                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     TEXT                                                             LSTDATE          LSTUSRPRF                        ATTRIBS     MEMORYOBJ   
------------------------------------------------------------------------------------------------ ---------------------------------------------------------------- ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- ---------------------------------------------------------------- ---------------- -------------------------------- ----------- ----------- 
CICSPLEX                                                                                         CICSPLEX                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  CICSPLEX Default Object                                          1010801120000000 XE_V100                          2048        0
DB2710:DB2plex:DSGROUP@zDB2_002_Threads                                                          zDB2_002_Threads                                                 DB2710:DB2plex:DSGROUP                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    1050324151441000 SYSADMIN                         0           1
Enterprise                                                                                       Aggregation                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
*/
  CREATE TABLE #KFWMOBJPROP
  (
	ID	NVARCHAR(96) NULL,
	PNAME   NVARCHAR(255) NULL,
	PTYPE	INT NULL,
	PVALUE	NVARCHAR(100) NULL
  )
  INSERT INTO #teps_tables SELECT 'KFWMOBJPROP','ID, PNAME, PTYPE, PVALUE',' WHERE PNAME IN ("-1021","-1022","NODE")'
  CREATE NONCLUSTERED INDEX #i_#KFWMOBJPROP_id  ON #KFWMOBJPROP (ID)

  DECLARE @sql NVARCHAR(4000), @rc INT, @err INT, @lev INT

  DECLARE @teps_server ObjName, @teps_from_clause NVARCHAR(100), @orderno INT, @anchor_business NVARCHAR(64), @anchor_physical NVARCHAR(64)
  DECLARE cur_cnps INSENSITIVE CURSOR FOR
    SELECT DISTINCT teps_server, teps_from_clause, anchor_business, anchor_physical
      FROM tep_config 
      WHERE enabled = 1 AND ISNULL(anchor_business,'') <> ''
      ORDER BY teps_server
  OPEN cur_cnps
  FETCH NEXT FROM cur_cnps INTO @teps_server, @teps_from_clause, @anchor_business, @anchor_physical
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @start = CURRENT_TIMESTAMP 
    SET @log_detail = ''
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR("%s: tep_oledb_bsv: Requesting information from %s ....", 0, 1, @now, @teps_server) WITH NOWAIT
    END

    -- replicate cnps tables to local TBSM temp tables
    DECLARE cur_teps_tables INSENSITIVE CURSOR FOR
      SELECT DISTINCT table_name, col_names, where_clause, orderno
        FROM #teps_tables 
        ORDER BY orderno
    OPEN cur_teps_tables
    FETCH NEXT FROM cur_teps_tables INTO @table_name, @col_names, @where_clause, @orderno
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      SET @sql = "
        TRUNCATE TABLE #"+@table_name+"
        SET ANSI_NULLS ON
        SET ANSI_WARNINGS ON
        EXEC('INSERT INTO #"+@table_name+"
                SELECT "+@col_names+" FROM "+@teps_from_clause+@table_name+@where_clause+"')
      "
      EXEC(@sql)
      SELECT @rc = @@ROWCOUNT, @err = @@ERROR
      SET @log_detail = @log_detail+@table_name+' '+CAST(@rc AS NVARCHAR)+', '
      IF @err <> 0 
      BEGIN
        SET @log_text = 'ERROR '+CAST(@err AS NVARCHAR)+' - '+@teps_server+' '+@table_name
        SET @log_detail = @sql
        EXEC tep_log @proc_name='tep_oledb_bsv', @log_text=@log_text, 
                                  @elapsed_msecs=@elapsed_msecs, @row_count=@rc, @log_detail=@log_detail
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tep_oledb_bsv: Error %d for %s',3,1,@now, @err,@sql) WITH NOWAIT
        DEALLOCATE cur_teps_tables
        DEALLOCATE cur_cnps
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tep_oledb_bsv:   %-8d row(s) inserted into #%s for %s', 0, 1, @now, @rc, @table_name, @teps_server) WITH NOWAIT
        IF @verbose >= 2
        BEGIN
          SET @sql = "
--            SET ROWCOUNT 100
            SELECT * FROM #"+@table_name+"
            SET ROWCOUNT 0
            "
          EXEC(@sql)     
        END
      END
      FETCH NEXT FROM cur_teps_tables INTO @table_name, @col_names, @where_clause, @orderno
    END  -- loop back for next cnps table
    DEALLOCATE cur_teps_tables


--------------------------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------------------------
    SET @lev = 0
    INSERT INTO #tep_bsv (parent_rowno, name, tep_name, teps_objname, teps_busview, teps_server, namepath, lev, topo_id)
      SELECT 0, @anchor_business, '', '', '', @teps_server, @anchor_business, @lev, 0

    SET @lev = @lev + 1
    INSERT INTO #tep_bsv (parent_rowno, name, tep_name, teps_objname, teps_busview, teps_server, namepath, lev, topo_id)
      SELECT DISTINCT p.rowno, 
                      CASE WHEN topo.OBJECTID = 'MOLOGICAL' AND topo.DISPLAYNAME = 'Kfw:KFW10002' THEN 'Logical'  -- reflect TEP hardcoded name
                           ELSE topo.DISPLAYNAME END,  -- name
                      '', topo.ROOTMOBJ, topo.OBJECTID, @teps_server, 
                      p.name+'/'+CASE WHEN topo.OBJECTID = 'MOLOGICAL' AND topo.DISPLAYNAME = 'Kfw:KFW10002' THEN 'Logical'  -- reflect TEP hardcoded name
                                      ELSE topo.DISPLAYNAME END,  -- namepath
                      @lev, topo.ID
        FROM #KFWTOPO AS topo,
             #tep_bsv AS p
        WHERE topo.OBJECTID NOT IN ('MOPHYSICAL','KCF-CONFIGURATION')
          AND p.lev = 0 AND p.teps_server = @teps_server AND p.name = @anchor_business
        ORDER BY topo.ROOTMOBJ
    SELECT @rc = @@ROWCOUNT, @err = @@ERROR
    IF @err <> 0 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tep_oledb_bsv: Error %d for insert into #tep_bsv at lev %d',3,1,@now, @err, @lev) WITH NOWAIT
      DEALLOCATE cur_cnps
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tep_oledb_bsv:   %-8d row(s) inserted into #tep_bsv at lev %d', 0, 1, @now, @rc, @lev) WITH NOWAIT
      IF @verbose >= 2
      BEGIN
        SET ROWCOUNT 10
        SELECT * FROM #tep_bsv WHERE teps_server = @teps_server AND lev = @lev
        SET ROWCOUNT 0
      END
    END

    -- Loop through Business View hierarchy
    WHILE @rc > 0
    BEGIN
      SET @lev = @lev + 1
      INSERT INTO #tep_bsv (parent_rowno, name, tep_name, teps_objname, teps_busview, teps_server, namepath, lev, topo_id)
        SELECT DISTINCT p.rowno,                                                            -- parent_rowno
                        CASE WHEN ISNULL(prop_1022.PVALUE,'') <> '' THEN prop_1022.PVALUE
                             ELSE ISNULL(prop_node.PVALUE,'') END,                          -- name
                        CASE WHEN ISNULL(prop_node.PVALUE,'') <> '' THEN prop_node.PVALUE
                             WHEN ISNULL(p.tep_name,'') <> ''
                              AND edge.CHILD LIKE p.tep_name+'@%'   THEN p.tep_name
                             ELSE ISNULL(tep.tep_name,'') END,                              -- tep_name
                        edge.CHILD,                                                         -- teps_objname
                        p.teps_busview,                 
                        @teps_server,
                        p.namepath,
                        @lev,
                        p.topo_id
          FROM #tep_bsv                AS p
          JOIN #KFWEDGE                AS edge      ON edge.TOPOLOGY = p.topo_id AND edge.NAME = p.teps_objname
          LEFT OUTER JOIN #KFWMOBJPROP AS prop_node ON prop_node.ID = edge.CHILD AND prop_node.PNAME = 'NODE'   -- TEMS name 
          LEFT OUTER JOIN #KFWMOBJPROP AS prop_1022 ON prop_1022.ID = edge.CHILD AND prop_1022.PNAME = '-1022'  -- name text
          LEFT OUTER JOIN tep_objects  AS tep       ON edge.CHILD LIKE tep.tep_name+'@%' AND tep.anchor_physical = @anchor_physical
          WHERE p.teps_server = @teps_server AND p.lev = @lev - 1
            AND ISNULL(prop_1022.PVALUE,'')+ISNULL(prop_node.PVALUE,'') <> ''  -- object has "name text" Or a "teps name"
          ORDER BY edge.CHILD
      SELECT @rc = @@ROWCOUNT, @err = @@ERROR
      IF @err <> 0 
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tep_oledb_bsv: Error %d for insert into #tep_bsv',3,1,@now, @err) WITH NOWAIT
        DEALLOCATE cur_cnps
        RETURN -30000
      END

      UPDATE #tep_bsv SET namepath = namepath + '/' + name
        WHERE teps_server = @teps_server AND lev = @lev
      SELECT @rc = @@ROWCOUNT, @err = @@ERROR
      IF @err <> 0 
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tep_oledb_bsv: Error %d for update of #tep_bsv',3,1,@now, @err) WITH NOWAIT
        DEALLOCATE cur_cnps
        RETURN -30000
      END

      -- remove objects that are the same as its parent (i.e. redundant), unless it has children itself too.
      DELETE c
        FROM #tep_bsv AS p
        JOIN #tep_bsv AS c ON c.parent_rowno = p.rowno AND c.tep_name = p.tep_name
        LEFT OUTER JOIN #KFWEDGE AS c_edge ON c_edge.TOPOLOGY = c.topo_id AND c_edge.NAME = c.teps_objname
        WHERE p.teps_server = @teps_server AND p.lev = @lev - 1
          AND c.tep_name <> ''
          AND c_edge.NAME IS NULL
      SELECT @rc = @@ROWCOUNT, @err = @@ERROR
      IF @err <> 0 
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tep_oledb_bsv: Error %d for delete from #tep_bsv',3,1,@now, @err) WITH NOWAIT
        DEALLOCATE cur_cnps
        RETURN -30000
      END

      SELECT @rc = COUNT(*) 
        FROM #tep_bsv 
        WHERE teps_server = @teps_server AND lev = @lev
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tep_oledb_bsv:   %-8d row(s) inserted into #tep_bsv as lev %d', 0, 1, @now, @rc, @lev) WITH NOWAIT
        IF @verbose >= 2
        BEGIN
          SET ROWCOUNT 50
          SELECT * FROM #tep_bsv WHERE lev = @lev
          SET ROWCOUNT 0
        END
      END

    END      -- loop back if @rc > 0


    SELECT @rc = COUNT(*) 
      FROM #tep_bsv 
      WHERE teps_server = @teps_server
    SET @elapsed_msecs = DATEDIFF(millisecond,@start,CURRENT_TIMESTAMP)
    SET @log_text = @teps_server
    EXEC tep_log @proc_name='tep_oledb_bsv', @log_text=@log_text, 
                              @elapsed_msecs=@elapsed_msecs, @row_count=@rc, @log_detail=@log_detail

    FETCH NEXT FROM cur_cnps INTO @teps_server, @teps_from_clause, @anchor_business, @anchor_physical
  END
  DEALLOCATE cur_cnps

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION tep_oledb_bsv
  ELSE
    SAVE TRANSACTION tep_oledb_bsv


  IF NOT EXISTS(SELECT * 
                  FROM #tep_bsv AS n
                  FULL OUTER JOIN tep_bsv AS o (NOLOCK) ON o.rowno = n.rowno
                  WHERE n.rowno IS NULL OR o.rowno IS NULL
                     OR n.parent_rowno <> o.parent_rowno
                     OR n.name <> o.name
                     OR n.teps_objname <> o.teps_objname
                     OR n.teps_busview <> o.teps_busview
                     OR n.teps_server <> o.teps_server )
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tep_oledb_bsv: tep_bsv does not need updating', 0, 1, @now) WITH NOWAIT
    END
  END
  ELSE
  BEGIN
    TRUNCATE TABLE tep_bsv
    INSERT INTO tep_bsv (parent_rowno, rowno, name, tep_name, teps_objname, teps_busview, namepath, teps_server)
      SELECT parent_rowno, rowno, name, tep_name, teps_objname, teps_busview, namepath, teps_server
        FROM #tep_bsv
    SELECT @rc = @@ROWCOUNT, @err = @@ERROR
    IF @err <> 0 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tep_oledb_bsv: Error %d for inserting into tep_bsv',3,1,@now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION tep_oledb_bsv
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tep_oledb_bsv: %-8d row(s) in tep_bsv', 0, 1, @now, @rc) WITH NOWAIT
    END
    SET @elapsed_msecs = DATEDIFF(millisecond,@start,CURRENT_TIMESTAMP)
    SET @log_text = 'Replaced tep_bsv'
    SET @log_detail = ''
    EXEC tep_log @proc_name='tep_oledb_bsv', @log_text=@log_text, 
                              @elapsed_msecs=@elapsed_msecs, @row_count=@rc, @log_detail=@log_detail
  END

  IF @tcount = 0
    COMMIT TRANSACTION tep_oledb_bsv


  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tep_oledb_bsv: completed', 0, 1, @now) WITH NOWAIT
  END

GO
