-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tracks/oa20786.sql, tbsm_base, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/05/07 17:20:25
--
-- Module Description
--   Defines job Delete old OPC daily plans

USE msdb
GO

DECLARE @job_name sysname
SET @job_name = 'Delete old OPC daily plans'

IF NOT EXISTS (SELECT * FROM sysjobs WHERE name = @job_name)
BEGIN

  -- Standard transaction logic
  DECLARE @tcount INT, @err INT, @rc INT, @now NVARCHAR(40)
  SET @tcount = @@TRANCOUNT

  IF @tcount = 0
    BEGIN TRANSACTION oa20786
  ELSE
    SAVE TRANSACTION oa20786

  -- Add the job
  EXEC Object..asisp_add_job @job_name = @job_name
    , @enabled = 0
    , @description = 'Cleans up OPCLoad database tables based on dates of daily plans'
    , @start_step_id = 1
    , @category_name = 'Database Maintenance'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: oa20786: Could not add job. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    SET @rc = -30000
    ROLLBACK TRANSACTION oa20786
  END

  EXEC sp_add_jobserver @job_name = @job_name
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: oa20786: Could not add jobserver. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    SET @rc = -30000
    ROLLBACK TRANSACTION oa20786
  END

  -- Add the step
  EXEC sp_add_jobstep @job_name = @job_name
    , @step_id = 1
    , @step_name = 'Delete old OPC daily plans'
    , @subsystem = 'TSQL'
    , @command = 'EXEC asisp_deleteoldopcplans @daystokeep = 60'
    , @on_success_action = 1
    , @on_fail_action = 2
    , @on_fail_step_id = 0
    , @database_name = 'OPCLoad'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: oa20786: Could not add jobstep. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    SET @rc = -30000
    ROLLBACK TRANSACTION oa20786
  END

  -- Add the schedule
  EXEC sp_add_jobschedule @job_name = @job_name
    , @name = 'Once per week'
    , @enabled = 1
    , @freq_type = 8 -- weekly
    , @freq_interval = 64 -- saturday
    , @freq_recurrence_factor = 1 -- every week
    , @active_start_time = 030000 -- start at 3AM
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: oa20786: Could not add jobschedule. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    SET @rc = -30000
    ROLLBACK TRANSACTION oa20786
  END

  -- Standard transaction logic
  IF @tcount = 0
    COMMIT TRANSACTION oa20786

END

GO
